/**********************************************************************
** Copyright (C) 2000-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qtopia Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#ifndef DATEENTRY_H
#define DATEENTRY_H

#include "evententrybase.h"
#include "utils/datepicker.h"

#include <qpe/event.h>

#include <qdatetime.h>
#include <qdialog.h>

class EventEntry : public EventEntryBase {
    Q_OBJECT

public:
    EventEntry( bool startOnMonday, const QDateTime &start,
	       const QDateTime &end, bool whichClock = FALSE,
	       QWidget* parent = 0, const char* name = 0 );
    EventEntry( bool startOnMonday, const Event &event, bool whichCLock = FALSE,
	       QWidget* parent = 0, const char* name = 0 );
    ~EventEntry();
    void setEvent(const Event &event);

    Event event();
    void setAlarmEnabled( bool alarmPreset, int presetTime, Event::SoundTypeChoice );
    void setWidth(int);

public slots:
    void slotStart();
    void slotEnd();
    void endDateChanged(QDate);
    void endTimeChanged( const QString & );
    void startDateChanged(QDate);
    void startTimeChanged( int index );
    void typeChanged( const QString & );
    void changeEndCombo( int change );
    void slotRepeat();
    void slotChangeClock( bool );
    void slotChangeStartOfWeek( bool );
    void slotCut();
    void slotCopy();
    void slotPaste();
    void slotFile();

private:
    Event e;
    void init();
    void initCombos();
    void setDates( const QDateTime& s, const QDateTime& e );
    void setRepeatLabel();

    QDate startDate, endDate;
    QTime startTime, endTime;
    Event::RepeatPattern rp;
    bool ampm;
    bool startWeekOnMonday;
};

#endif // DATEENTRY_H
