#include "viewonly.h"
#include "eventview.h"
#include <qmultilineedit.h>
#include <qtextcodec.h>
#include <qdatetime.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qcombobox.h>
#include <qfont.h>
#include <qlayout.h>
#include <qfileinfo.h>
#include <qaction.h>
#include <qlayout.h>
#include <qhbox.h>
#include <qmessagebox.h>
#include <qpe/event.h>
#include <qpe/stringutil.h>
#include <qpe/qpetoolbar.h>
#include <qpe/qpemenubar.h>
#include <qpe/resource.h>
#include "qualendar.h"
#include "qualendarglobal.h"

EventView::EventView(QualendarItem _item, QWidget* parent) 
  : QWidget(parent), item(_item) {

  QVBoxLayout *vbl = new QVBoxLayout(this);
  QHBoxLayout *hbl = new QHBoxLayout(vbl);
  QPEMenuBar *menuBar = new QPEMenuBar(this);
  hbl->addWidget(menuBar);
  QHBox *hbox = new QHBox(this);
  hbl->addWidget(hbox);
  QPopupMenu *editMenu = new QPopupMenu(this);
  menuBar->insertItem(tr("Edit"), editMenu);

  QAction *a;
  QToolButton *b;

  if (_item.type == QualendarItem::Event) {
    a = new QAction(tr("Beam"), Resource::loadPixmap("qualendar/beam"),
		    QString::null, 0, this, 0);
    a->addTo(editMenu);
    connect(a, SIGNAL(activated()), this, SLOT(beamClicked()));
    b = new QToolButton(hbox);
    b->setPixmap(Resource::loadPixmap("qualendar/beam"));
    connect(b, SIGNAL(clicked()), this, SLOT(beamClicked()));
  }

  a = new QAction(tr("Copy"), Resource::loadPixmap("qualendar/copy"),
		  QString::null, 0, this, 0);
  a->addTo(editMenu);
  connect(a, SIGNAL(activated()), this, SLOT(copyClicked()));
  b = new QToolButton(hbox);
  b->setPixmap(Resource::loadPixmap("qualendar/copy"));
  connect(b, SIGNAL(clicked()), this, SLOT(copyClicked()));

#ifndef VIEWONLY
  if (_item.type == QualendarItem::Event) {
    a = new QAction(tr("Clone"), Resource::loadPixmap("qualendar/clone"),
		    QString::null, 0, this, 0);
    a->addTo(editMenu);
    connect(a, SIGNAL(activated()), this, SLOT(cloneClicked()));
    b = new QToolButton(hbox);
    b->setPixmap(Resource::loadPixmap("qualendar/clone"));
    connect(b, SIGNAL(clicked()), this, SLOT(cloneClicked()));

    a = new QAction(tr("Exception"), Resource::loadPixmap("qualendar/exception"),
		    QString::null, 0, this, 0);
    a->addTo(editMenu);
    connect(a, SIGNAL(activated()), this, SLOT(exceptionClicked()));
    b = new QToolButton(hbox);
    b->setPixmap(Resource::loadPixmap("qualendar/exception"));
    connect(b, SIGNAL(clicked()), this, SLOT(exceptionClicked()));
  }

  a = new QAction(tr("Edit"), Resource::loadPixmap("qualendar/edit"),
		  QString::null, 0, this, 0);
  a->addTo(editMenu);
  connect(a, SIGNAL(activated()), this, SLOT(editClicked()));
  b = new QToolButton(hbox);
  b->setPixmap(Resource::loadPixmap("qualendar/edit"));
  connect(b, SIGNAL(clicked()), this, SLOT(editClicked()));

  a = new QAction(tr("Delete"), Resource::loadPixmap("qualendar/trash"),
		  QString::null, 0, this, 0);
  a->addTo(editMenu);
  connect(a, SIGNAL(activated()), this, SLOT(deleteClicked()));
  b = new QToolButton(hbox);
  b->setPixmap(Resource::loadPixmap("qualendar/trash"));
  connect(b, SIGNAL(clicked()), this, SLOT(deleteClicked()));
#endif

  browser = new MyTextBrowser(this, "");
  vbl->addWidget(browser);
  connect(browser, SIGNAL(linkSelected(QString)), 
	  this, SLOT(linkSelected(QString)));
  connect(browser, SIGNAL(keyPressed(QKeyEvent*)), 
	  this, SLOT(keyPressed(QKeyEvent*)));

  switch (item.type) {
  case QualendarItem::Invalid:
    return;
  case QualendarItem::Task:
    setTask(*item.task);
    break;
  case QualendarItem::Event:
    setEvent(*item.event);
  }
}

void EventView::setEvent(const EffectiveEvent &eev) {
  setCaption(tr("Event view"));

  const Event e = eev.event();
  QString s;
  QDate sd = e.start().date(), ed = e.end().date();

  pathList.clear();
  s.append("<qt><table>");
  s.append("<tr><td><font color=#008000>");
  s.append(tr("Start"));
  s.append("</font></td><td>");
  {
    s.append(dayString(eev.startDate()));
    if (e.type() == Event::AllDay) {
      s.append("("+tr("All day")+")");
    } else {
      QTime t = e.start().time();
      s.append(QString("%1:%2").arg(t.hour()).arg(t.minute(), 2)
	       .replace(QRegExp(" "), "0"));
    }
  }
  s.append("</td></tr>");

  s.append("<tr><td><font color=#008000>");
  s.append(tr("End"));
  s.append("</font></td><td>");
  {
    s.append(dayString(eev.endDate()));
    if (e.type() == Event::AllDay) {
    } else {
      QTime t = e.end().time();
      s.append(QString("%1:%2").arg(t.hour()).arg(t.minute(), 2)
	       .replace(QRegExp(" "), "0"));
    }
  }
  s.append("</td></tr>");

  {
    Event::RepeatPattern rp = e.repeatPattern();
    if (rp.type != Event::NoRepeat) {
      s.append("<tr><td><font color=#008000>");
      s.append(tr("repeat"));
      s.append("</font></td><td>");
      switch (rp.type) {
      case Event::Daily:
	s.append(tr("Every day"));
	break;
      case Event::Weekly:
	s.append(tr("Every week"));
	for(int i=0, d = 0x01; i<7; i++, d = d<<1)
	  if (rp.days & d) {
	    s.append(" ");
	    s.append(QObject::tr(sd.dayName(i+1)));
	  }
	s.append(" "+tr("(week day)"));
	break;
      case Event::MonthlyDay:
	s.append(tr("Every month"));
	s.append(tr("not implemented yet"));
	break;
      case Event::MonthlyDate:
	s.append(tr("Every month"));
	s.append(QString::number(sd.day()));
	if (sd.day() != ed.day()) {
	  s.append("-");
	  s.append(QString::number(ed.day()));
	}
	break;
      case Event::Yearly:
	s.append(tr("Every year"));
	s.append(tr("%1.%2").arg(sd.month()).arg(sd.day()));
	if (sd.month() != ed.month() || sd.day() != ed.day()) {
	  s.append("-");
	  s.append(tr("%1.%2").arg(ed.month()).arg(ed.day()));
	}
	break;
      default:
	break;
      }
      s.append("</td></tr>");
    }
  }

  s.append("<tr><td><font color=#008000>");
  s.append(tr("Description:")+"     ");
  s.append("</font></td><td>");
  s.append(Qtopia::escapeString(e.description()));
  s.append("</td></tr>");
  s.append("<tr><td><font color=#008000>");
  s.append(tr("location"));
  s.append("</font></td><td>");
  s.append(Qtopia::escapeString(e.location()));
  s.append("</td></tr></table>");

  if (e.notes().length() > 0) {
    s.append("<hr>");
    QString t = Qtopia::escapeString(e.notes());
    t.replace(QRegExp("\n"), "<br>");
    uint i;
    for (i=0; i < t.length(); ) {
      int j = t.find("\[\[", i);
      if (j < 0) { 
	s.append(t.mid(i, t.length() - i));
	break;
      }
      s.append(t.mid(i, j-i));
      int k = t.find("]]", j);
      if (k < 0) {
	s.append(t.mid(j, t.length() - j));
	break;
      }
      QString u = t.mid(j+2, k-j-2);
      s.append("<a href=\"" + u + "\">" + 
	       ((pathList.count() < 10) ?
		("(" + QString::number(pathList.count()) + ") ")
		: QString(""))
	       + QFileInfo(u).fileName() + "</a>");
      if (pathList.count() < 10)
	pathList.append(u);
      i = k + 2;
    }
  }
  s.append("</qt>");
  browser->setText(s);
}

void EventView::setTask(const QualendarTask &e) {
  setCaption(tr("Task view"));

  pathList.clear();

  QString s;

  s.append("<qt><table>");

  s.append("<tr><td><font color=#008000>");
  s.append(tr("Description:")+"     ");
  s.append("</font></td><td>");
  s.append(Qtopia::escapeString(e.description()));
  s.append("</td></tr>");

  s.append("<tr><td><font color=#008000>");
  s.append(tr("Priority")+"     ");
  s.append("</font></td><td>");
  s.append(QString::number(e.priority()));
  s.append("</td></tr>");

  s.append("<tr><td><font color=#008000>");
  s.append(tr("Start date"));
  s.append("</font></td><td>");
  if (e.hasStartDate()) 
    s.append(dayString(e.startDate()));
  else
    s.append(tr("void"));
  s.append("</td></tr>");

  s.append("<tr><td><font color=#008000>");
  s.append(tr("Due"));
  s.append("</font></td><td>");
  if (e.hasDueDate()) 
    s.append(dayString(e.dueDate()));
  else
    s.append(tr("void"));
  s.append("</td></tr>");

  s.append("<tr><td><font color=#008000>");
  s.append(tr("Finish"));
  s.append("</font></td><td>");
  if (e.isCompleted())
    s.append(dayString(e.finishDate()));
  else
    s.append(tr("void"));
  s.append("</td></tr>");

  s.append("</table>");

  if (e.notes().length() > 0) {
    s.append("<hr>");
    QString t = Qtopia::escapeString(e.notes());
    t.replace(QRegExp("\n"), "<br>");
    uint i;
    for (i=0; i < t.length(); ) {
      int j = t.find("\[\[", i);
      if (j < 0) { 
	s.append(t.mid(i, t.length() - i));
	break;
      }
      s.append(t.mid(i, j-i));
      int k = t.find("]]", j);
      if (k < 0) {
	s.append(t.mid(j, t.length() - j));
	break;
      }
      QString u = t.mid(j+2, k-j-2);
      s.append("<a href=\"" + u + "\">" + 
	       ((pathList.count() < 10) ?
		("(" + QString::number(pathList.count()) + ") ")
		: QString(""))
	       + QFileInfo(u).fileName() + "</a>");
      if (pathList.count() < 10)
	pathList.append(u);
      i = k + 2;
    }
  }
  s.append("</qt>");

  browser->setText(s);
}

void EventView::beamClicked() {
  emit beamItem(item);
  emit done();
}
void EventView::cloneClicked() {
#ifndef VIEWONLY
  emit cloneItem(item);
  emit done();
#endif
}
void EventView::copyClicked() {
  browser->copy();
}
void EventView::editClicked() {
#ifndef VIEWONLY
  emit editItem(item);
  emit done();
#endif
}
void EventView::deleteClicked() {
#ifndef VIEWONLY
  emit removeItem(item);
  emit done();
#endif
}

void EventView::exceptionClicked() {
#ifndef VIEWONLY
  Event e = (*item.event).event();
  if (e.repeatPattern().type == Event::NoRepeat) {
    QMessageBox::information(this, tr("Exception"), 
			     tr("This is not a repeating event."),
			     QMessageBox::Ok);
  } else {
    if (QMessageBox::warning(this, tr("Exception"), 
			 tr("This operation may break\n" 
			    "the repeating event into two or three\n"
			    "(before, this, after)."), 
			     QMessageBox::Ok, QMessageBox::Cancel)
	== QMessageBox::Ok) {
      emit exceptEvent(*item.event);
      emit done();
    }
  }
#endif
}

void EventView::keyPressEvent(QKeyEvent *e) {
  switch(e->key()) {
  case Key_Delete:
    deleteClicked();
    break;
  case Key_C:
    cloneClicked();
    break;
  case 0x201a: // Fn + m
    editClicked();
    break;
  default:
    QWidget::keyPressEvent(e);
  }
}

void EventView::linkSelected(QString s) {
  QualendarGlobal::openDocument(s);
}
void EventView::keyPressed(QKeyEvent *e) {
  int k = e->ascii();
  if (k >= '0' && k <= '9') {
    int i = k - '0';
    if (i < pathList.count()) {
      linkSelected(pathList[i]);
    }
  }
}

QString EventView::dayString(QDate d) {
  return 
    tr("%1.%2.%3(%4)")
    .arg(QString::number(d.year()))
    .arg(QString::number(d.month()))
    .arg(QString::number(d.day()))
    .arg(QObject::tr(d.dayName(d.dayOfWeek())));
}
