/*
 * Copyright (C) 2003 ichitokumei
 * 
 * Derived from qtopia-free-1.5.0/datebook/datebook.cpp
 * 
 * Implementation of controls in Qualendar.
 */

/**********************************************************************
** Copyright (C) 2000-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qtopia Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include "viewonly.h"

#include "qualendar.h"
#include "qualendarview.h"

#include "utils/fileselector.h"

#include "qualendarglobal.h"
#include "templatedialog.h"
#include "sysdep/qualendardb.h"
#include "sysdep/qualendarcategory.h"
#include "sysdep/qualendarsysdep.h"
#include "version.h"
#include "debug.h"

#include <qlayout.h>
#include <qmultilineedit.h>
#include <qpe/qpeapplication.h>
#include <qpe/qcopenvelope_qws.h>
#include <qpe/config.h>
#include <qpe/qpedebug.h>
#include <qpe/event.h>
#include <qpe/qpemenubar.h>
#include <qpe/resource.h>
#include <qpe/qpetoolbar.h>
#include <qpe/timestring.h>

#include <qaction.h>
#include <qdatetime.h>
#include <qdialog.h>
#include <qfileinfo.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <qpushbutton.h>

#include <stdlib.h>

void Qualendar::writeLog(QString s) {
  /*
  QFile f("/home/tmp/qualendarlog");
  f.open(IO_WriteOnly | IO_Append);
  QTextStream ts(&f);
  ts << s << "\n";
  f.close();
  */
}

Qualendar::Qualendar(QWidget *parent, const char *, WFlags f)
  : QMainWindow(parent, "qualendar", f), 
    syncing(false), syncingTasks(false), inSearch(false) {

  QualendarGlobal::load();
  db = QualendarSysdep::eventDB();
  connect(db, SIGNAL(exception(QString, QString)), 
	  this, SLOT(report(QString, QString)));
  taskDB = QualendarSysdep::taskDB();
  connect(taskDB, SIGNAL(exception(QString, QString)), 
	  this, SLOT(report(QString, QString)));

  if (QualendarGlobal::fontSize >= 0)
    setFont(QFont(font().family(), QualendarGlobal::fontSize));

  setCaption(
#ifdef VIEWONLY
	     "QualendarView: "
#else
	     "Qualendar: "
#endif
	     + QualendarGlobal::filterName[0]);
  setIcon(Resource::loadPixmap("datebook_icon"));

  makeSystemConnections();

  initWidgets();
}

////////
// initialize widgets.
////////
#define LOAD_ICON(s) icon.convertFromImage(Resource::loadPixmap(s).convertToImage().smoothScale(is, is))
void Qualendar::initWidgets() {
  setToolBarsMovable(false);

  QWidget *d = QApplication::desktop();
  int is = (QualendarGlobal::iconSize > 0) ? 
    QualendarGlobal::iconSize : QMAX(d->width(), d->height())/20-6;

  QPEToolBar *bar = new QPEToolBar(this);
  bar->setHorizontalStretchable(true);

  QPEMenuBar *mb = new QPEMenuBar(bar);
  mb->setMargin(0);

  QPEToolBar *sub_bar = new QPEToolBar(this);

  QPopupMenu *view = new QPopupMenu(this);
  QPopupMenu *special = new QPopupMenu(this);
  templ = new QPopupMenu(this);

  mb->insertItem(tr("View"), view);
  mb->insertItem(tr("Special"), special);
  mb->insertItem(tr("Templ"), templ);

  QAction *a;
  QPixmap icon;

  a = new QAction(tr("Full screen"), QString::null, 0, this);
  connect(a, SIGNAL(activated()), this, SLOT(showFullScreen()));
  a->addTo(view);
  a = new QAction(tr("Normal screen"), QString::null, 0, this);
  connect(a, SIGNAL(activated()), this, SLOT(showNormal()));
  a->addTo(view);

#ifndef VIEWONLY
  LOAD_ICON("qualendar/new");
  a = new QAction(tr("New"), icon, 
		  QString::null, 0, this, 0);
  connect(a, SIGNAL(activated()), this, SLOT(createEventOrTask()));
  a->addTo(sub_bar);
#endif

  LOAD_ICON("qualendar/reload");
  a = new QAction(tr("Reload"), icon, 
		  QString::null, 0, this, 0);
  connect(a, SIGNAL(activated()), this, SLOT(reloadEventsPrivate()));
  a->addTo(special);
  LOAD_ICON("qualendar/reload");
  a = new QAction(tr("Reload ToDo"), icon, 
		  QString::null, 0, this, 0);
  connect(a, SIGNAL(activated()), this, SLOT(reloadTasksPrivate()));
  a->addTo(special);

  LOAD_ICON("qualendar/today");
  a = new QAction(tr("Today"), icon, 
		  QString::null, 0, this, 0);
  connect(a, SIGNAL(activated()), this, SLOT(slotToday()));
  a->addTo(sub_bar);
  a->addTo(view);

  /*
  LOAD_ICON("qualendar/find");
  a = new QAction(tr("Find"), icon, 
		  QString::null, 0, this, 0);
  connect(a, SIGNAL(activated()), this, SLOT(slotFind()));
  a->addTo(sub_bar);
  */

  popupFilter = new QPopupMenu(view);
  view->insertItem(tr("Filter"), popupFilter);
  connect(popupFilter, SIGNAL(activated(int)),
	  this, SLOT(changeFilterFromMenu(int)));

  QActionGroup *ag = new QActionGroup(this);
  ag->setExclusive(true);
  for(uint i=0; i<10; i++) {
    filterAction[i] = new QAction(QualendarGlobal::filterName[i], 
				  QString::null, 0, ag, 0, true);
    filterAction[i]->addTo(popupFilter);
  }
  filterAction[0]->setOn(true);

  QPopupMenu *popupViewOption;
  popupViewOption = new QPopupMenu(view);
  view->insertItem(tr("View option"), popupViewOption);

  a = new QAction(tr("Settings"), QString::null, 0, this);
  connect(a, SIGNAL(activated()), this, SLOT(slotSettings()));
  a->addTo(special);

  a = new QAction(tr("Reload settings"), QString::null, 0, this);
  connect(a, SIGNAL(activated()), this, SLOT(reloadSettings()));
  a->addTo(special);

  a = new QAction(tr("File"), QString::null, 0, this);
  connect(a, SIGNAL(activated()), this, SLOT(slotFile()));
  a->addTo(special);

#ifndef VIEWONLY
  a = new QAction(tr("Remove events of this month"), QString::null, 0, this);
  connect(a, SIGNAL(activated()), this, SLOT(removeMonth()));
  a->addTo(special);
#endif

  /*
    a = new QAction(tr("Export to VCalendar"), QString::null, 0, this);
    connect(a, SIGNAL(activated()), this, SLOT(exportToVCalendar()));
    a->addTo(special);
  */

  a = new QAction(tr("About"), QString::null, 0, this);
  connect(a, SIGNAL(activated()), this, SLOT(slotAbout()));
  a->addTo(special);

  a = new QAction(tr("Edit"), QString::null, 0, this);
  connect(a, SIGNAL(activated()), this, SLOT(slotEditTemplate()));
  a->addTo(templ);
  for(uint i=0; i<10; i++) {
    templateAction[i] = new QAction("", QString::null, 0, this);
    templateAction[i]->addTo(templ);
  }
  setupTemplate(true);
  connect(templ, SIGNAL(activated(int)), this, SLOT(slotTemplate(int)));

  QArray<QualendarDB*> dataBases;
  dataBases.resize(2);
  dataBases[0] = db;
  dataBases[1] = taskDB;

  views = new QualendarViewStack(this, "", dataBases, popupViewOption);
  setCentralWidget(views);
  views->setFilter(0, false);

#ifndef VIEWONLY
  connect(views, SIGNAL(editItem(QualendarItem)), 
	  this, SLOT(editItem(QualendarItem))); 
  connect(views, SIGNAL(removeItem(QualendarItem)), 
	  this, SLOT(removeItem(QualendarItem))); 
  connect(views, SIGNAL(createEvent(QDateTime, QDateTime, QString)),
	  this, SLOT(createEvent(QDateTime, QDateTime, QString)));
#endif
  connect(views, SIGNAL(viewItem(QualendarItem)), 
	  this, SLOT(viewItem(QualendarItem))); 
  connect(views, SIGNAL(viewChanged(int)),
	  this, SLOT(slotViewChanged(int)));


  //  connect(this, SIGNAL(newEvent()),
  //	  this, SLOT(redraw()));

  // ISSUES: should be handled by each view?
  connect(db, SIGNAL(dataChanged()), this, SLOT(redraw()));
  connect(taskDB, SIGNAL(dataChanged()), this, SLOT(redraw()));

  // Create buttons corresponding to views.
  // FIXME: it is possible to make QualendarViewStack::icons() static.
  QActionGroup *g = new QActionGroup(this);
  g->setExclusive(true);
  connect(g, SIGNAL(selected(QAction *)), 
	  this, SLOT(viewActionSelected(QAction *)));

  QValueList<QPixmap> iconList = views->icons();
  QStringList viewNameList = views->trViewNames();
  viewActions.resize(viewNameList.count());
  for(uint i=0; i<viewNameList.count(); i++) {
    QPixmap pm;
    if (i < iconList.count())
      pm.convertFromImage(iconList[i].convertToImage().smoothScale(is+3, is+3));
    a = new QAction(viewNameList[i], pm, QString::null, 0, g, 0);
    if (i < iconList.count())
      a->addTo(sub_bar);
    a->addTo(view);
    a->setToggleAction(true);
    if (i == 0)
      a->setOn(true);
    viewActions.insert(i, a);
  }

#ifndef VIEWONLY
  entry = 0;
  editDlg = 0;
#endif
}

void Qualendar::reload() {
  reloadEvents();
  //  reloadTasks();
}

void Qualendar::reloadEvents() {
  writeLog("reload");
  db->reload();
  syncing = false;
}

void Qualendar::reloadTasks() {
  writeLog("reload todo");
  taskDB->reload();
  syncingTasks = false;
}

void Qualendar::closeEvent(QCloseEvent *e) {
  INPUT_DEBUG(qDebug("Qualendar::closeEvent"));

  if(syncing || syncingTasks) {
    /* no need to save, did that at flush */
    e->accept();
    return;
  }

  // save settings will generate it's own error messages, no
  // need to do checking ourselves.

#ifndef VIEWONLY
  if (db->save())
#endif
    e->accept();
#ifndef VIEWONLY
  else {
    if (QMessageBox::critical(this, tr("Out of space"),
			      tr("Qualendar was unable to save\n"
				 "your changes.\n"
				 "Free up some space and try again.\n"
				 "\nQuit anyway?"),
			      QMessageBox::Yes|QMessageBox::Escape,
			      QMessageBox::No|QMessageBox::Default)
	!= QMessageBox::No)
      e->accept();
    else
      e->ignore();
  }
#endif // VIEWONLY
}

void Qualendar::keyPressEvent(QKeyEvent *e) {
  qDebug("Qualendar::keyPressEvent %d %c", e->key(), e->ascii());
  switch(e->key()) {
    case 0x2014: // Fn + q
  case Key_Escape:
    close();
    break;
#ifndef VIEWONLY
  case 0x2019: // Fn + n
    qDebug("Fn-n in Qualendar");
    createEventOrTask();
    break;
#endif
  case Key_Y:
    views->gotoView("Year");
    break;
  case Key_M:
    views->gotoView("Month");
    break;
  case Key_W:
    views->gotoView("Week");
    break;
  case Key_D:
    views->gotoView("Day");
    break;
  case Key_F:
    views->gotoView("Find");
    break;
  case Key_T:
    views->gotoView("Todo");
    break;
  case Key_Period:
    slotToday();
    break;
  default:
    int i = e->ascii() - '0';
    if (i >= 0 && i <= 9) {
      filterAction[i]->setOn(true);
      changeFilter(i);
    } else
      QMainWindow::keyPressEvent(e);
  }
}

////////
void Qualendar::report(QString title, QString mes) {
  QString s;
  QDialog dlg(this, 0, true);
  dlg.setCaption(title);
  QMultiLineEdit *mle = new QMultiLineEdit(&dlg);
  mle->setReadOnly(true);
  QVBoxLayout *l = new QVBoxLayout(&dlg);
  l->addWidget(mle);

  s.append(mes + "\n----------------\n");
  s.append(tr("Name of binary: %1").arg(QualendarGlobal::binName()) + "\n");
  s.append(tr("Version: %1").arg(VERSION) + "\n");
  s.append(tr("Device: %1").arg(QualendarSysdep::deviceName()) + "\n");
  s.append(tr("Type: %1").arg(QualendarSysdep::type()) + "\n");

  mle->setText(s);
  dlg.showMaximized();
  dlg.exec();
}
////////

////////
void Qualendar::slotAbout() {
  report("About Qualendar", "This is Qualendar.");
}

////////
// From menus and buttons... 
////////
void Qualendar::viewActionSelected(QAction *a) {
  for(uint i=0; i<viewActions.count(); i++) 
    if (viewActions[i] == a)
      views->gotoView(i);
}

void Qualendar::slotToday() {
  QDate dt = QDate::currentDate();
  views->setDate(dt); 
}

////////
// Reload the data if not syncing
////////
void Qualendar::reloadEventsPrivate()
{
  if (!syncing)
    reloadEvents();
}
void Qualendar::reloadTasksPrivate()
{
  if (!syncingTasks)
    reloadTasks();
}

////////
// Reload settings.
// Call this only after everything(widgets etc.) are ready.
////////
void Qualendar::reloadSettings()
{
  QualendarGlobal::load();
  if (QualendarGlobal::fontSize >= 0)
    setFont(QFont(font().family(), QualendarGlobal::fontSize));
  for(uint i=0; i<4; i++) 
    filterAction[i]->setText(QualendarGlobal::filterName[i]);
  views->loadConfig();
  views->redraw(); 
}

////////
// Redraw... But this might be unnecessary.
////////
void Qualendar::redraw() {
  views->redraw(); 
}

////////
// Set the button corresponding to the view.
// Invoked by signals from the view stack.
////////
void Qualendar::slotViewChanged(int i) {
  viewActions[i]->setOn(true);
}

////////
// Edit event template.
////////
void Qualendar::slotEditTemplate()
{
#ifndef VIEWONLY
  QDate d = views->currentDate();
  TemplateDialog dlg(d, eventTemplate);
  dlg.showMaximized();
  dlg.exec();
  setupTemplate();
#endif  
}
////////
// Edit event template.
////////
void Qualendar::slotTemplate(int id) {
  int i = templ->indexOf(id);
  i--; // "Edit". FIXME
  if (i < 0)
    return;
  Event e = eventTemplate[i];

  QDateTime dt0 = e.start(), dt1 = e.end();
  e.setStart(QDateTime(views->currentDate(), dt0.time()));
  e.setEnd(e.start().addSecs(dt0.secsTo(dt1)));

  db->setRid(e, e, true);
  db->setRinfo(e, 1);
  e.assignUid();
  db->add(e);
  //  emit newEvent();
}

////////
// Launch the setting application,
// which, when done, emits 'reloadSetting' message.
////////
void Qualendar::slotSettings()
{
  QCopEnvelope e("QPE/System", "execute(QString,QString)");
  e << QString("qualendarutil") << QString(QualendarGlobal::binName());
}

////////
// Open a file selector.
////////
void Qualendar::slotFile() {
  QString dir;
  {
    Config config("Qualendar2Tmp");
    config.setGroup("Main");
    dir = config.readEntry("CurrentDir", 
                           QPEApplication::documentDir().latin1());
  }
  FileSelector dlg(views->currentDate(), dir, this, "file", true);
  dlg.showMaximized();
  if (dlg.exec() == QDialog::Accepted) {
    QualendarGlobal::openDocument("file://" + dlg.absName());

  }
  Config config("Qualendar2Tmp");
  config.setGroup("Main");
  config.writeEntry("CurrentDir", dlg.currentDir());
}

void Qualendar::changeFilter(int i) {
  views->setFilter(i);
  setCaption(
#ifdef VIEWONLY
	     "QualendarView: "
#else
	     "Qualendar: "
#endif
	     + QualendarGlobal::filterName[i]);
}

void Qualendar::changeFilterFromMenu(int i) {
  for(int j=0; j<10; j++) 
    if (popupFilter->idAt(j) == i)
      changeFilter(j);
}

void Qualendar::setupTemplate(bool init) {
  if (init) {
    Config config("QualendarTemplate");
    config.setGroup("Main");
    for(uint i=0; i<10; i++) {
      eventTemplate[i].setDescription(config.readEntry(QString("Description%1").arg(i)));
      eventTemplate[i].setLocation(config.readEntry(QString("Location%1").arg(i)));
      eventTemplate[i].setCategories(QualendarGlobal::categories.id("Calendar", config.readEntry(QString("Category%1").arg(i))));
      eventTemplate[i].setStart(TimeConversion::fromISO8601(config.readEntry(QString("Start%1").arg(i)).utf8()));
      eventTemplate[i].setEnd(TimeConversion::fromISO8601(config.readEntry(QString("End%1").arg(i)).utf8()));
      eventTemplate[i].setType(config.readBoolEntry(QString("AllDay%1").arg(i)) ? Event::AllDay : Event::Normal);
      eventTemplate[i].setAlarm(config.readBoolEntry(QString("HasAlarm%1").arg(i)),
				config.readNumEntry(QString("AlarmTime%1").arg(i)),						     
				config.readEntry(QString("AlarmSound%1").arg(i)) == "Loud" ? Event::Loud : Event::Silent);
      //      eventTemplate[i].setNotes(config.readEntry(QString("Notes%1").arg(i)));
    }
  }

  for(uint i=0; i<10; i++) 
    templateAction[i]->setMenuText(TimeString::timeString(eventTemplate[i].start().time()) + " " + eventTemplate[i].description().left(5));
}

