/*
 * Copyright (C) 2003 ichitokumei
 * 
 * Derived from qtopia-free-1.5.0/datebook/datebook.h
 * 
 * The header file for the main window. 
 * Implementation is in qualendar.cpp, qualendar_eventslots.cpp, 
 * qualendar_find.cpp and qualendar_messages.cpp
 */

/**********************************************************************
** Copyright (C) 2000-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qtopia Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#ifndef QUALENDAR_H
#define QUALENDAR_H

#include <qmainwindow.h>
#include <qvector.h>
#include <qaction.h>

#include <qfile.h>
#include <qtextstream.h>

#include "sysdep/qualendarsysdep.h"
#ifndef VIEWONLY
#include "evententry.h"
#endif

class QWidgetStack;
class QualendarViewStack;
class Event;
class QDate;
class QDateTime;
class Ir;
class QScrollView;

class QualendarItem;

/**
 * The main window.
 **/
class Qualendar : public QMainWindow
{
  Q_OBJECT

 public:
  Qualendar(QWidget *parent = 0, const char *name = 0, WFlags f = 0);

 private:
  void makeSystemConnections();
  void initWidgets();

 public slots:
  /** 
   * Reload the data. 
   * Invoked from the menu, buttons and QCop messages(after sync).
   **/
  void reload();
  /** 
   * Save the data. 
   * Invoked from QCop messages before sync.
   **/
  void flush();
  void flushTasks();

 signals:
  /**
   * Emitted when an event is created, modified or removed. 
   * (2003.12.23) replaced by signal QualendarDB::dataChanged().
   **/ 
  //  void newEvent();

 protected:
  /**
   * Handles escape key etc. Save if necessary.
   **/
  void closeEvent(QCloseEvent *e);
  void keyPressEvent(QKeyEvent *e);
  /**
   * For alarm. Reimplemented.
   **/
  void timerEvent(QTimerEvent *e);


private slots:
  void reloadEvents();
  void reloadTasks();
  void reloadEventsPrivate();
  void reloadTasksPrivate();

  void report(QString, QString);

  void slotAbout();
  void viewActionSelected(QAction *);
  void slotToday(); 
  void slotEditTemplate();
  void slotTemplate(int);

  void reloadSettings();
  void redraw();

  // For commnunication with the view stack.
  void slotViewChanged(int);

  // Invoked from the menu and tool buttons, etc.
  void slotSettings();
  void slotFile();
  void changeFilter(int);
  void changeFilterFromMenu(int);

  // Manipulates events.
  // Implemented in qualendar_eventslots.cpp
  void createEventOrTask();
  void createEvent();
  void createEvent(QDateTime, QDateTime, QString);
  void createTask();
  void editItem(QualendarItem);
  void cloneItem(QualendarItem);
  void exceptEvent(const EffectiveEvent &);
  void removeItem(QualendarItem);
  void beamItem(QualendarItem);
  void beamDone(Ir *ir);
  void viewItem(QualendarItem);
  void removeMonth();
  // void exportToVCalendar();

  // Handle system messages
  void changeClock(bool newClock);
  void changeWeek(bool newDay);
  void appMessage(const QCString& msg, const QByteArray& data);
  void receive(const QCString &msg, const QByteArray &data);
  void receiveSync(const QCString &msg, const QByteArray &data);
  void setDocument(const QString &);

  void writeLog(QString);

private:
  // Implemented in qualendar_eventslots.cpp
  void editEventCommon(const Event &, QDate, bool isNew);
  QString checkEvent(const Event &);

  void setupTemplate(bool init = false);

  QualendarEventDB *db;
  QualendarTaskDB *taskDB;
  QualendarViewStack *views;
  QVector<QAction> viewActions;
  QPopupMenu *popupFilter, *templ;
  QAction *filterAction[4];
  QAction *templateAction[10];
  Event eventTemplate[10];
#ifndef VIEWONLY
  EventEntry *entry;
  QDialog *editDlg;
  QScrollView *sv;
#endif

  bool syncing, syncingTasks;
  bool inSearch;
};

#endif // QUALENDAR_H
