/*
 * Copyright (C) 2003 ichitokumei
 * 
 * Derived from qtopia-free-1.5.0/datebook/datebook.cpp
 * 
 * Implementation of event manipulation in Qualendar.
 */

/**********************************************************************
** Copyright (C) 2000-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qtopia Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/


#include <time.h>

#include <qpe/ir.h>
#include <qpe/qpemessagebox.h>
#include <qpe/qpeapplication.h>
#include <qpe/tzselect.h>
#include "sysdep/qualendardb.h"

#include <qmessagebox.h>
#include <qdialog.h>
#include <qlayout.h>

#include "qualendar.h"
#include "qualendarglobal.h"
#include "eventview.h"
#include "qualendarview.h"
#include "evententry.h"
#include "qualendaritem.h"
#include "tododialog.h"

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>

void Qualendar::viewItem(QualendarItem item) {
  if (item.type == QualendarItem::Invalid)
    return;

  if (syncing || syncingTasks) {
    QMessageBox::warning( this, "Qualendar",
			  tr( "Can not view data, currently syncing") );
    return;
  }

  // workaround added for text input.
  QDialog viewDlg(this, 0, TRUE);
  if (QualendarGlobal::eventFontSize > 0)
    viewDlg.setFont(QFont(font().family(), QualendarGlobal::eventFontSize));
  QVBoxLayout vdl(&viewDlg);

  EventView *eventView = new EventView(item, &viewDlg);
  vdl.addWidget(eventView);
  connect(eventView, SIGNAL(beamItem(QualendarItem)), 
	  this, SLOT(beamItem(QualendarItem)));
#ifndef VIEWONLY
  connect(eventView, SIGNAL(editItem(QualendarItem)),
	  this, SLOT(editItem(QualendarItem)));
  connect(eventView, SIGNAL(removeItem(QualendarItem)),
	  this, SLOT(removeItem(QualendarItem)));
  connect(eventView, SIGNAL(cloneItem(QualendarItem)), 
	  this, SLOT(cloneItem(QualendarItem)));
  connect(eventView, SIGNAL(exceptEvent(const EffectiveEvent &)), 
	  this, SLOT(exceptEvent(const EffectiveEvent &)));
#endif
  connect(eventView, SIGNAL(done()), &viewDlg, SLOT(reject()));
  viewDlg.setCaption(tr("Event view"));
  viewDlg.showMaximized();
  viewDlg.exec();
  return;
}

////////
// Edit an event(called from createEvent and editItem)
////////
#ifndef VIEWONLY
void Qualendar::editEventCommon(const Event &e0, QDate d, bool isNew) {
  Event e = e0; // work around for bug?
  // workaround added for text input.
  if (!editDlg) {
    editDlg = new QDialog( this, 0, TRUE );

    QVBoxLayout *vb = new QVBoxLayout( editDlg );
    sv = new QScrollView( editDlg, "scrollview" );
    sv->setHScrollBarMode(QScrollView::AlwaysOff);
    sv->setVScrollBarMode(QScrollView::AlwaysOn);
    sv->setResizePolicy(QScrollView::AutoOneFit);
    vb->addWidget( sv );
    entry = new EventEntry(QualendarGlobal::onMonday, e,
			   QualendarGlobal::ampm, sv->viewport(), "editor" );
    sv->addChild( entry );
  } else
    entry->setEvent(e);
  if (isNew) {
    entry->timezone->setEnabled( true );
    entry->setAlarmEnabled( QualendarGlobal::aPreset, QualendarGlobal::presetTime, Event::Loud );
  } else {
    entry->timezone->setEnabled( FALSE );
  }

  if (isNew)
    editDlg->setCaption( tr("New Event") );
  else
    editDlg->setCaption( tr("Edit Event") );
#if defined(Q_WS_QWS) || defined(_WS_QWS_)
  editDlg->showMaximized();
  entry->setMaximumWidth(sv->visibleWidth());
#endif
  while (editDlg->exec() ) {
    Event newEv = entry->event();
    if (isNew) {
      newEv.assignUid();
    } else {
      newEv.setUid(e.uid()); // from opie-datebook: FIXME: Hack not to clear uid
    }
    QString error = checkEvent(newEv);
    if (!error.isNull()) {
      //	    if (QMessageBox::warning(this, "error box",
      //			error, "Fix it", "Continue", 0, 0, 1) == 0)
      QMessageBox::warning(this, "error box",
			   error, tr("Fix it"), 0, 0);
      continue;
    }

    db->setRid(newEv, e, isNew);
    if (isNew)
      db->add(newEv);
    else {
      db->edit(e, newEv);
    }
    qDebug("New event: day = %d", newEv.start().date().day());
    //    emit newEvent();
    break;
  }
#endif // VIEWONLY
}

#ifdef VIEWONLY
void Qualendar::editItem(QualendarItem) {
#else
void Qualendar::editItem(QualendarItem item) {
  if (item.type == QualendarItem::Invalid)
    return;

  if (syncing || syncingTasks) {
    QMessageBox::warning(this, "Qualendar",
			 tr("Can not edit data, currently syncing"));
    return;
  }

  if (item.type == QualendarItem::Task) {
    const QualendarTask &t = *item.task;
    TodoDialog dlg(t, views->currentDate(), this);
    dlg.showMaximized();
    if (dlg.exec() == QDialog::Accepted) {
      QualendarTask t1 = dlg.task();
      t1.setUid(t.uid());
      taskDB->edit(t, t1);
      //      emit newEvent();
    }
  } else if (item.type == QualendarItem::Event) {
    const Event &e = (*item.event).event();
    QDate d = (*item.event).date();
    editEventCommon(e, d, false);
  }
#endif // VIEWONLY
}

#ifdef VIEWONLY
void Qualendar::removeItem(QualendarItem) {
#else
void Qualendar::removeItem(QualendarItem item) {
  if (item.type == QualendarItem::Invalid)
    return;
  if (syncing || syncingTasks) {
    QMessageBox::warning(this, "Qualendar",
			 tr("Can not edit data, currently syncing"));
    return;
  }

  if (item.type == QualendarItem::Task) {
    const QualendarTask &t = *item.task;
    QString strName = t.description();

    if (!QPEMessageBox::confirmDelete(this, "Qualendar", strName))
      return;

    taskDB->remove(t);
    //    emit newEvent();
  } else if (item.type == QualendarItem::Event) {
    const Event &e = (*item.event).event();
    QString strName = e.description();
    if (!QPEMessageBox::confirmDelete(this, "Qualendar", strName))
      return;

    db->remove(e);
    //    emit newEvent();
  }
#endif // VIEWONLY
}

void Qualendar::createEventOrTask() {
  if (views->currentViewName() == "Todo") {
    createTask();
  } else
    createEvent();
}

void Qualendar::createEvent()
{
#ifndef VIEWONLY
  QDateTime start, end;
  views->selectedDateTime(start, end); 
  createEvent(start, end, "");
#endif
}

void Qualendar::createEvent(QDateTime start, QDateTime end, QString s) {
#ifndef VIEWONLY
  if (syncing || syncingTasks) {
    QMessageBox::warning( this, "Qualendar",
			  tr( "Can not edit data, currently syncing") );
    return;
  }

  Event ev;
  ev.setDescription(s);
  ev.setLocation("");
  ev.setStart(start);
  ev.setEnd(end);
  if (start.date() != end.date())
    ev.setType(Event::AllDay);

  editEventCommon(ev, start.date(), true);
#endif // VIEWONLY
}

void Qualendar::createTask() {
#ifndef VIEWONLY
  QualendarTask t;
  /*
  t.setStartDate(views->currentDate(), false);
  t.setDueDate(views->currentDate(), false);
  t.setFinishDate(views->currentDate(), false);
  */

  TodoDialog dlg(t, views->currentDate(), this);
  dlg.showMaximized();
  if (dlg.exec() == QDialog::Accepted) {
    QualendarTask t1 = dlg.task();
    t1.assignUid();
    taskDB->add(t1);
    //    emit newEvent();
  }
#endif
}

void Qualendar::cloneItem(QualendarItem item) {
#ifndef VIEWONLY
  if (item.type != QualendarItem::Event)
    return;

  if (syncing || syncingTasks) {
    QMessageBox::warning(this, "Qualendar",
			 tr("Can not edit data, currently syncing"));
    return;
  }

  const Event &e = (*item.event).event();
  Event ev;
  //  ev.assignUid();
  ev.setDescription(e.description());
  ev.setCategories(e.categories());
  ev.setLocation(e.location());
  ev.setType(e.type());
  ev.setStart(e.start());
  ev.setEnd(e.end());
  ev.setTimeZone(e.timeZone());
  ev.setAlarm(e.hasAlarm(), e.alarmTime(), e.alarmSound());
  ev.setNotes(e.notes());
  Event::RepeatPattern rp = e.repeatPattern();
  rp.createTime = time(NULL);
  ev.setRepeat(e.hasRepeat(), rp);
  db->setRid(ev, ev, true);
  db->setRinfo(ev, 1);

  db->add(ev);
  //  emit newEvent();
#endif // VIEWONLY
}

void Qualendar::exceptEvent(const EffectiveEvent &eev) { 
#ifndef VIEWONLY
  const Event &e = eev.event();
  if (e.repeatPattern().type == Event::NoRepeat)
    return;

  if (syncing || syncingTasks) {
    QMessageBox::warning(this, "Qualendar",
			 tr("Can not edit data, currently syncing"));
    return;
  }

  db->setSignalEnabled(false);

  int duration = e.start().secsTo(e.end()),  
    days = e.start().date().daysTo(e.end().date());
  bool b0, b1;
  QDateTime t0, t1;
  b0 = db->nextOccurance(e, e.start().date(), t0);
  b1 = db->nextOccurance(e, eev.date().addDays(1), t1);

  if (b0 && t0.date() < eev.date()) {
    Event ev;
    //    ev.assignUid();
    ev.setDescription(e.description());
    ev.setCategories(e.categories());
    ev.setLocation(e.location());
    ev.setType(e.type());
    ev.setStart(e.start());
    ev.setEnd(e.end());
    ev.setTimeZone(e.timeZone());
    ev.setAlarm(e.hasAlarm(), e.alarmTime(), e.alarmSound());
    ev.setNotes(e.notes());
    Event::RepeatPattern rp = e.repeatPattern();
    rp.createTime = time(NULL);
    rp.hasEndDate = true;
    rp.setEndDate(eev.date().addDays(-1));
    ev.setRepeat(e.hasRepeat(), rp);
    db->setRid(ev, ev, true);
    db->setRinfo(ev, 1);

    db->add(ev);
  }
  if (b1) {
    Event ev;
    //    ev.assignUid();
    ev.setDescription(e.description());
    ev.setCategories(e.categories());
    ev.setLocation(e.location());
    ev.setType(e.type());
    ev.setStart(t1);
    ev.setEnd(t1.addSecs(duration));
    ev.setTimeZone(e.timeZone());
    ev.setAlarm(e.hasAlarm(), e.alarmTime(), e.alarmSound());
    ev.setNotes(e.notes());
    Event::RepeatPattern rp = e.repeatPattern();
    rp.createTime = time(NULL);
    ev.setRepeat(true, rp);
    db->setRid(ev, ev, true);
    db->setRinfo(ev, 1);

    db->add(ev);
  }

  {
    Event ev;
    //    ev.assignUid();
    ev.setDescription(e.description());
    ev.setCategories(e.categories());
    ev.setLocation(e.location());
    ev.setType(e.type());
    ev.setStart(QDateTime(eev.date(), eev.start()));
    ev.setEnd(ev.start().addSecs(duration));
    ev.setTimeZone(e.timeZone());
    ev.setAlarm(e.hasAlarm(), e.alarmTime(), e.alarmSound());
    ev.setNotes(e.notes());
    Event::RepeatPattern rp = e.repeatPattern();
    rp.type = Event::NoRepeat;
    rp.createTime = time(NULL);
    ev.setRepeat(rp);
    db->setRid(ev, ev, true);
    db->setRinfo(ev, 1);

    db->remove(e);

    db->setSignalEnabled(true);

    editEventCommon(ev, QDate(), true);
  }
#endif // VIEWONLY
}


static const char * beamfile = "/tmp/obex/event.vcs";

void Qualendar::beamItem(QualendarItem item) {
  if (item.type != QualendarItem::Event)
    return;

  qDebug("trying to beamn");
  unlink( beamfile ); // delete if exists
  mkdir("/tmp/obex/", 0755);
  // work around against a bug(?) in writeVCalendar on Zaurus
  const Event &e = (*item.event).event();
  Event ev = e;
  if (ev.notes()[ev.notes().length()-1] == '\n')
    ev.setNotes(e.notes() + "\n");
  Event::writeVCalendar( beamfile, ev );
  Ir *ir = new Ir( this );
  connect( ir, SIGNAL( done( Ir * ) ), this, SLOT( beamDone( Ir * ) ) );
  QString description = e.description();
  ir->send( beamfile, description, "text/x-vCalendar" );
}

void Qualendar::beamDone(Ir *ir) {
  delete ir;
  unlink(beamfile);
}

void Qualendar::removeMonth() {
#ifndef VIEWONLY
  int year = views->currentDate().year();
  int month = views->currentDate().month();
  if (!QPEMessageBox::confirmDelete(this, "Qualendar",
				    tr("all nonrepeating events in %1/%2")
				    .arg(year).arg(month)))
    return;
  QString backupFile =
    (QPEApplication::documentDir() + "qualendar_" +
     QString::number(year) +
     (month < 10 ? "0" : "") + QString::number(month) +
     ".vcs").latin1();
  //    unlink( backupFile ); // delete if exists

  QDate dtStart( year, month, 1 );
  QValueList<EffectiveEvent> l
    = db->getEffectiveEvents( dtStart,
			      QDate( year, month,
				     dtStart.daysInMonth() ) );
  QValueList<Event> m, m1;
  QValueListIterator<EffectiveEvent> it = l.begin();
  while ( it != l.end() ) {
    EffectiveEvent e = *it;
    ++it;
    if (!e.event().hasRepeat() &&
	m.findIndex(e.event()) < 0) {
      m.append(e.event());
      // work around against a bug(?) in writeVCalendar on Zaurus
      Event ev = e.event();
      if (ev.notes()[ev.notes().length()-1] == '\n')
	ev.setNotes(e.notes() + "\n");
      m1.append(ev);
    }
  }
  Event::writeVCalendar( backupFile, m1 );
  QValueListIterator<Event> it1 = m.begin();

  db->setSignalEnabled(false);
  while ( it1 != m.end() ) {
    db->remove(*it1);
    ++it1;
  }
  db->setSignalEnabled(true);

  //  emit newEvent();
#endif // VIEWONLY
}

#if 0
void Qualendar::exportToVCalendar() {
  if (QMessageBox::warning(this, "Export to VCalendar",
    "Export to VCalendar ?\n("
    + QPEApplication::documentDir()
    + "qualendar_all.vcs)",
    tr("Yes"),
    tr("No"),
    QString::null,
    1)
    != 0)
    return;
  QString exportFile =
	(QPEApplication::documentDir() + "qualendar_all.vcs").latin1();

  QValueList<Event> m;
  {
	QValueList<Event> repeats = db->getRawRepeats();

    QValueListIterator<Event> it = repeats.begin();
    while ( it != repeats.end() ) {
	  // work around against a bug(?) in writeVCalendar on Zaurus
	  Event ev = *it;
	  ++it;
	  if (ev.notes()[ev.notes().length()-1] == '\n')
		ev.setNotes(ev.notes() + "\n");
	  m.append(ev);
    }
  }
  {
	QValueList<Event> nonRepeats =
	  db->getNonRepeatingEvents(QDate(1900, 1, 1),
								QDate(2100, 1, 1)); // FIXME

    QValueListIterator<Event> it = nonRepeats.begin();
    while ( it != nonRepeats.end() ) {
	  // work around against a bug(?) in writeVCalendar on Zaurus
	  Event ev = *it;
	  ++it;
	  if (ev.notes()[ev.notes().length()-1] == '\n')
		ev.setNotes(ev.notes() + "\n");
	  m.append(ev);
    }
  }
  Event::writeVCalendar( exportFile, m );
}
#endif // 0

////////
// Sanity check
////////
QString Qualendar::checkEvent(const Event &e) {
#ifndef VIEWONLY
  /* check if overlaps with itself */
  bool checkFailed = FALSE;

  /* check the next 12 repeats. should catch most problems */
  QDate current_date = e.start().date();
  Event previous = e;
  for(int i = 0; i < 12; i++) {
    QDateTime next;
    if (!db->nextOccurance(previous, current_date.addDays(1), next)) {
      break;  // no more repeats
    }
    if(next < previous.end()) {
      checkFailed = TRUE;
      break;
    }
    current_date = next.date();
  }

  if(checkFailed)
    return tr("Event duration is potentially longer\n"
	      "than interval between repeats.");

  QDateTime next;
  if (e.hasRepeat() && !db->nextOccurance(e, e.start().date(), next)) {
    return tr("No occurence");
  }
#endif // VIEWONLY
  return QString::null;
}

