/*
 * Copyright (C) 2003 ichitokumei
 * 
 * Derived from qtopia-free-1.5.0/datebook/datebook.cpp
 * 
 * Implementation of qapp/qcop message handling in Qualendar.
 */

/**********************************************************************
** Copyright (C) 2000-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qtopia Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include <qaction.h>
#include <qdatastream.h>
#include <qdatetime.h>
#include <qfileinfo.h>
#include <qmessagebox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qscrollview.h>
#include <qtextcodec.h>

#include <qcopchannel_qws.h>
#include <qpe/qpeapplication.h>
#include <qpe/timestring.h>

#include "viewonly.h"
#include "qualendar.h"
#include "qualendarglobal.h"
#include "qualendarview.h"
#include "sysdep/qualendardb.h"
#include "sysdep/qualendarsysdep.h"
#include "sysdep/qualendarcategory.h"

////////
// Make connections for qApp signals and QCop messages.
////////
void Qualendar::makeSystemConnections() {
  connect(qApp, SIGNAL(clockChanged(bool)), this, SLOT(changeClock(bool)));
  connect(qApp, SIGNAL(weekChanged(bool)), this, SLOT(changeWeek(bool)));

#if defined(Q_WS_QWS) && !defined(QT_NO_COP)
  connect(qApp, SIGNAL(appMessage(const QCString&, const QByteArray&)),
	  this, SLOT(appMessage(const QCString&, const QByteArray&)));
#endif

  // listen on QPE/System
#if defined(Q_WS_QWS)
#if !defined(QT_NO_COP)
  QCopChannel *channel = new QCopChannel("QPE/System", this);
  connect(channel, SIGNAL(received(const QCString&, const QByteArray&)),
	  this, SLOT(receive(const QCString&, const QByteArray&)));
  channel = new QCopChannel("QPE/Sync", this);
  connect(channel, SIGNAL(received(const QCString&, const QByteArray&)),
	  this, SLOT(receiveSync(const QCString&, const QByteArray&)));
#endif
#endif
}


void Qualendar::receive( const QCString &msg, const QByteArray &data )
{
  writeLog(QString("receive: ") + QString(msg));
  QDataStream stream( data, IO_ReadOnly );
  if ( msg == "timeChange(QString)" ) {
    if (isVisible())
      views->redraw();
  }
}
void Qualendar::receiveSync( const QCString &msg, const QByteArray &data )
{
  writeLog(QString("receiveSync: ") + QString(msg));
}

void Qualendar::appMessage(const QCString& msg, const QByteArray& data)
{
  writeLog(QString("appMessage: ") + QString(msg));

  bool needShow = FALSE;
  if ( msg == "alarm(QDateTime,int)" ) {
#ifndef VIEWONLY
    QDataStream ds(data,IO_ReadOnly);
    QDateTime when; int warn;
    ds >> when >> warn;

    // check to make it's okay to continue,
    // this is the case that the time was set ahead, and
    // we are forced given a stale alarm...
    QDateTime current = QDateTime::currentDateTime();
    if (current.time().hour() != when.time().hour() &&
	current.time().minute() != when.time().minute() )
      return;

    QValueList<EffectiveEvent> list = db->getEffectiveEvents(when.addSecs(warn*60));
    if ( list.count() > 0 ) {
      QString msg;
      bool bSound = false;
      int stopTimer = 0;
      bool found = false;
      for (QValueList<EffectiveEvent>::ConstIterator it=list.begin();
	   it!=list.end(); ++it ) {
	if ((*it).event().hasAlarm()) {
	  found = true;
	  msg += "<CENTER><B>" + (*it).description() + "</B>"
	    + "<BR>" + (*it).location() + "<BR>"
	    + TimeString::dateString((*it).event().start(),
				     QualendarGlobal::ampm)
	    + (warn ? tr(" (in %1 minutes)").arg(warn) : QString(""))
	    + "<BR>"
	    + (*it).notes() + "</CENTER>";
	  if ((*it).event().alarmSound() != Event::Silent) {
	    bSound = true;
	  }
	}
      }
      if ( found ) {
	if ( bSound ) {
	  QualendarSysdep::playScheduleAlarm();
	  stopTimer = startTimer(5000);
	}

	QDialog dlg( this, 0, TRUE );
	QVBoxLayout *vb = new QVBoxLayout( &dlg );
	QScrollView *view = new QScrollView( &dlg, "scrollView");
	view->setResizePolicy( QScrollView::AutoOneFit );
	vb->addWidget( view );
	QLabel *lblMsg = new QLabel( msg, &dlg );
	view->addChild( lblMsg );
	QPushButton *cmdOk = new QPushButton( tr("OK"), &dlg );
	connect( cmdOk, SIGNAL(clicked()), &dlg, SLOT(accept()) );
	vb->addWidget( cmdOk );

#if defined(Q_WS_QWS) || defined(_WS_QWS_)
	dlg.showMaximized();
#endif
	QualendarSysdep::setLED(true);

	needShow = dlg.exec();

	if ( bSound )
	  killTimer( stopTimer );

	QualendarSysdep::setLED(false);
      }
    }
#endif // VIEWONLY
  } else if ( msg == "nextView()" ) {
    QString v = views->currentViewName();
    if (v == "Month")
      views->gotoView("Week");
    if (v == "Week")
      views->gotoView("Day");
    if (v == "Day")
      views->gotoView("Month");
  } else if ( msg == "reloadSettings()" ) {
    reloadSettings();
    needShow = FALSE;
  } else if (msg == "gotoView(QString)") {
    QDataStream ds(data, IO_ReadOnly);
    QString str;
    ds >> str;
    views->gotoView(str);
    showMaximized();
  } else if (msg == "reloadTodo()") {
    reloadTasks();
#ifndef VIEWONLY
  } else if (msg == "flushTodo()") {
    flushTasks();
  } else if ( msg == "newEvent(QString,QString,QString,QString,QString,QString)") {
    needShow = FALSE;
    // start time, end time, description, notes, category
    // 00:00 -- 23:59 => all day
    QDataStream ds(data,IO_ReadOnly);
    QString description, location, notes, category;
    QDateTime start, end;
    QString startStr, endStr;
    ds >> startStr >> endStr >> description >> location >> notes >> category;
    start.setDate(QDate(startStr.left(4).toInt(),
			startStr.mid(4, 2).toInt(),
			startStr.mid(6, 2).toInt()));
    start.setTime(QTime(startStr.mid(8, 2).toInt(),
			startStr.mid(10, 2).toInt()));
    end.setDate(QDate(endStr.left(4).toInt(),
		      endStr.mid(4, 2).toInt(),
		      endStr.mid(6, 2).toInt()));
    end.setTime(QTime(endStr.mid(8, 2).toInt(),
		      endStr.mid(10, 2).toInt()));
    bool allDay =
      start.time() == QTime(0, 0) && end.time() == QTime(23, 59);


    if (start.isValid() && end.isValid() &&
	start < end) {
      Event newEv;
      newEv.assignUid();
      newEv.setStart(start);
      newEv.setEnd(end);
      newEv.setType(allDay ? Event::AllDay : Event::Normal);
      newEv.setDescription(QTextCodec::codecForLocale()->toUnicode(description));
      newEv.setLocation(QTextCodec::codecForLocale()->toUnicode(location));
      newEv.setNotes(QTextCodec::codecForLocale()->toUnicode(notes));
      int catId =
	QualendarGlobal::categories.id("Calendar", QTextCodec::codecForLocale()->toUnicode(category));
      if (catId != 0)
	newEv.setCategories(catId);

      db->setRid(newEv, newEv, true);

      db->add(newEv);
      //      emit newEvent();
    }
#endif
  }

  if ( needShow ) {
#if defined(Q_WS_QWS) || defined(_WS_QWS_)
    showMaximized();
#else
    show();
#endif
    raise();
    QPEApplication::setKeepRunning();
    setActiveWindow();
  }
}
void Qualendar::flush()
{
  writeLog("flush");
#ifndef VIEWONLY
  syncing = TRUE;
  db->save();
#endif
}

void Qualendar::flushTasks()
{
  writeLog("flushTasks");
#ifndef VIEWONLY
  syncingTasks = TRUE;
  taskDB->save();
#endif
}

void Qualendar::timerEvent( QTimerEvent *e )
#ifndef VIEWONLY
{
  static int stop = 0;
  if ( stop < 10 ) {
    QualendarSysdep::playScheduleAlarm();
    stop++;
  } else {
    stop = 0;
    killTimer(e->timerId());
  }
#endif // VIEWONLY
}

void Qualendar::changeClock( bool newClock )
{
    QualendarGlobal::ampm = newClock;
		views->redraw();
}

void Qualendar::changeWeek( bool m )
{
  QualendarGlobal::onMonday = m;
	views->redraw(); 
}

void Qualendar::setDocument( const QString &filename )
{
#ifndef VIEWONLY
  if (filename.find(".vcs") != int(filename.length()) - 4) return;

  if (QMessageBox::warning(this, tr("Import vCalendar"),
			   tr("Import %1 \n in %2 \n?")
			   .arg(QFileInfo(filename).fileName())
			   .arg(QFileInfo(filename).dirPath()), 
			   tr("Yes"),
			   tr("No"),
			   QString::null,
			   1)
      != 0)
    return;

  QValueList<Event> tl = Event::readVCalendar(filename);
  db->setSignalEnabled(false);
  for(QValueList<Event>::Iterator it = tl.begin(); it != tl.end(); ++it ) {
    Event e = (*it);
    e.assignUid();
    db->add(e);
  }
  db->setSignalEnabled(true);
  //  redraw();
#endif // VIEWONLY
}

