#include <stdlib.h>

#include <qapplication.h>
#include <qfileinfo.h>
#include <qtextstream.h>
#include <qtextcodec.h>
#include <qimage.h>
#include <qregexp.h>

#include <qpe/qpeapplication.h>
#include <qpe/config.h>
#include <qpe/resource.h>
#include <qpe/qcopenvelope_qws.h>

#include "sysdep/qualendarcategory.h"
#include "sysdep/qualendarsysdep.h"

#define HOLIDAY_FILE "/opt/Qtopia/etc/holidays.txt"

#include "qualendarglobal.h"

#include "globals/globals.cpp_0"

QString QualendarGlobal::filterName[10];

QMap<QDate, QString> QualendarGlobal::holidays;
QMap<int, QColor> QualendarGlobal::colors;
QMap<QString, QMap<int, QMap<int, bool> > > QualendarGlobal::filter;

QualendarCategories QualendarGlobal::categories;
int QualendarGlobal::unfiledCatId;

QMap<QString, QString> QualendarGlobal::icons;
QDict<QPixmap> QualendarGlobal::iconPixmaps;
QMap<QString, QString> QualendarGlobal::backgrounds;
QDict<QPixmap> QualendarGlobal::backgroundPixmaps;


QString QualendarGlobal::binName() {
  return QFileInfo(qApp->argv()[0]).fileName();
}


void QualendarGlobal::load() {

#include "globals/globals.cpp_1"

  {
    Config config("Qualendar");
    config.setGroup("Filter");
    for (uint i=0; i<10; i++) 
      filterName[i] = config.readEntry(QString("FilterName%1").arg(i), 
				       QString::number(i));
  }

  categories.load();
  unfiledCatId = QualendarGlobal::categories.id("Calendar", QObject::tr("Unfiled"));

  holidays.clear();
  QFile holidayFile(HOLIDAY_FILE);
  holidayFile.open(IO_ReadOnly);
  if (holidayFile.isOpen()) {
    QTextStream s(&holidayFile);
    s.setCodec(QTextCodec::codecForName("eucJP"));
    while(!s.atEnd()) {
      int y, m, d;
      d = 0;
      s >> y >> m >> d >> ws; 
      if (d > 0) {
	QString *desc = new QString;
	s >> *desc >> ws;
	holidays.insert(QDate(y, m, d), *desc);
      }
    }
    holidayFile.close();
  }

  QRegExp re("=[^=]*$");

  colors.clear();
  QFile colorFile((QString(getenv("HOME")) + "/Settings/QualendarColor.txt"));
  colorFile.open(IO_ReadOnly);
  if (colorFile.isOpen()) {
    QTextStream s(&colorFile);
    s.setCodec(QTextCodec::codecForName("utf8"));
    while(!s.atEnd()) {
      QString str = s.readLine();
      int i;
      if ((i = re.find(str, 0)) > 0) {
	int catId = categories.id("Calendar", str.left(i));
	if (str.left(i) == QObject::tr("Unfiled") || catId != 0) {
	  int col = str.right(str.length() - (i+1)).toInt();
	  colors.insert(catId, 
			QColor(col >> 16, (col >> 8) & 0xff, col & 0xff));
	}
      }
    }
    colorFile.close();
  }

  backgrounds.clear();
  backgroundPixmaps.setAutoDelete(true);
  backgroundPixmaps.clear();
  QFile backgroundFile((QString(getenv("HOME")) + 
			"/Settings/QualendarBackground.txt"));
  backgroundFile.open(IO_ReadOnly);
  if (backgroundFile.isOpen()) {
    QTextStream s(&backgroundFile);
    s.setCodec(QTextCodec::codecForName("utf8"));
    while(!s.atEnd()) {
      QString str = s.readLine();
      int i;
      if ((i = re.find(str, 0)) > 0) {
	backgrounds.insert(str.left(i), 
			   str.right(str.length() - (i+1)));
      }
    }
    backgroundFile.close();
  }

  icons.clear();
  iconPixmaps.setAutoDelete(true);
  iconPixmaps.clear();
  QFile iconFile((QString(getenv("HOME")) + "/Settings/QualendarIcon.txt"));
  iconFile.open(IO_ReadOnly);
  if (iconFile.isOpen()) {
    QTextStream s(&iconFile);
    s.setCodec(QTextCodec::codecForName("utf8"));
    while(!s.atEnd()) {
      QString str = s.readLine();
      int i;
      if ((i = re.find(str, 0)) > 0) {
	icons.insert(str.left(i),
		     str.right(str.length() - (i+1)));
      }
    }
    iconFile.close();
  }

  filter.clear();
  QFile filterFile((QString(getenv("HOME")) 
		    + "/Settings/QualendarFilter.txt"));
  filterFile.open(IO_ReadOnly);
  QRegExp re1("\\[[a-zA-Z]*\\]=[^=]*$");
  if (filterFile.isOpen()) {
    QTextStream s(&filterFile);
    s.setCodec(QTextCodec::codecForName("utf8"));
    while(!s.atEnd()) {
      QString str = s.readLine();
      int i;
      if ((i = re1.find(str, 0)) > 0) {
	int j = re.find(str, 0);
	QString viewName = str.mid(i+1, (j-1)-(i+1));
	QString filterString = str.right(str.length() - (j+1)); 
	int catId = categories.id("Calendar", str.left(i));
	if (str.left(i) == QObject::tr("Unfiled") || catId != 0) {
	  for(int k=0; k<filterString.length(); k++) {
	    int n = filterString[k].latin1() - '0';
	    if (n >= 0)
	      filter[viewName][n][catId] = true;
	  }
	}
      }
    }
    filterFile.close();
  }
}


void QualendarGlobal::openDocument(QString str) {
  QString s;
  if (str.length() >= 7 && str.left(7) == "file://") {
    s = str.right(str.length() - 7);
    if (s.length() >= 4 && s.right(4) == ".txt") {
      QCopEnvelope e(QCString("QPE/Application/" + textEditor), 
		     "setDocument(QString)");
      e << s;
    } else if ((s.length() >= 4 && 
		(s.right(4) == ".png" || s.right(4) == ".jpg" ||
		 s.right(4) == ".bmp" || s.right(4) == ".gif")) ||
	       (s.length() >= 5 && s.right(4) == ".jpeg")) {
      QCopEnvelope e(QCString("QPE/Application/" + imageViewer), 
		     "setDocument(QString)");
      e << s;
    } else if (s.length() >= 4 && 
	       s.right(4) == ".vcs") {
      QCopEnvelope e(QCString("QPE/Application/" + binName()), 
		     "setDocument(QString)");
      e << s;
    } else if (s.length() >= 4 && 
	       s.right(4) == ".pdf") {
      QCopEnvelope e("QPE/Application/qpdf", 
		     "setDocument(QString)");
      e << s;
    } else if (s.length() >= 4 && s.right(4) == ".doc") {
      QCopEnvelope e("QPE/Application/HancomMobileWord", 
		     "setDocument(QString)");
      e << s;
    } else if (s.length() >= 4 && s.right(4) == ".xls") {
      QCopEnvelope e("QPE/Application/hancomsheet", 
		     "setDocument(QString)");
      e << s;
    } else if ((s.length() >= 4 && (s.right(4) == ".htm" ||
									s.right(4) == ".mht" ||
									s.right(4) == ".url")) ||
			   (s.length() >= 5 && s.right(5) == ".html")) {
      QCopEnvelope e(QCString("QPE/Application/" + webBrowser), 
					 "setDocument(QString)");
      e << s;
    }
  } else if (str.length() >= 7 && str.left(7) == "http://") {
    s = str;
    QCopEnvelope e(QCString("QPE/Application/" + webBrowser), 
		   "setDocument(QString)");
    e << s;
  }
}
void QualendarGlobal::launchTodo() {
  QCopEnvelope e("QPE/System", "execute(QString)");
  e << QualendarSysdep::todoBin(); 
}

QMap<int, bool> QualendarGlobal::getFilter(const QString &view, int filterNo) {
	if (filter.contains(view) && filter[view].contains(filterNo))
		return filter[view][filterNo];
	else {
		QMap<int, bool> map;
		return map;
	}
}

// this can be slow when you have many icon definitions.
QPixmap QualendarGlobal::getIcon(const QString &desc, int h, QString &key) {
  key = "";
  QPixmap *pm = 0;
  int match = 0;
  QMapIterator<QString, QString> it;
  for (it = icons.begin(); it != icons.end(); ++it) {
    QString s = it.key();
    if (s.length() <= match)
      continue;
    if (desc.left(s.length()) == s) {
      key = s;
      match = s.length();
      pm = iconPixmaps[key];
      if (!pm || pm->height() != h) {
	qDebug(QString("loading icon: qualendar/") + it.data());
	QImage im = Resource::loadImage((QString("qualendar/") + it.data()));
	if (im.isNull()) {
	  pm = 0;
	} else {
	  int w1 = im.width(), h1 = im.height();
	  QImage im1 = im.smoothScale(w1*h/h1, h);
	  pm = new QPixmap;
	  *pm = im1;
	  iconPixmaps.insert(key, pm);
	}
      }
    }
  }
  if (pm)
    return *pm;
  return QPixmap();
}

QPixmap QualendarGlobal::getBackground(const QString &desc, int w, int h) {
#if 1
  QString key;
  QPixmap *pm = 0;
  int match = 0;
  QMapIterator<QString, QString> it;
  for (it = backgrounds.begin(); it != backgrounds.end(); ++it) {
    QString s = it.key();
    if (s.length() <= match)
      continue;
    if (desc.left(s.length()) == s) {
      key = s;
      match = s.length();
      pm = backgroundPixmaps[key];

      if (!pm || (pm->width() != w && pm->height() != h)) {
	qDebug(QString("loading background: qualendar/") + it.data());
	QImage im = Resource::loadImage((QString("qualendar/") + it.data()));
	if (im.isNull()) {
	  pm = 0;
	} else {
	  int w1 = im.width(), h1 = im.height();

	  QImage im1 = 
	    (w * h1 > h * w1) ? 
	    im.smoothScale(w1*h/h1, h) : im.smoothScale(w, h1*w/w1);
	  pm = new QPixmap;
	  *pm = im1;
	  backgroundPixmaps.insert(key, pm);
	}
      }
    }
  }
  if (pm)
    return *pm;
  return QPixmap();
#else
  QPixmap *pm = backgroundPixmaps[desc];
  if (pm && (pm->isNull() || (pm->width() == w && pm->height() == h))) 
    return *pm;

  if (w <= 0 || h <= 0 || !backgrounds.contains(desc)) 
    return QPixmap();

  QString s = backgrounds[desc];
  qDebug(QString("loading background: qualendar/") + s);
  QImage im = Resource::loadImage((QString("qualendar/") + s));
  if (im.isNull()) {
    backgroundPixmaps.insert(desc, new QPixmap);
    return QPixmap();
  }
  QImage im1 = im.smoothScale(w, h);
  QPixmap *p = new QPixmap;
  *p = im1;
  backgroundPixmaps.insert(desc, p);
  return *p;
#endif // 1
}

QString QualendarGlobal::dayName(int i) {
  switch(i) {
    case 0:
    return QObject::tr("Sun");
    case 1:
    return QObject::tr("Mon");
    case 2:
    return QObject::tr("Tue");
    case 3:
    return QObject::tr("Wed");
    case 4:
    return QObject::tr("Thu");
    case 5:
    return QObject::tr("Fri");
    case 6:
    return QObject::tr("Sat");
    default:
    return QString();
  }
}
