#ifndef QUALENDARGLOBAL_H
#define QUALENDARGLOBAL_H

#include <qobject.h>
#include <qstring.h>
#include <qarray.h>
#include <qdict.h>
#include <qintdict.h>
#include <qcolor.h>
#include <qpixmap.h>
#include <qmap.h>
#include <qdatetime.h>

class QualendarCategories;

/**
 * The class for global data for Qualendar.
 * 
 * ... such as settings, holidays, filters and pixmaps.
 **/

class QualendarGlobal {
 public:
  /**
   * Load all data.
   **/
  static void load();
  /**
   * The base name of the binary.
   **/
  static QString binName();
  /**
   * Open a document by an external application 
   * according to the extention(or something).
   **/
  static void openDocument(QString);
  /**
   * Launch the Todo application, 
   * whose name is given by QualendarSysdep::todoBin().
   **/
  static void launchTodo();
  /**
   * Get the filter as a map (category id -> bool).
   * There are 10 filters(for no logical reason), 
   * so filterNo must be 0 to 9. 
   **/
  static QMap<int, bool> getFilter(const QString &view, int filterNo);
  /**
   * Returns an icon pixmap if @p desc begins with a keyword
   * (in Settings/QualendarIcon.txt, longest match).
   * The size is determined by the height h.
   * The keyword is returned as key.
   * If not, returns a null QPixmap.
   **/
  static QPixmap getIcon(const QString &desc, int h, QString &key);
  /**
   * Returns a background pixmap if @p desc equals a keyword
   * (in Settings/QualendarBackground.txt).
   * If not, returns a null QPixmap.
   **/
  static QPixmap getBackground(const QString &desc, int w, int h);

  static QString dayName(int i);


  static QString filterName[10];
  static QMap<QDate, QString> holidays;
  /**
   * category id -> color
   **/
  static QMap<int, QColor> colors;
  static QualendarCategories categories;
  static int unfiledCatId;

  // And then, auto generated list of variables
#include "globals/globals.h_0"

 private:
  static QMap<QString, QMap<int, QMap<int, bool> > > filter;
  enum {Day, Week, Month};
  static QMap<QString, QString> icons;
  static QDict<QPixmap> iconPixmaps;
  static QMap<QString, QString> backgrounds;
  static QDict<QPixmap> backgroundPixmaps;
};


#endif // QUALENDARGLOBAL_H
