#include <qaction.h>
#include <qpopupmenu.h>
#include <qpe/config.h>
#include <qimage.h>
#include <qpe/resource.h>

#include "qualendarview.h"
#include "qualendaritem.h"

#include "qualendarglobal.h"
#include "views/qualendaryear.h"
#include "views/qualendarmonth.h"
#include "views/qualendarweek.h"
#include "views/qualendarday.h"
#include "views/qualendarfind.h"
#include "views/qualendartodoview.h"
#include "sysdep/qualendareventdb.h"
#include "sysdep/qualendartaskdb.h"

#include "debug.h"

QualendarViewStack::QualendarViewStack(QWidget *parent, const char* name,  
				       QArray<QualendarDB *> dataBases, 
				       QPopupMenu *viewMenu)
  : QWidgetStack(parent, name), menu_(viewMenu) {
  Config config("Qualendar");
  config.setGroup("ViewStack");
  QStringList sl, sl0, sl1;
  sl.append("Month");
  sl.append("Week");
  sl.append("Day");
  sl.append("Year");
  sl.append("Find");
  sl.append("Todo");

  sl0 = sl;
QTime t;
t.start();
  for(uint i=0; i<6; i++) {
    QString s = config.readEntry("View" + QString::number(i), sl0[i]);
    if (sl.findIndex(s) >= 0) {
      sl1.append(s);
      sl.remove(s);
    }
  }
  iconCount = sl1.count();
  for(uint i = 0; i < sl.count(); i++)
    sl1.append(sl[i]);

  for(uint i=0; i<6; i++) {
    QString viewName = sl1[i];

    if (viewName == "Year") {
      int n = views_.count();
      views_.resize(n+1);
      views_[n] = new QualendarYear(this, 0, dataBases, viewMenu);
    } else if (viewName == "Month") {
      int n = views_.count();
      views_.resize(n+1);
      views_[n] = new QualendarMonth(this, 0, dataBases, viewMenu);
    } else if (viewName == "Week") {
      int n = views_.count();
      views_.resize(n+1);
      views_[n] = new QualendarWeek(this, 0, dataBases, viewMenu);
    } else if (viewName == "Day") {
      int n = views_.count();
      views_.resize(n+1);
      views_[n] = new QualendarDay(this, 0, dataBases, viewMenu);
    } else if (viewName == "Find") {
      int n = views_.count();
      views_.resize(n+1);
      views_[n] = new QualendarFind(this, 0, dataBases, viewMenu);
    } else if (viewName == "Todo") {
      int n = views_.count();
      views_.resize(n+1);
      views_[n] = new QualendarTodoView(this, 0, dataBases, viewMenu);
    }
qDebug("%s %d ms", viewName.latin1(), t.elapsed());
  }
  dirty_.fill(false, views_.count());
  for (uint i=0; i < views_.count(); i++) {
    views_[i]->loadConfig();
    connect(views_[i], SIGNAL(viewItem(QualendarItem)), 
	    this, SIGNAL(viewItem(QualendarItem))); 
    connect(views_[i], SIGNAL(editItem(QualendarItem)), 
	    this, SIGNAL(editItem(QualendarItem))); 
    connect(views_[i], SIGNAL(removeItem(QualendarItem)), 
	    this, SIGNAL(removeItem(QualendarItem))); 
    connect(views_[i], SIGNAL(createEvent(QDateTime, QDateTime, QString)),
	    this, SIGNAL(createEvent(QDateTime, QDateTime, QString)));
  }
  currentView_ = 0;
  setDate(QDate::currentDate(), false);
  views_[0]->setupMenu();
  raiseWidget(views_[0]);
}

bool QualendarViewStack::mayClose() {
  return (currentView_ == 0);
}

QString QualendarViewStack::currentViewName() const {
  if (currentView_ < 0 || currentView_ >= views_.count() ||
      !views_[currentView_])
    return "";
  else
    return views_[currentView_]->viewName();
}

QValueList<QPixmap> QualendarViewStack::icons() const {
  QValueList<QPixmap> l;
  for (uint i=0; i < iconCount; i++) 
    l.append(views_[i]->icon());
  return l;
}
QStringList QualendarViewStack::trViewNames() const {
  QStringList l;
  for (uint i=0; i < views_.count(); i++) 
    l.append(views_[i]->trViewName());
  return l;
}

void QualendarViewStack::selectedDateTime(QDateTime &start, QDateTime &end) {
	views_[currentView_]->selectedDateTime(start, end);
}
void QualendarViewStack::gotoView(const QString &viewName) {
  DRAW_DEBUG(qDebug("gotoView: " + viewName));
  for(uint i=0; i < views_.count(); i++) {
    if (views_[i]->viewName() == viewName) 
      gotoView(i);
  }
}
void QualendarViewStack::gotoView(int i) {
  DRAW_DEBUG(qDebug("gotoView: " + QString::number(i)));
  if (i >= 0 && i < views_.count()) {
    QDate d = views_[currentView_]->currentDate();
    currentView_ = i;
    views_[i]->setDate(d, false);
    menu_->clear();
    views_[i]->setupMenu();
    raiseWidget(views_[i]);
    if (dirty_[i]) {
      views_[i]->redraw();
      dirty_[i] = false;
    }
    emit viewChanged(i);
  }
}
void QualendarViewStack::nextView() {
  if (currentView_ < iconCount - 1)
    gotoView(currentView_ + 1);
}
void QualendarViewStack::prevView() {
  if (currentView_ > 0)
    gotoView(currentView_ - 1);
}

void QualendarViewStack::setDate(QDate d, bool draw) {
	views_[currentView_]->setDate(d, draw);
}
//void QualendarViewStack::setFilter(QMap<int, bool> filter, bool draw) {
void QualendarViewStack::setFilter(int f, bool draw) {
  for(int i=0; i<views_.count(); i++) {
    views_[i]->setFilter(QualendarGlobal::getFilter(views_[i]->viewName(), f), draw && (i == currentView_));
    dirty_[i] = (i != currentView_);
  }
}

QDate QualendarViewStack::currentDate() {
	return views_[currentView_]->currentDate();
}

void QualendarViewStack::redraw() {
  if (currentView_ >= 0 && currentView_ < views_.count() &&
      views_[currentView_]) {
    views_[currentView_]->redraw();
    for (uint i=0; i < dirty_.size(); i++) 
      dirty_[i] = (i != currentView_);
  }
}
void QualendarViewStack::loadConfig() {
  for(uint i=0; i<views_.count(); i++)
    views_[i]->loadConfig();
}
void QualendarViewStack::keyPressEvent(QKeyEvent *e) {
  switch (e->key()) {
  case Key_Return:
    if (QualendarGlobal::spaceReturn) 
      prevView();
    else
      nextView();
    break;
  case Key_Space:
    if (QualendarGlobal::spaceReturn) 
      nextView();
    break;
  case Key_Escape:
    if (QualendarGlobal::cancelToClose || QualendarGlobal::spaceReturn ||
	currentView_ == 0) 
      e->ignore();
    else
      prevView();
    break;
  default:
    e->ignore();
  }
}

QualendarView::QualendarView(QWidget *parent, const char *name, 
			     QArray<QualendarDB *> dataBases,
			     QPopupMenu *viewMenu)
  : QVBox(parent, name), menu(viewMenu) {

  // FIXME: it is assumed that there are one QualendarEventDB and one QualendarTaskDB.
  for(int i=0; i<dataBases.size(); i++) { 
    if (dataBases[i]->inherits("QualendarEventDB")) {
      db = (QualendarEventDB *) dataBases[i];
    }
    if (dataBases[i]->inherits("QualendarTaskDB")) {
      taskDB = (QualendarTaskDB *) dataBases[i];
    }
  }
  setFrameStyle(QFrame::NoFrame);
  setFocusPolicy(StrongFocus);
}

QColor QualendarView::getColor(const Event &e) {
  int catId = (e.categories().count() > 0 ? e.categories()[0] : 
	       QualendarGlobal::unfiledCatId);
  if (QualendarGlobal::colors.contains(catId))
    return QualendarGlobal::colors[catId];
  else
    return QColor(0, 0, 0);
}

QString QualendarView::timeString(const EffectiveEvent &e) {
  QString s;
  if (e.event().type() == Event::AllDay)
    return "====";
  QTime t0 = e.start(), t1 = e.end();
  if (t0.hour() < 10)
    s.append(" ");
  s.append(QString::number(t0.hour()));
  if (t0.minute() < 10)
    s.append("0");
  s.append(QString::number(t0.minute()));
  s.append("-");
  if (t1.hour() < 10)
    s.append(" ");
  s.append(QString::number(t1.hour()));
  if (t1.minute() < 10)
    s.append("0");
  s.append(QString::number(t1.minute()));
  return s;
}

QString QualendarView::dayName(int i) {
  switch(i) {
    case 0:
    return tr("Sun");
    case 1:
    return tr("Mon");
    case 2:
    return tr("Tue");
    case 3:
    return tr("Wed");
    case 4:
    return tr("Thu");
    case 5:
    return tr("Fri");
    case 6:
    return tr("Sat");
    default:
    return QString();
  }
}

QString QualendarView::dateString(const QualendarTask &t, 
				    bool showStart, bool showDue) {
  QString s;
  if (showStart && t.hasStartDate()) {
    QDate d = t.startDate();
    s += QString::number(d.month()) + "/" + QString::number(d.day());
  }
  s += "-";
  if (showDue && t.hasDueDate()) {
    QDate d = t.dueDate();
    s += QString::number(d.month()) + "/" + QString::number(d.day());
  }
  return s;
}

QString QualendarView::getIcon(const QString &s, int h, QPixmap &pm) {
  QString key;
  pm = QualendarGlobal::getIcon(s, h, key);
  return s.right(s.length() - key.length());
}

QColor QualendarView::getDayColor(const QDate &d) {
  QColor c(255, 255, 255);
  if (d == QDate::currentDate()) 
    c = QualendarGlobal::todayColor;
  if (QualendarGlobal::colorOnBG) {
    if ((d.dayOfWeek())%7 == 0 || QualendarGlobal::holidays.contains(d)) { 
      if (d == QDate::currentDate())
	c = QualendarGlobal::todayHolidayColor;
      else
	c = QualendarGlobal::holidayColor;
    } else if ((d.dayOfWeek()) == 6) {
      if (d == QDate::currentDate())
	c = QualendarGlobal::todaySaturdayColor;
      else
	c = QualendarGlobal::saturdayColor;
    }
  }

  return c;
}

QValueList<QualendarTask> QualendarView::filterTasks(const QValueList<QualendarTask> &orig, const QMap<int, bool> &filter) {
  QValueList<QualendarTask> result;
  for (QValueListConstIterator<QualendarTask> it = orig.begin();
       it != orig.end();
       ++it) 
    if (!filter.contains(((*it).categories().count() > 0) ? 
			 (*it).categories()[0] : QualendarGlobal::unfiledCatId)) 
      result.append(*it);
  return result;
}

QValueList<QualendarTask> QualendarView::getTasks(QDate sd, QDate ed, const QMap<int, bool> &filter) {

  // ugly. 

  bool todoStart = QualendarGlobal::todoStart, 
    todoDue = QualendarGlobal::todoDue, 
    todoToday = QualendarGlobal::todoToday, 
    todoTodayAfterStart = QualendarGlobal::todoTodayAfterStart, 
    todoBetween = QualendarGlobal::todoBetween, 
    todoBetweenStart = QualendarGlobal::todoBetweenStart, 
    todoBetweenDue = QualendarGlobal::todoBetweenDue;

  const QValueList<QualendarTask> &taskList = taskDB->getTasks();

  QValueList<QualendarTask> list;
  for(uint i = 0; i < taskList.count(); i++) {
    const QualendarTask &task = taskList[i];
    QDate cD = QDate::currentDate();

    if (task.isCompleted() ||
	filter.contains((task.categories().count() > 0) ? 
		     task.categories()[0] : QualendarGlobal::unfiledCatId)) 
      continue;

    if ((todoStart && task.hasStartDate() && 
	 task.startDate() >= sd && task.startDate() <= ed)
	||
	(todoDue && task.hasDueDate() && 
	 task.dueDate() >= sd && task.dueDate() <= ed)
	||
	(todoToday && cD >= sd && cD <= ed && 
	 (!todoTodayAfterStart || !task.hasStartDate() || 
	  task.startDate() <= cD))
	||
	(todoBetween && 
	 (todoBetweenStart ?
	  task.hasStartDate() && task.startDate() <= ed :
	  !task.hasStartDate() || task.startDate() <= ed)
	 &&
	 (todoBetweenDue ?
	  task.hasDueDate() && task.dueDate() >= sd :
	  !task.hasDueDate() || task.dueDate() >= sd))
	) {
      QValueListIterator<QualendarTask> it = list.begin();
      for( ; it != list.end(); ++it) {
	if ((*it).priority() + 
	    (((*it).hasDueDate() && (*it).dueDate() <= ed) ? 0:10)
	    > 
	    task.priority() + 
	    (((task.hasDueDate() && task.dueDate() <= ed) ? 0:10))) {
	  list.insert(it, task);
	  break;
	}
      }
      if (it == list.end())
	list.append(task);
    }
  }
  return list;
}

QColor QualendarView::getTodoColor(const QualendarTask &task, QDate d) {
  return (task.hasDueDate() && task.dueDate() <= d) ? 
    QColor(255, 0, 0) : QColor(0, 0, 0);
}
QPixmap QualendarView::getTodoPixmap(const QualendarTask &task, QDate d, 
				     int h) {
  QImage im;
  if (task.hasDueDate() && task.dueDate() <= d) {
    if (todoPixmap2.isNull()) {
      im = Resource::loadPixmap("qualendar/todo2").convertToImage();
      todoPixmap2 = im.smoothScale(h/2, h);
    }
    return todoPixmap2;
  } 
  if (task.hasStartDate() && task.startDate() == d) {
    if (todoPixmap0.isNull()) {
      im = Resource::loadPixmap("qualendar/todo0").convertToImage();
      todoPixmap0 = im.smoothScale(h/2, h);
    }
    return todoPixmap0;
  } 
  if (todoPixmap1.isNull()) {
    im = Resource::loadPixmap("qualendar/todo1").convertToImage();
    todoPixmap1 = im.smoothScale(h/2, h);
  }
  return todoPixmap1;
}

