/* uses code from todo of qtopia-1.5 */

/* LinuZau ToolBox Wiki, zdbat $B$r;29M$K$7$^$7$?(B */

#include "config.h"
#ifdef DBISBOX

#include <qstring.h>
#include <qfile.h>
#include <qasciidict.h>
#include <qmessagebox.h>
#include <qpe/global.h>
#include <qpe/stringutil.h>
#include <qpe/qpeapplication.h>
#include "qualendartaskdbbox.h"

QualendarTaskDBBox::QualendarTaskDBBox() {
  load();
}

void QualendarTaskDBBox::reload() {
  taskList.clear();
  idList.clear();
  load();

  emitSignal();
}

bool QualendarTaskDBBox::save() {
  // Nothing.
  return true;
}

const QValueList<QualendarTask> &QualendarTaskDBBox::getTasks() const {
  return taskList;
}

void QualendarTaskDBBox::add(const QualendarTask &t) {
  SlZDataBase *zdb = openForWrite();
  if (!zdb) 
    return;

  CardId cardId = 0;
  bool result = editTaskBox(zdb, cardId, t);
  //  qDebug("addTask %d", (int) cardId);

  if (result) {
    QualendarTask t = toTask(zdb, cardId); 
    t.assignUid(); 
    if (zdb->checkGarbageAlert(SlZDataBase::todolistFileName()))
      emit exception("Information", 
		     "Garbage collection for ToDo performed.");
    addTaskRaw(t, cardId);

    emitSignal();
  } 

  zdb->close();
  delete zdb;
}

void QualendarTaskDBBox::edit(const QualendarTask &old, 
			      const QualendarTask &t) {
  SlZDataBase *zdb = openForWrite();
  if (!zdb) 
    return;

  CardId cardId;
  int i;

  if (!findCard(old, i, cardId, "edit")) {
    zdb->close();
    delete zdb;
    return;
  }

  bool result = editTaskBox(zdb, cardId, t);
  //  qDebug("editTask %d", (int) cardId);

  if (result) {
    taskList.remove(taskList.at(i));
    idList.remove(idList.at(i));
    QualendarTask t = toTask(zdb, cardId); 
    t.setUid(old.uid()); 
    addTaskRaw(t, cardId);
    emitSignal();
  } else {
    emit exception("Error", 
		   tr("THIS MIGHT BE A BUG.\n"
		      "Error in edit(Task).\n"
		      "cardId=%1")
		   .arg(cardId));
  }

  zdb->close();
  delete zdb;
}

void QualendarTaskDBBox::remove(const QualendarTask &t) {
  SlZDataBase *zdb = openForWrite();
  if (!zdb) 
    return;

  CardId cardId;
  int i;

  if (!findCard(t, i, cardId, "remove")) {
    zdb->close();
    delete zdb;
    return;
  }

  //  qDebug("remove Task %d", (int) cardId);
  bool result = zdb->deleteCard(&cardId);

  if (result) {
    taskList.remove(taskList.at(i));
    idList.remove(idList.at(i));

    emitSignal();
  } else {
    emit exception("Error", 
		   tr("THIS MIGHT BE A BUG.\n"
		      "Error in remove(Task).\n"
		      "cardId=%1")
		   .arg(cardId));
  }

  zdb->close();
  delete zdb;
}

void QualendarTaskDBBox::load() {
  QTime t;
  t.start();

  qDebug("QualendarTaskDBBox: opening SlZDataBase... %d", t.elapsed());

  SlZDataBase *zdb;
  zdb = new SlZDataBase(SlZDataBase::todolistFileName(),
			SlZDataBase::todolistItems(), 
			NULL, true);
  if (!zdb || !zdb->isOpened()) {
    QMessageBox::critical(0, "Can't open ToDo Database",
			  tr("Use preinstalled ToDo\n"
			     "to create the data file"));
    qApp->quit();
  }

  taskList.clear();
  int i = 0;
  for(CardId cardId = zdb->firstCard(); 
      cardId != 0;
      cardId = zdb->nextCard(cardId)) {
    QualendarTask task = toTask(zdb, cardId);
    task.assignUid(); 
    if (!addTaskRaw(task, cardId))
      break;
    i++;
  }

  qDebug("QualendarTaskDBBox: there are %d tasks", i);
  qDebug("QualendarTaskDBBox: done %d", t.elapsed() );

  zdb->close();
  delete zdb;

  dataDirty = false;
} 

// add the task to the list in memory.
bool QualendarTaskDBBox::addTaskRaw(const QualendarTask &t, CardId cardId) {
  if (t.uid() == 0 || taskList.contains(t) || 
      cardId == 0 || idList.contains(cardId) ||
      taskList.count() != idList.count()) {
    emit exception("Error", 
		   tr("THIS MIGHT BE A BUG.\n"
		      "Error in addTaskRaw.\n"
		      "uid=%1, cardId=%2, "
		      "taskList.count()=%3, idList.count()=%4")
		   .arg(t.uid()).arg(cardId)
		   .arg(taskList.count()).arg(idList.count()));
    return false;
  }

  taskList.append(t);
  idList.append(cardId);
  return true;
}


QualendarTask QualendarTaskDBBox::toTask(SlZDataBase *zdb, CardId &cardId) {
  QualendarTask todo;

  todo.setCompleted((zdb->readUcharField(ZdbTodo::Mark, cardId) 
		     == ZdbTodo::Completed) ? true : false);
  QDate dueDate = zdb->readDateField(ZdbTodo::DueDate, cardId);
  if (dueDate.isValid()) 
    todo.setDueDate(dueDate, true);
  else
    todo.setDueDate(QDate(), false);
  todo.setPriority(zdb->readUcharField(ZdbTodo::Priority, cardId));
  todo.setCategories(zdb->readCategories(cardId));
  todo.setDescription(zdb->readField(ZdbTodo::Description, cardId));
  todo.setNotes(zdb->readField(ZdbTodo::Notes, cardId));
  QDate startDate = zdb->readDateField(ZdbTodo::StartDate, cardId);
  if (startDate.isValid()) 
    todo.setStartDate(startDate, true);
  else 
    todo.setStartDate(QDate(), false);
  QDate finishDate = zdb->readDateField(ZdbTodo::FinishDate, cardId);
  if (finishDate.isValid()) 
    todo.setFinishDate(finishDate);

  return todo;
}

// edit a task data in BOX database. 
// if cardId = 0, a task is added, and cardId is set.
// returns true on success.
bool QualendarTaskDBBox::editTaskBox(SlZDataBase *zdb, 
				     CardId &cardId, const QualendarTask &t) {
  zdb->startEditCard(cardId);
  //  zdb->updateCategories(cardId, t.categories()); // COMMENTED OUT, 040116

  zdb->writeItem("MARK", (uchar) (t.isCompleted() ? 0 : 1));
  zdb->writeItem("PRTY", (uchar) t.priority());
  zdb->writeItem("TITL", t.description());
  zdb->writeItem("MEM1", t.notes());
  if (t.hasStartDate())
    zdb->writeItem("ETDY", t.startDate());
  else
    zdb->clearItem("ETDY");
  if (t.hasDueDate())
    zdb->writeItem("LTDY", t.dueDate());
  else
    zdb->clearItem("LTDY");
  if (t.isCompleted())
    zdb->writeItem("FNDY", t.finishDate());
  else
    zdb->clearItem("FNDY");

  CardId newId = cardId;
  if (!zdb->finishEditCard(&newId)) {
    emit exception("Error", 
		   tr("THIS MIGHT BE A BUG.\n"
		      "Couldn't edit a task.\n"
		      "cardId=%1, newId=%2")
		   .arg(cardId).arg(newId));
    return false;
  }

  cardId = newId;
  if (!zdb->updateCategories(cardId, t.categories(), 
			     SlZDataManager::OneCard, true))
    emit exception("Warning", 
		   tr("Couldn't set category.\n"
		      "cardId=%1")
		   .arg(cardId));
  return true;
}

bool QualendarTaskDBBox::findCard(const QualendarTask &t, 
				  int &i, CardId &cardId, 
				  QString op) {
  if (!&t) {
    emit exception("Error", 
		   tr("THIS MIGHT BE A BUG.\n"
		      "Null pointer in findCard, %1(Task).")
		   .arg(op));
    return false;
  }

  i = taskList.findIndex(t);
  if (t.uid() == 0 || i < 0 || 
      taskList.count() != idList.count()) {
    emit exception("Error", 
		   tr("THIS MIGHT BE A BUG.\n"
		      "Error in findCard, %1(Task).\n"
		      "uid=%2, taskList.count()=%3, idList.count()=%4")
		   .arg(op)
		   .arg(t.uid()).
		   arg(taskList.count()).arg(idList.count()));
    return false;
  } else {
    cardId = idList[i];
    return true;
  }
}

SlZDataBase *QualendarTaskDBBox::openForWrite() {
  SlZDataBase *zdb;

  zdb = new SlZDataBase(SlZDataBase::todolistFileName(),
			SlZDataBase::todolistItems()); 
  if (zdb) {
    if (zdb->isOpened()) 
      return zdb;
    else
      delete zdb;
  }

  emit exception("Error", 
		 tr("Couldn't open ToDo database for write.\n"
		    "Stop(and disable fastloading)\n"
		    "of other applications using SLTODO.BOX"));
  return 0;
}

#endif // DBISBOX
