#include <qlayout.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qscrollview.h>
#include <qpe/config.h>
#include <qpe/timeconversion.h>
#include <qpe/timestring.h>
#include "templatedialog.h"
#include "evententry.h"
#include "qualendarglobal.h"
#include "sysdep/qualendarcategory.h"

TemplateDialog::TemplateDialog(QDate d, Event *events) : QDialog(0, 0, true), date_(d), events_(events) {
  QVBoxLayout *l = new QVBoxLayout(this);
  lb = new QListBox(this);
  l->addWidget(lb);

  for(int i=0; i<10; i++) 
    lb->insertItem(TimeString::timeString(events_[i].start().time()) + 
		   "-" + 
		   TimeString::timeString(events_[i].end().time()) +
		   " " + events_[i].description(), i);

  QPushButton *b = new QPushButton(this);
  b->setText(tr("Edit"));
  l->addWidget(b);
  connect(b, SIGNAL(clicked()), this, SLOT(slotEdit()));
}

void TemplateDialog::slotEdit() {
  int i = lb->currentItem();
  if (i<0)
    return;
  QDialog editDlg(this, 0, true);

  QVBoxLayout vb(&editDlg);
  QScrollView sv(&editDlg, "scrollview");
  sv.setHScrollBarMode(QScrollView::AlwaysOff);
  sv.setVScrollBarMode(QScrollView::AlwaysOn);
  sv.setResizePolicy(QScrollView::AutoOneFit);
  vb.addWidget(&sv);

  Event e = events_[i];
  QDateTime dt0 = e.start(), dt1 = e.end();
  e.setStart(QDateTime(date_, dt0.time()));
  e.setEnd(e.start().addSecs(dt0.secsTo(dt1)));

  EventEntry entry(QualendarGlobal::onMonday, e, 
		   QualendarGlobal::ampm, 
		   sv.viewport(), "editor");
  sv.addChild(&entry);
  entry.setAlarmEnabled(QualendarGlobal::aPreset, 
			 QualendarGlobal::presetTime, Event::Loud );
  editDlg.setCaption(tr("Template"));
  editDlg.showMaximized();
  entry.setMaximumWidth(sv.visibleWidth());
  while(editDlg.exec()) {
    Event newEv = entry.event();
    if (newEv.hasRepeat()) {
      QMessageBox::warning(this, "",
			   tr("Repeat not allowed in a template"),
			   tr("Fix it"), 0, 0);
      continue;
    }
    if (newEv.notes().find(0xa) >= 0) {
      int i = QMessageBox::warning(this, "", 
				   tr("Notes must be 1 line(currently)"), 
				   tr("Fix it"), tr("Truncate"), 0);
      if (i == 0)
	continue;
      else
	newEv.setNotes(newEv.notes().left(newEv.notes().find(0xa)));
    }
    events_[i] = newEv;
    lb->changeItem(TimeString::timeString(newEv.start().time()) + 
		   "-" + 
		   TimeString::timeString(newEv.end().time()) +
		   " " + newEv.description(), i);

    Config config("QualendarTemplate");
    config.setGroup("Main");
    config.writeEntry(QString("Description%1").arg(i), newEv.description());
    config.writeEntry(QString("Location%1").arg(i), newEv.location());
    QString cat;
    if (newEv.categories().count() > 0)
      cat = QualendarGlobal::categories.label("Calendar", 
					      newEv.categories()[0]);
    config.writeEntry(QString("Category%1").arg(i), cat);
    config.writeEntry(QString("Start%1").arg(i), QString(TimeConversion::toISO8601(newEv.start())));
    config.writeEntry(QString("End%1").arg(i), QString(TimeConversion::toISO8601(newEv.end())));
    config.writeEntry(QString("AllDay%1").arg(i), newEv.type() == Event::AllDay);
    config.writeEntry(QString("HasAlarm%1").arg(i), newEv.hasAlarm());
    config.writeEntry(QString("AlarmTime%1").arg(i), newEv.alarmTime());
    config.writeEntry(QString("AlarmSound%1").arg(i), newEv.alarmSound() == Event::Loud ? "Loud" : "Silent");
    //    config.writeEntry(QString("Notes%1").arg(i), newEv.notes());
    break;
  }
}
