#include "tododialog.h"

#include <qaction.h>
#include <qcheckbox.h>
#include <qdialog.h>
#include <qlineedit.h>
#include <qpopupmenu.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qtoolbutton.h>

#include <qpe/config.h>
#include <qpe/qpeapplication.h>
#include <qpe/qpemenubar.h>
#include <qpe/resource.h>

#include "mymultilineedit.h"
#include "qualendarglobal.h"
#include "utils/datepicker.h"
#include "utils/fileselector.h"
#include "utils/qualendarcategorycombo.h"
#include "sysdep/qualendartask.h"
#include "sysdep/qualendarcategory.h"


// FIXME: Should we make EventEntry and TodoDialog inherit
// a common base class? 
TodoDialog::TodoDialog(const QualendarTask &t, QDate d, QWidget *parent) 
  : TodoDialogBase(parent, 0, true) {
  QPopupMenu *editMenu = new QPopupMenu(this);
  MenuBar->insertItem(tr("Edit"), editMenu);
  MenuBar->setFixedWidth(120); // FIXME
  QAction *a;
  a = new QAction(tr("Attach file"), Resource::loadPixmap("open"),
		  QString::null, 0, this, 0);
  a->addTo(editMenu);
  connect(a, SIGNAL(activated()), this, SLOT(slotFile()));

  QWidget *disp = QApplication::desktop();
  // dirty hack
  bool bigDisplay = (disp->width() * disp->height() >= 100000); 
  if (bigDisplay) {
    ToolButtonCut->setFixedSize(32, 32);
    ToolButtonCopy->setFixedSize(32, 32);
    ToolButtonPaste->setFixedSize(32, 32);
    ToolButtonFile->setFixedSize(32, 32);
  }

  connect(ToolButtonCut, SIGNAL(clicked()), this, SLOT(slotCut()));
  connect(ToolButtonCopy, SIGNAL(clicked()), this, SLOT(slotCopy()));
  connect(ToolButtonPaste, SIGNAL(clicked()), this, SLOT(slotPaste()));
  connect(ToolButtonFile, SIGNAL(clicked()), this, SLOT(slotFile()));

  QString catName;
  if (t.categories().count() == 0)
    catName = tr("Unfiled");
  else
    catName = QualendarGlobal::categories.label("Calendar", t.categories()[0]);
  comboCategory->setCurrent(catName);


  LineEditDescription->setText(t.description());
  SpinBoxPriority->setValue(t.priority());
  CheckBoxStartDate->setChecked(t.hasStartDate());
  CheckBoxDueDate->setChecked(t.hasDueDate());
  CheckBoxFinishDate->setChecked(t.isCompleted());
  editNote->setText(t.notes());

  sd = (t.hasStartDate() ? t.startDate() : d);
  dd = (t.hasDueDate() ? t.dueDate() : d);
  fd = (t.isCompleted() ? t.finishDate() : d);

  setButtonTexts();

  connect(CheckBoxStartDate, SIGNAL(toggled(bool)), 
	  this, SLOT(slotCheckBox()));
  connect(CheckBoxDueDate, SIGNAL(toggled(bool)), 
	  this, SLOT(slotCheckBox()));
  connect(CheckBoxFinishDate, SIGNAL(toggled(bool)), 
	  this, SLOT(slotCheckBox()));

  PushButtonStartDate->setAutoDefault(false);
  PushButtonDueDate->setAutoDefault(false);
  PushButtonFinishDate->setAutoDefault(false);

  connect(PushButtonStartDate, SIGNAL(clicked()), 
	  this, SLOT(slotStart()));
  connect(PushButtonDueDate, SIGNAL(clicked()), 
	  this, SLOT(slotDue()));
  connect(PushButtonFinishDate, SIGNAL(clicked()), 
	  this, SLOT(slotFinish()));
  slotCheckBox();
}

QualendarTask TodoDialog::task() {
  QualendarTask t;

  if (comboCategory->currentText() != tr("Unfiled"))
    t.setCategories(QualendarGlobal::categories.id("Calendar", comboCategory->currentText()));
  t.setDescription(LineEditDescription->text());
  t.setPriority(SpinBoxPriority->value());
  t.setStartDate(sd, CheckBoxStartDate->isChecked());
  t.setDueDate(dd, CheckBoxDueDate->isChecked());
  t.setFinishDate(fd);
  t.setCompleted(CheckBoxFinishDate->isChecked());
  t.setNotes(editNote->text());
  return t;
}

void TodoDialog::slotCheckBox() {
  PushButtonStartDate->setEnabled(CheckBoxStartDate->isChecked());
  PushButtonDueDate->setEnabled(CheckBoxDueDate->isChecked());
  PushButtonFinishDate->setEnabled(CheckBoxFinishDate->isChecked());
}

void TodoDialog::slotStart() {
  DatePickerDialog dlg(this, sd);
  dlg.exec();
  sd = dlg.date();
  setButtonTexts();
}
void TodoDialog::slotDue() {
  DatePickerDialog dlg(this, dd);
  dlg.exec();
  dd = dlg.date();
  setButtonTexts();
}
void TodoDialog::slotFinish() {
  DatePickerDialog dlg(this, fd);
  dlg.exec();
  fd = dlg.date();
  setButtonTexts();
}
QString TodoDialog::dateString(const QDate &d) {
  return 
    tr("%1/%2/%3(%4)")
    .arg(d.year()).arg(d.month()).arg(d.day())
    .arg(QObject::tr(d.dayName(d.dayOfWeek())));
}

void TodoDialog::setButtonTexts() {
  PushButtonStartDate->setText(dateString(sd));  
  PushButtonDueDate->setText(dateString(dd));  
  PushButtonFinishDate->setText(dateString(fd));  
}

void TodoDialog::slotCut()
{
  QWidget *w = focusWidget();
  if (w->inherits("QLineEdit"))
    ((QLineEdit*)w)->cut();
  if (w->inherits("QMultiLineEdit"))
    ((QMultiLineEdit*)w)->cut();
}
void TodoDialog::slotCopy()
{
  QWidget *w = focusWidget();
  if (w->inherits("QLineEdit"))
    ((QLineEdit*)w)->copy();
  if (w->inherits("QMultiLineEdit"))
    ((QMultiLineEdit*)w)->copy();
}
void TodoDialog::slotPaste()
{
  QWidget *w = focusWidget();
  if (w->inherits("QLineEdit"))
    ((QLineEdit*)w)->paste();
  if (w->inherits("QMultiLineEdit"))
    ((QMultiLineEdit*)w)->paste();
}
void TodoDialog::slotFile()
{
  QString dir;
  {
    Config config("QualendarTmp");
    config.setGroup("Main");
    dir = config.readEntry("CurrentDir", 
			   QPEApplication::documentDir().latin1());
  }
  FileSelector dlg(QDate::currentDate(), dir,
		   this, "file", true);
  dlg.showMaximized();
  if (dlg.exec() == QDialog::Accepted) {
    QString s = dlg.absName();
    editNote->insert("[[file://" + s + "]]\n");

  }
  Config config("QualendarTmp");
  config.setGroup("Main");
  config.writeEntry("CurrentDir", dlg.currentDir());
}

