#include "fileselector.h"
#include "datepicker.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <dirent.h>
#include <qlistbox.h>
#include <qtoolbutton.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qtabwidget.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qpe/timestring.h>
#include <qpe/timeconversion.h>
#include <qpe/qpeapplication.h>
#define RAM_PATH (QPEApplication::documentDir().latin1())
//#define SD_PATH "/mnt/card/Documents/"
//#define CF_PATH "/mnt/cf/Documents/"
#define SD_PATH "/mnt/card/"
#define CF_PATH "/mnt/cf/"
#define FIND_MAX 100

FileSelector::FileSelector(QDate date, QString dir, 
			   QWidget* parent, const char* name, 
			   bool modal, WFlags fl) : 
  FileSelectorBase(parent, name, modal, fl), d(date)
{
  if (dir.left(strlen(SD_PATH)) == SD_PATH) 
    ComboBoxFS->setCurrentItem(1);
  if (dir.left(strlen(CF_PATH)) == CF_PATH) 
    ComboBoxFS->setCurrentItem(2);
  connect(TabWidget, SIGNAL(currentChanged(QWidget*)), 
	  this, SLOT(redraw()));
  connect(ComboBoxFS, SIGNAL(activated(int)), 
	  this, SLOT(slotFSChanged(int)));
  connect(LineEditFind, SIGNAL(returnPressed()), this, SLOT(slotFind()));
  connect(PushButtonFind, SIGNAL(clicked()), this, SLOT(slotFind()));
  connect(ToolButtonBack, SIGNAL(clicked()), this, SLOT(slotBack()));
  connect(ToolButtonForward, SIGNAL(clicked()), this, SLOT(slotForward()));
  connect(ListBoxFile, SIGNAL(clicked(QListBoxItem*)), 
	  this, SLOT(slotClicked()));
  connect(ListBoxFile, SIGNAL(returnPressed(QListBoxItem*)), 
	  this, SLOT(slotClicked()));
  connect(PushButtonDate, SIGNAL(clicked()), 
	  this, SLOT(slotDate()));
  curDir.cd(dir);
  redraw();
}

void FileSelector::slotClicked() {
  QFileInfo fi(absName());
  if (fi.isFile()) 
    accept();
  if (fi.isDir()) {
    curDir.cd(absName());
    if (TabWidget->currentPageIndex() == 0) {
      redraw();
    } else {
      TabWidget->setCurrentPage(0);
    }
  }
}

void FileSelector::slotBack() {
  d = d.addDays(-1);
  redraw();
}
void FileSelector::slotForward() {
  d = d.addDays(1);
  redraw();
}
void FileSelector::slotDate() {
  DatePickerDialog dlg(this, d);
  dlg.exec();
  d = dlg.date();
  redraw();
}

void FileSelector::slotFSChanged(int i) {
  switch(i) {
  case 0:
    curDir.cd(RAM_PATH);
    break;
  case 1:
    curDir.cd(SD_PATH);
    break;
  case 2:
    curDir.cd(CF_PATH);
    break;
  }
  redraw();
}

void FileSelector::slotFind() {
  redraw();
}

void FileSelector::keyPressEvent(QKeyEvent *e) {
  if (e->isAutoRepeat())
    return;
  switch(e->key()) {
  case Key_Right:
    slotForward();
    break;
  case Key_Left:
    slotBack();
    break;
  case Key_Down:
    if (ListBoxFile->currentItem() < ((int)ListBoxFile->count()) - 1)
      ListBoxFile->setCurrentItem(ListBoxFile->currentItem() + 1);
    break;
  case Key_Up:
    if (ListBoxFile->currentItem() > 0)
      ListBoxFile->setCurrentItem(ListBoxFile->currentItem() - 1);
    break;
  case Key_Backspace:
    if (TabWidget->currentPageIndex() == 0 &&
	ListBoxFile->text(0) == "../") {
      ListBoxFile->setCurrentItem(0);
      slotClicked();
    }
    break;
  case Key_Escape:
    reject();
    break;
  }
}

void FileSelector::searchForFiles(time_t start, time_t end, const char* path) {
  struct dirent **dir;
  int n_files;
  if (chdir(path))
    return;
  if ((n_files = scandir(".", &dir, 0, alphasort)) < 0)
    return ;

  for (int i=0; i<n_files; i++) {
    if (dir[i]->d_name[0] != '.') {
      if (dir[i]->d_type == DT_DIR) {
	char* newPath = (char*)malloc(strlen(path)+strlen(dir[i]->d_name)+2);
	sprintf(newPath, "%s%s/", path, dir[i]->d_name);
	searchForFiles(start, end, newPath);
	free(newPath);
	chdir(path);
      } else if (dir[i]->d_type == DT_REG) {
	struct stat buf;
	stat(dir[i]->d_name, &buf);
	if (buf.st_ctime >= start && buf.st_ctime < end) {
	  char* pathName = (char*)malloc(strlen(path)+strlen(dir[i]->d_name)+1);
	  sprintf(pathName, "%s%s", path, dir[i]->d_name);
	  pathNameList.append(QString::fromUtf8(pathName));
	  QListBoxText *lbi = 
	    new QListBoxText(QString::fromUtf8(dir[i]->d_name));
	  ListBoxFile->insertItem(lbi, -1);
	}
      }
    }
    free(dir[i]);
  }
  free(dir);
}


void FileSelector::searchForFiles(QString str, const char* path) {
  struct dirent **dir;
  int n_files;
  if (str.isEmpty() || ListBoxFile->count() >= FIND_MAX)
    return;
  if (chdir(path))
    return;
  if ((n_files = scandir(".", &dir, 0, alphasort)) < 0)
    return ;

  for (int i=0; i<n_files; i++) {
    if (dir[i]->d_name[0] != '.') {
      if (dir[i]->d_type == DT_DIR) {
	char* newPath = (char*)malloc(strlen(path)+strlen(dir[i]->d_name)+2);
	sprintf(newPath, "%s%s/", path, dir[i]->d_name);
	searchForFiles(str, newPath);
	free(newPath);
	chdir(path);
      } else if (dir[i]->d_type == DT_REG) {
	QString fileName = QString::fromUtf8(dir[i]->d_name);
	if (fileName.contains(str, false) > 0 && 
	    ListBoxFile->count() < FIND_MAX) {
	  char* pathName = (char*)malloc(strlen(path)+strlen(dir[i]->d_name)+1);
	  sprintf(pathName, "%s%s", path, dir[i]->d_name);
	  pathNameList.append(QString::fromUtf8(pathName));
	  QListBoxText *lbi = 
	    new QListBoxText(QString::fromUtf8(dir[i]->d_name));
	  ListBoxFile->insertItem(lbi, -1);
	}
      }
    }
    free(dir[i]);
  }
  free(dir);
}


void FileSelector::searchForFiles() {
  TextLabelCurrentDir->setText(curDir.absPath());
  QStringList dirs = curDir.entryList("*", QDir::Dirs | QDir::Hidden);
  dirs.remove(".");
  for(int i=0; i<dirs.count(); i++) {
    pathNameList.append(curDir.absFilePath(dirs[i]));
    dirs[i].append("/");
  }
  ListBoxFile->insertStringList(dirs);

  QStringList files = curDir.entryList("*", QDir::Files);
  for(int i=0; i<files.count(); i++) 
    pathNameList.append(curDir.absFilePath(files[i]));
  ListBoxFile->insertStringList(files);
}


void FileSelector::redraw() {
  ListBoxFile->clear();
  pathNameList.clear();

  int i = TabWidget->currentPageIndex();
  if (i == 0) {
    searchForFiles();
  } 
  if (i == 1) {
    QString str = LineEditFind->text();
    searchForFiles(str, RAM_PATH);
    searchForFiles(str, SD_PATH);
    searchForFiles(str, CF_PATH);
  } 
  if (i == 2) {
    PushButtonDate->setText(TimeString::dateString(d));
    //  time_t start = TimeConversion::toUTC(QDateTime(d, QTime(0, 0, 0)));
    //  time_t end = TimeConversion::toUTC(QDateTime(d.addDays(1), QTime(0, 0, 0)));
    time_t start = TimeConversion::toUTC(QDateTime(d, QTime(0, 0, 0)))
      + 32400;
    time_t end = TimeConversion::toUTC(QDateTime(d.addDays(1), QTime(0, 0, 0)))
      + 32400;
    searchForFiles(start, end, RAM_PATH);
    searchForFiles(start, end, SD_PATH);
    searchForFiles(start, end, CF_PATH);
  } 
}

QString FileSelector::absName() {
  QListBoxItem *lbi = ListBoxFile->item(ListBoxFile->currentItem());
  if (lbi == NULL)
    return QString("");
  int i = ListBoxFile->index(lbi);
  return pathNameList[i];
}
QString FileSelector::currentDir() {
  return curDir.absPath();
}
