#ifndef QUALENDARDAY_H
#define QUALENDARDAY_H

#include <qvaluelist.h>
#include <qarray.h>
#include <qdatetime.h>

#include "qualendarviewex.h"

class QualendarItem;

class QAction;
class QListBox;
class QListBoxItem;
class QLabel;
class QScrollBar;
class QualendarDayHeader;

class MyScrollView : public QHBox {
 Q_OBJECT
 public:
  MyScrollView(QWidget *parent = 0, const char *name = 0); 
  QualendarMainArea *viewport();
  void setContentsHeight(int);
  int currentY();
  void setCurrentY(int);
  void setLineStep(int);
 private slots:
  void barMoved();
 private:
  QScrollBar *bar_;
  QualendarMainArea *viewport_;
  int lastValue;
};


/**
 * Day view.
 *
 * Variables from QualendarGlobal: ampm, fontSize 
 *
 * Config entries: 
 *
 * File Qualendar, Group Day
 * FontSize, 
 * TodoColorByCategory(not yet), TodoColorOver(not yet)
 **/
class QualendarDay : public QualendarView {
	Q_OBJECT
 public:
  QualendarDay(QWidget *parent, const char *name, 
	       QArray<QualendarDB *>, QPopupMenu *);
  QString viewName() const;
  QString trViewName() const;
  QPixmap icon() const;
  QDate currentDate() const;
  void selectedDateTime(QDateTime &, QDateTime &) const;
  void setupMenu();

 public slots:
  void setFilter(QMap<int, bool>, bool draw = true);
  void setDate(const QDate &, bool mustDraw);
  void setDate(const QDate &);
  void loadConfig(); 
  void redraw();
  void paintMainArea();

 protected:
  void resizeEvent(QResizeEvent *);
  void keyPressEvent(QKeyEvent *);

  void relayout();
  void draw(QRect, bool useDirty = false);
  QualendarItem selectedItem();

 private slots:
  void slotViewOption(QAction *);
  void slotMouseEvent(QMouseEvent *);
 private:
  void getEvents();
  void openItemRect(const ItemRect &);

  QualendarDayHeader *header;
  MyScrollView *sv;
  QualendarMainArea *mainArea;
  QPopupMenu *menu_;
  QAction *actionShowBackgroundEvent;

  QValueList<QualendarTask> tl;
  uint headerWidth, headerHeight, cellWidth, cellHeight;
  uint cellHeaderHeight;

  uint numHol, numTodo, numAllDay;
  QString holDesc;

  // configs
  int fontSize;
  // display a task on its start/due day.
  bool todoStart, todoDue;
  // set the color of a task according to its category.
  bool todoColorByCategory; 
  // make the color of a task red if the due date is over. 
  bool todoColorOver; 
  int rowHeight, minimalEventHeight;
  QColor selectedColor;

  QPixmap upPixmap, downPixmap, repeatPixmap;

  QDate date_;
  QMap<int, bool> filter_;
  DateRect dr;
  QValueList<ItemRect> itemRects0;
  QValueList<ItemRect> itemRects;
  uint currentItem;

  bool selectionMode;
  int selectionStart, selectionEnd, selectionOrig;
};

#endif // QUALENDARDAY_H
