#include "qualendardayheader.h"
#include "../qualendarglobal.h"
#include "../utils/datepicker.h"
#include <qlayout.h>
#include <qpushbutton.h>
#include <qhbuttongroup.h>
#include <qobjectlist.h>
#include <qtoolbutton.h>
#include <qpe/resource.h>

QualendarDayHeader::QualendarDayHeader(QWidget *parent) 
  : QualendarViewHeader(parent) { 
  setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Maximum));
  QHBoxLayout *l = new QHBoxLayout(this);
  l->setMargin(0);
  l->setSpacing(0);

  btnPicker = new QPushButton(this);
  l->addWidget(btnPicker);
  btnPicker->setFocusPolicy(NoFocus);
  connect(btnPicker, SIGNAL(clicked()), 
	  this, SLOT(slotDatePicker()));

  QToolButton *tb;

  tb = new QToolButton(this);
  tb->setFocusPolicy(NoFocus);
  tb->setPixmap(Resource::loadPixmap("fastback"));
  tb->setAutoRaise(true);
  l->addWidget(tb);
  connect(tb, SIGNAL(pressed()), this, SLOT(slotFB()));

  tb = new QToolButton(this);
  tb->setFocusPolicy(NoFocus);
  tb->setPixmap(Resource::loadPixmap("back"));
  tb->setAutoRaise(true);
  l->addWidget(tb);
  connect(tb, SIGNAL(pressed()), this, SLOT(slotB()));

  bgDays = new QHButtonGroup(this);
  bgDays->setAlignment(Qt::AlignCenter);

  QObjectList *layouts = bgDays->queryList("QLayout");
  QObjectListIt it(*layouts);
  for( ; it.current(); ++it) {
    QLayout *l = (QLayout *)it.current();
    l->setMargin(0);
    l->setSpacing(0);
  }
  delete layouts;

  bgDays->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, 
				    QSizePolicy::Expanding));
  l->addWidget(bgDays);
  bgDays->setExclusive(true);
  bgDays->setFrameStyle(QFrame::NoFrame);
  connect(bgDays, SIGNAL(clicked(int)), 
	  this, SLOT(slotDayButton(int)));

  QPushButton *pb;
  for(int i=0; i<7; i++) {
    pb = new QPushButton(bgDays);
    pb->setToggleButton(true);
    pb->setFlat(true);
    pb->setFocusPolicy(NoFocus);
  }
  setLabels();

  tb = new QToolButton(this);
  tb->setFocusPolicy(NoFocus);
  tb->setPixmap(Resource::loadPixmap("forward"));
  tb->setAutoRaise(true);
  l->addWidget(tb);
  connect(tb, SIGNAL(pressed()), this, SLOT(slotF()));

  tb = new QToolButton(this);
  tb->setFocusPolicy(NoFocus);
  tb->setPixmap(Resource::loadPixmap("fastforward"));
  tb->setAutoRaise(true);
  l->addWidget(tb);
  connect(tb, SIGNAL(pressed()), this, SLOT(slotFF()));
}

void QualendarDayHeader::setDate(const QDate &_date) {
  date_ = _date; // thus, sbYear and cbWeek won't emit signals.
  btnPicker->setText(QString("%1/%2/%3")
		     .arg(QString::number(date_.year()).right(2))
		     .arg(date_.month())
		     .arg(date_.day()));
  int i = date_.dayOfWeek();
  lastButton = QualendarGlobal::onMonday ? (i-1)%7 : i%7;
  bgDays->setButton(lastButton);
}

void QualendarDayHeader::setFontSize(int h) {
  setFixedHeight(h * 2);
  btnPicker->setFixedWidth(h * 5);

  for(int i = 0; i < 7; i++) {
    QButton *pb = bgDays->find(i);
    pb->setFixedSize(h * 3 / 2, h * 3 / 2);
  }
}

void QualendarDayHeader::slotDatePicker() {
  DatePickerDialog dlg(this, date_);
  dlg.exec();
  //  setDate(QDate(dlg.year(), dlg.month(), dlg.day()));
  date_ = dlg.date();
  emit dateChanged(date_);
}
void QualendarDayHeader::slotDayButton(int i) {
  date_ = date_.addDays(i - lastButton);
  emit dateChanged(date_);
}
void QualendarDayHeader::setLabels() {
  QString s[7];
  s[0] = tr("S");
  s[1] = tr("M");
  s[2] = tr("T");
  s[3] = tr("W");
  s[4] = tr("Th");
  s[5] = tr("F");
  s[6] = tr("Sa");
  for(int i=0; i<7; i++) {
    QButton *b = bgDays->find(i);
    b->setText(s[QualendarGlobal::onMonday ? (i+1)%7 : i]);
  }
}

void QualendarDayHeader::slotFB() {
  setDate(date_.addDays(-7));
  emitSignal();
}
void QualendarDayHeader::slotB() {
  setDate(date_.addDays(-1));
  emitSignal();
}
void QualendarDayHeader::slotF() {
  setDate(date_.addDays(1));
  emitSignal();
}
void QualendarDayHeader::slotFF() {
  setDate(date_.addDays(7));
  emitSignal();
}

void QualendarDayHeader::emitSignal() {
  qDebug("QualendarDayHeader::emitSignal");
  emit dateChanged(date_);
}

