#ifndef QUALENDARMONTH_H
#define QUALENDARMONTH_H

#include <qvaluelist.h>
#include <qarray.h>
#include <qdatetime.h>

#include "qualendarviewex.h"

class QAction;
class QActionGroup;
class QListBox;
class QListBoxItem;

/**
 * Month view.
 *
 *
 * Config entries: 
 *
 * File Qualendar, Group Month
 * FontSize, ShowOtherMonth, TodoStart, TodoDue, 
 * TodoColorByCategory(not yet), TodoColorOver(not yet)
 **/
class QualendarMonth : public QualendarViewEx {
	Q_OBJECT
 public:
	QualendarMonth(QWidget *parent, const char *name, 
		       QArray<QualendarDB *>, QPopupMenu *);
  QString viewName() const;
  QString trViewName() const;
  QPixmap icon() const;
  void setupMenu();

 protected:
  void keyPressEvent(QKeyEvent *);

  QDate dateStart() const;
  QDate dateEnd() const;
  QDate tableDateStart() const;
  QDate tableDateEnd() const;
  QDate eventsDateStart() const;
  QDate eventsDateEnd() const;
  int stepRight() {return 1;}
  int stepDown() {return 7;}

  void setupDateRects();
  void loadConfig(); 
  void relayout();
  void draw(QRect, bool useDirty = false);
  QualendarItem selectedItem();

 private slots:
  void slotViewOption(QAction *);
  void slotDayBox(QListBoxItem*);
 private:
  // returns the new 'iy'.
  int layoutEvents(DateRect &, const QFontMetrics &, int, int);
  int layoutTasks(DateRect &, const QFontMetrics &, int, int);

  QPopupMenu *menu_;
  QActionGroup *ag;
  QListBox *dayBox;
  int headerWidth, headerHeight, cellWidth, cellHeight;
  int cellHeaderHeight;
  bool desc, graph, both2, iconOnly;
  QValueList<QualendarItem> dayItems;
  QValueList<QualendarTask> tasks;

  int fontSize, dateFontSize;
  bool showOtherMonths;
  // display a task on its start/due day.
  bool todoColorByCategory; 
  // make the color of a task red if the due date is over. 
  bool todoColorOver; 
  bool returnDayBox;
  bool eventsFirst, dayBoxEventsFirst;

  QPixmap downPixmap;
};

#endif // QUALENDARMONTH_H
