#include "qualendarmonthheader.h"
#include <qlayout.h>
#include <qtoolbutton.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qpe/resource.h>

QualendarMonthHeader::QualendarMonthHeader(QWidget *parent) 
  : QualendarViewHeader(parent), ignoreSignals(false) {
  QHBoxLayout *l = new QHBoxLayout(this);

  sbYear = new QSpinBox(1970, 2100, 1, this); // FIXME
  sbYear->setFocusPolicy(NoFocus);
  sbYear->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, 
				    QSizePolicy::Expanding));
  l->addWidget(sbYear);
  connect(sbYear, SIGNAL(valueChanged(int)),
  				this, SLOT(slotYearChange(int)));

  cbMonth = new QComboBox(this);
  cbMonth->setFocusPolicy(NoFocus);
  for(int i=1; i <= 12; i++) 
	cbMonth->insertItem(QString::number(i));
  l->addWidget(cbMonth);
  connect(cbMonth, SIGNAL(activated(int)),
  				this, SLOT(slotMonthChange(int)));

  QToolButton *tb;

  tb = new QToolButton(this);
  tb->setFocusPolicy(NoFocus);
  tb->setPixmap(Resource::loadPixmap("start"));
  tb->setAutoRaise(true);
  l->addWidget(tb);
  connect(tb, SIGNAL(pressed()), this, SLOT(slotFB()));

  tb = new QToolButton(this);
  tb->setFocusPolicy(NoFocus);
  tb->setPixmap(Resource::loadPixmap("back"));
  tb->setAutoRaise(true);
  l->addWidget(tb);
  connect(tb, SIGNAL(pressed()), this, SLOT(slotB()));

  tb = new QToolButton(this);
  tb->setFocusPolicy(NoFocus);
  tb->setPixmap(Resource::loadPixmap("forward"));
  tb->setAutoRaise(true);
  l->addWidget(tb);
  connect(tb, SIGNAL(pressed()), this, SLOT(slotF()));

  tb = new QToolButton(this);
  tb->setFocusPolicy(NoFocus);
  tb->setPixmap(Resource::loadPixmap("finish"));
  tb->setAutoRaise(true);
  l->addWidget(tb);
  connect(tb, SIGNAL(pressed()), this, SLOT(slotFF()));
}

void QualendarMonthHeader::setDate(const QDate &_date) {
  qDebug("MonthHeader: %d %d %d", _date.year(), _date.month(), _date.day());
  date_ = _date; 
  ignoreSignals = true;
  sbYear->setValue(date_.year());
  cbMonth->setCurrentItem(date_.month() - 1);
  ignoreSignals = false;
}

void QualendarMonthHeader::setFontSize(int h) {
  setFixedHeight(h * 2);
  sbYear->setFixedWidth(h * 5);
  cbMonth->setFixedWidth(h * 5);
}

void QualendarMonthHeader::slotFB() {
  int yr = date_.year();
  // just taking date_.day() should be OK...
  int dy = QMIN(QDate(yr, 1, 1).daysInMonth(), date_.day()); 
  date_.setYMD(yr, 1, dy);
  ignoreSignals = true;
  cbMonth->setCurrentItem(0);
  ignoreSignals = false;
  emitSignal();
}
void QualendarMonthHeader::slotB() {
  int yr = date_.year(), mt = date_.month(), dy = date_.day();
  int y, m;
  if (mt == 1) {
    y = yr - 1;
    m = 12;
  } else {
    y = yr;
    m = mt - 1;
  }
  // tricky...
  date_.setYMD(y, m, QMIN(dy, QDate(y, m, 1).daysInMonth()));
  ignoreSignals = true;
  sbYear->setValue(y);
  cbMonth->setCurrentItem(m-1);
  ignoreSignals = false;
  emitSignal();
}
void QualendarMonthHeader::slotF() {
  int yr = date_.year(), mt = date_.month(), dy = date_.day();
  int y, m;
  if (mt == 12) {
    y = yr + 1;
    m = 1;
  } else {
    y = yr;
    m = mt + 1;
  }
  // tricky...
  date_.setYMD(y, m, QMIN(dy, QDate(y, m, 1).daysInMonth()));
  ignoreSignals = true;
  sbYear->setValue(y);
  cbMonth->setCurrentItem(m-1);
  ignoreSignals = false;
  emitSignal();
}
void QualendarMonthHeader::slotFF() {
  int yr = date_.year();
  // just taking date_.day() should be OK...
  int dy = QMIN(QDate(yr, 12, 1).daysInMonth(), date_.day());
  date_.setYMD(yr, 12, dy);
  ignoreSignals = true;
  cbMonth->setCurrentItem(11);
  ignoreSignals = false;
  emitSignal();
}

void QualendarMonthHeader::slotYearChange(int yr) {
  if (ignoreSignals)
    return;
  int mt = date_.month();
  int dy = QMIN(QDate(yr, mt, 1).daysInMonth(), date_.day());
  date_.setYMD(yr, mt, dy);
  emitSignal();
}
void QualendarMonthHeader::slotMonthChange(int i) {
  if (ignoreSignals)
    return;
  int mt = i + 1;
  int yr = date_.year();
  int dy = QMIN(QDate(yr, mt, 1).daysInMonth(), date_.day());
  date_.setYMD(yr, mt, dy);
  emitSignal();
}

void QualendarMonthHeader::emitSignal() {
  qDebug("QualendarMonthHeader::emitSignal");

  sbYear->setEnabled(false);
  cbMonth->setEnabled(false);

  emit dateChanged(date_);

  sbYear->setEnabled(true);
  cbMonth->setEnabled(true);
}

