#ifndef QUALENDARTODOVIEW_H
#define QUALENDARTODOVIEW_H

#include <qvaluelist.h>
#include <qarray.h>
#include <qdatetime.h>
#include <qlistview.h>

#include "../qualendarview.h"

class QListViewItem;
class QualendarItem;
class QAction;

class QualendarTodoListView : public QListView {
  Q_OBJECT
 public:
  QualendarTodoListView(QWidget *);
 protected:
  void keyPressEvent(QKeyEvent *);
 signals:
  void spacePressed(QListViewItem *);
};

/**
 * Todo view.
 **/
class QualendarTodoView : public QualendarView {
  Q_OBJECT
 public:
  QualendarTodoView(QWidget *parent, const char *name, 
		    QArray<QualendarDB *>, QPopupMenu *);
  ~QualendarTodoView();
  QString viewName() const;
  QString trViewName() const;
  QPixmap icon() const;
  QDate currentDate() const;
  void selectedDateTime(QDateTime &, QDateTime &) const;
  void setupMenu();

 public slots:
  void setFilter(QMap<int, bool>, bool draw = true);
  void setDate(const QDate &, bool mustDraw = true);
  void loadConfig(); 
  void redraw();

 protected:
  void keyPressEvent(QKeyEvent *);
  QualendarItem selectedItem();

 private slots:
  void slotTaskSelected(QListViewItem *);
  void slotTaskDone(QListViewItem *);
  void slotClicked(QListViewItem *, const QPoint &, int);

 private:
  QListView *lvResult;
  QPopupMenu *menu_;
  QAction *actionShowDone, *actionShowNotDone;

  // configs
  int fontSize;

  QDate date_;
  QMap<int, bool> filter_;

  QMap<QString, int> nameToColumn;
  int savedUid;
};

#endif // QUALENDARTODOVIEW_H
