/***************************************************************************
                          datebook2viewutil.cpp  -  description
                             -------------------
    begin                : 2003.9.1
    copyright            : (C) 2003 by ichitokumei
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qpainter.h>

#include "qualendarviewutil.h"

#include "../debug.h"

void QualendarMainArea::paintEvent(QPaintEvent *e) {
  DRAW_DEBUG(qDebug("mainArea: paintEvent, rect = %d %d %d %d",
		    e->rect().x(), e->rect().y(), e->rect().width(), e->rect().height()));
  emit paintMe(e->rect());
}

void QualendarMainArea::showEvent(QShowEvent *e) {
  DRAW_DEBUG(qDebug("mainArea: showEvent, spontaneous = %d", e->spontaneous()));
}
void QualendarMainArea::mousePressEvent(QMouseEvent *e) {
  emit mouseEvent(e);
}
void QualendarMainArea::mouseMoveEvent(QMouseEvent *e) {
  emit mouseEvent(e);
}
void QualendarMainArea::mouseReleaseEvent(QMouseEvent *e) {
  emit mouseEvent(e);
}

/////////////////////////////////////////////////////
DateRect::DateRect() : QRect() {
	dirty_ = true;
	tooMany = false;
}

void DateRect::setDirty(bool d) {dirty_ = d;}
bool DateRect::dirty() const { return dirty_;}
void DateRect::clear() { tooMany = false; events_.clear(); tasks_.clear(); hide_.resize(0); }
void DateRect::setEventHidden(int i, bool b) {hide_[i] = b;}
bool DateRect::eventHidden(int i) const {return hide_[i];}
void DateRect::setDate(QDate d) { date_ = d; }
QDate DateRect::date() const {return date_;}
void DateRect::setBackground(QPixmap pm) { back_ = pm; }
const QPixmap &DateRect::background() const { return back_; }


// let's hope that this is not time-critical.
void DateRect::addEvent(const EffectiveEvent &e) {
  int n = events_.count();
  hide_.resize(n + 1);
  QValueListIterator<EffectiveEvent> it;
  for(it = events_.begin(); it != events_.end(); ++it) {
    if((*it).start().hour() > e.start().hour() ||
       ((*it).start().hour() == e.start().hour() &&
	(*it).start().minute() > e.start().minute())) {
      events_.insert(it, e);
      break;
    }
  }
  if (it == events_.end())
    events_.append(e);
}

const QValueList<EffectiveEvent> &DateRect::events() const {return events_;}

void DateRect::addTask(const QualendarTask &task) {
  QValueListIterator<QualendarTask> it;
  for(it = tasks_.begin(); it != tasks_.end(); ++it) {
    if ((*it).priority() + 
	(((*it).hasDueDate() && (*it).dueDate() <= date_) ? 0:10)
	> 
	task.priority() + 
	(((task.hasDueDate() && task.dueDate() <= date_) ? 0:10))) {
      tasks_.insert(it, task);
      break;
    }
  }
  if (it == tasks_.end())
    tasks_.append(task);
}
const QValueList<QualendarTask> &DateRect::tasks() const {return tasks_;}

DateRectArray::DateRectArray() {a.setAutoDelete(true);}
void DateRectArray::resize(int x, int y) {
  w_ = x; h_ = y; a.resize(x*y);
  for(uint i=0; i<a.size(); i++)
	a.insert(i, new DateRect);
}
DateRect &DateRectArray::operator[](int i) { return *(a[i]); }
const DateRect &DateRectArray::operator[](int i) const {return *(a[i]);}
DateRect &DateRectArray::operator()(int i, int j) { return *(a[j*w_ + i]);}
const DateRect &DateRectArray::operator()(int i, int j) const { return *(a[j*w_ + i]);}
int DateRectArray::w() const {return w_;}
int DateRectArray::h() const {return h_;}

/////////////////////////////////////////////////////
ItemRect::ItemRect() : QRect() {
  dirty = true;
  x_ = 1;
  y_ = 1;
  fixedRect = false;
}

void ItemRect::setItem(const EffectiveEvent &e) {
  item_.setItem(e);
}
void ItemRect::setItem(const QualendarTask &t) {
  item_.setItem(t);
}
void ItemRect::setItem(const QString &s) {
  item_.clear();
  s_ = s;
}
const QualendarItem &ItemRect::item() const {
  return item_;
}

/*
const EffectiveEvent &ItemRect::effectiveEvent() const {
  if (item_.type == QualendarItem::Event) 
    return *(item_.event);
  else
    return *((const EffectiveEvent *)0);
}
const QualendarTask &ItemRect::task() const {
  if (item_.type == QualendarItem::Task) 
    return *(item_.task);
  else
    return *((const QualendarTask *)0);
}
*/

const QString &ItemRect::text() const {
  return s_;
}
void ItemRect::addText(const QFontMetrics &fm, const QString &s, QColor c, int _x, int _y) {
  if (_x >= 0 && _y >= 0) {
    x_ = _x;
    y_ = _y;
  }
  QStringList strList = QStringList::split(0x0a, s, true);

  for(int i=0; i<strList.count(); i++) {
    QString t = strList[i];
    textXY.append(QPoint(x_, y_));
    textColor.append(c);
    texts.append(t);
    if (!fixedRect) {
      QRect r = *this | QRect(x() + x_, y() + y_, fm.width(t), fm.height());
      setRect(r.x(), r.y(), r.width() + 1, r.height() + 1);
    } 
    if (i == strList.count() - 1)
      x_ += fm.width(t);
    else{
      x_ = 0;
      y_ += fm.height();
    }
    if (x_ >= width() || y_ >= height())
      break;
  }
}
void ItemRect::addPixmap(const QPixmap &pm, int _x, int _y) {
  if (_x >= 0 && _y >= 0) {
    x_ = _x;
    y_ = _y;
  }
  pixmapXY.append(QPoint(x_, y_));
  pixmaps.append(pm);
  QRect r = *this | QRect(x() + x_, y() + y_, pm.width(), pm.height());
  setRect(r.x(), r.y(), r.width(), r.height());
  x_ += pm.width();
}

void ItemRect::setFixedRect(bool b) {
  fixedRect = b;
}

void ItemRect::clip(const QRect &r) {
  QRect r1 = r & *this;
  setRect(r1.x(), r1.y(), r1.width(), r1.height());
}
void ItemRect::draw(QPainter &p, int yy) {
  QFontMetrics fm = p.fontMetrics();
  p.save();
  p.setClipping(true);
  p.setClipRegion(p.clipRegion() & QRect(x(), y() + yy, width(), height()));
  int ht = fm.height();
  for(uint i=0; i < texts.count(); i++) {
    p.setPen(textColor[i]);
    p.drawText(this->topLeft() + textXY[i] + QPoint(0, yy + ht - 1), texts[i]);
  }
  for(uint i=0; i < pixmaps.count(); i++) {
    p.drawPixmap(this->topLeft() + pixmapXY[i] + QPoint(0, yy), pixmaps[i]);
  }
  p.restore();
}

int ItemRect::currentX() {
  return x_;
}
int ItemRect::currentY() {
  return y_;
}
