/***************************************************************************
                          qualendarviewutil.h  -  description
                             -------------------
    begin                : 2003.9.1
    copyright            : (C) 2003 by ichitokumei
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef QUALENDARVIEWUTIL_H
#define QUALENDARVIEWUTIL_H

#include <qpixmap.h>
#include <qvector.h>
#include <qbitarray.h>
#include "../qualendarview.h"
#include "../qualendaritem.h"
#include "../sysdep/qualendardb.h"

/**
 *  The class for a widget to display the main content(calendar).
 *
 * The widget simply passes QPaintEvent and QMouseEvent
 * by signals paintMe() and mouseEvent().
 * On paintEvent(), background is not cleared.
 */

class QualendarMainArea : public QWidget {
  Q_OBJECT
 public:
  QualendarMainArea(QWidget *parent) : QWidget(parent) {
    setBackgroundMode(NoBackground);
    setFocusPolicy(StrongFocus);
  }
 protected slots:
  void paintEvent(QPaintEvent *);
  void showEvent(QShowEvent *);
  void mousePressEvent(QMouseEvent *);
  void mouseMoveEvent(QMouseEvent *);
  void mouseReleaseEvent(QMouseEvent *);
 signals:
  void paintMe(QRect);
  void mouseEvent(QMouseEvent *);
};


class DateRect : public QRect {
 public:
  DateRect();
  void setDirty(bool d);
  bool dirty() const;
  void clear();
  void setEventHidden(int i, bool b);
  bool eventHidden(int i) const;
  void setDate(QDate d);
  QDate date() const;
  void setBackground(QPixmap pm);
  const QPixmap &background() const;
  void addEvent(const EffectiveEvent &);
  const QValueList<EffectiveEvent> &events() const;
  void addTask(const QualendarTask &);
  const QValueList<QualendarTask> &tasks() const;
  bool tooMany;

 private:
  QValueList<EffectiveEvent> events_;
  QValueList<QualendarTask> tasks_;
  QDate date_;
  bool dirty_;
  QPixmap back_;
  QBitArray hide_;
};

class DateRectArray {
 public:
  DateRectArray();
  void resize(int x, int y);
  DateRect &operator[](int i);
  const DateRect &operator[](int i) const;
  DateRect &operator()(int i, int j);
  const DateRect &operator()(int i, int j) const;
  int w() const;
  int h() const;
 private:
  QVector<DateRect> a;
  int w_, h_;
};

class ItemRect : public QRect {
 public:
  ItemRect();
  void setItem(const EffectiveEvent &);
  void setItem(const QualendarTask &);
  void setItem(const QString &);
  const QualendarItem &item() const;
  const QString &text() const;
  void addText(const QFontMetrics &, const QString &, QColor c, int x = -1, int y = -1);
  void addPixmap(const QPixmap &, int x = -1, int y = -1);
  void setFixedRect(bool);
  void clip(const QRect &);
  void draw(QPainter &, int y = 0);
  int currentX();
  int currentY();

 private:
  QualendarItem item_;
  QString s_;

  QValueList<QPoint> pixmapXY, textXY;
  QValueList<QColor> textColor;
  QValueList<QPixmap> pixmaps;
  QValueList<QString> texts;
  bool dirty;
  int x_, y_;
  bool fixedRect;
};

#endif // QUALENDARVIEWUTIL_H
