#ifndef QUALENDARWEEK_H
#define QUALENDARWEEK_H

#include <qvaluelist.h>
#include <qarray.h>
#include <qdatetime.h>

#include "qualendarviewex.h"

class QualendarItem;

class QAction;
class QListBox;
class QListBoxItem;

/**
 * Week view.
 *
 * Variables from QualendarGlobal: onMonday, fontSize 
 *
 * Config entries: 
 *
 * File Qualendar, Group Week
 * FontSize, TodoStart, TodoDue, 
 * TodoColorByCategory(not yet), TodoColorOver(not yet)
 **/
class QualendarWeek : public QualendarViewEx {
	Q_OBJECT
 public:
  QualendarWeek(QWidget *parent, const char *name, 
		QArray<QualendarDB *>, QPopupMenu *);
  QString viewName() const;
  QString trViewName() const;
  QPixmap icon() const;
  void setupMenu() {};

 protected:
  void myMouseEvent(QMouseEvent *);
  QDate dateStart() const;
  QDate dateEnd() const;
  int stepRight() {return 7;}
  int stepDown() {return 1;}

  void loadConfig(); 
  void setupDateRects();
  void relayout();
  void draw(QRect r, bool useDirty = false);
  QualendarItem selectedItem();

 private slots:
  void slotViewOption(QAction *);
 private:
  // returns the new 'iy'.
  int layoutEvents(DateRect &, const QFontMetrics &, int, int);
  int layoutTasks(DateRect &, const QFontMetrics &, int, int);

  int headerWidth, headerHeight, cellWidth, cellHeight;
  int cellHeaderHeight;

  int fontSize, dateFontSize;
  // set the color of a task according to its category.
  bool todoColorByCategory; 
  // make the color of a task red if the due date is over. 
  bool todoColorOver; 
  bool eventsFirst;

  QPixmap downPixmap;
};

#endif // QUALENDARWEEK_H
