#include "qualendarweekheader.h"
#include "../qualendarglobal.h"
#include <qlayout.h>
#include <qtoolbutton.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qpe/resource.h>
#include "../qualendarcalendar.h"

QualendarWeekHeader::QualendarWeekHeader(QWidget *parent) 
  : QualendarViewHeader(parent), ignoreSignals(true) { 
  QHBoxLayout *l = new QHBoxLayout(this);

  sbYear = new QSpinBox(1970, 2100, 1, this); // FIXME
  sbYear->setFocusPolicy(NoFocus);
  sbYear->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, 
				    QSizePolicy::Expanding));
  l->addWidget(sbYear);
  connect(sbYear, SIGNAL(valueChanged(int)),
	  this, SLOT(slotYearChange(int)));
  sbWeek = new QSpinBox(0, 55, 1, this); 
  sbWeek->setFocusPolicy(NoFocus);
  sbWeek->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, 
				    QSizePolicy::Expanding));
  l->addWidget(sbWeek);
  connect(sbWeek, SIGNAL(valueChanged(int)),
	  this, SLOT(slotWeekChange(int)));

  QToolButton *tb;

  tb = new QToolButton(this);
  tb->setFocusPolicy(NoFocus);
  tb->setPixmap(Resource::loadPixmap("back"));
  tb->setAutoRaise(true);
  l->addWidget(tb);
  connect(tb, SIGNAL(pressed()), this, SLOT(slotB()));

  labelDates = new QLabel(this);
  l->addWidget(labelDates);

  tb = new QToolButton(this);
  tb->setFocusPolicy(NoFocus);
  tb->setPixmap(Resource::loadPixmap("forward"));
  tb->setAutoRaise(true);
  l->addWidget(tb);
  connect(tb, SIGNAL(pressed()), this, SLOT(slotF()));
}

void QualendarWeekHeader::setDate(const QDate &_date) {
  date_ = _date; 

  int w;
  calcWeek(_date, w);
  ignoreSignals = true;
  sbYear->setValue(date_.year());
  sbWeek->setValue(w);
  ignoreSignals = false;
  setLabel();
}

void QualendarWeekHeader::setFontSize(int h) {
  setFixedHeight(h * 2);
  sbYear->setFixedWidth(h * 5);
  sbWeek->setFixedWidth(h * 5);
  labelDates->setFixedWidth(h * 10);
}

void QualendarWeekHeader::slotB() {
  setDate(date_.addDays(-7));
  emitSignal();
}
void QualendarWeekHeader::slotF() {
  setDate(date_.addDays(7));
  emitSignal();
}

void QualendarWeekHeader::calcWeek(const QDate &d, int &wk) {
  int y = d.year();
  QDate d0, 
    d1 = QualendarCalendar::firstDayOfWeek(d, QualendarGlobal::onMonday); 
  d0 = QualendarCalendar::firstDayOfWeek(QDate(y, 1, 1), QualendarGlobal::onMonday);
  wk = d0.daysTo(d1)/7 + 1;
}

void QualendarWeekHeader::setLabel() {
  QDate
    d1 = QualendarCalendar::firstDayOfWeek(date_, QualendarGlobal::onMonday), 
    d2 = d1.addDays(6);
  labelDates->setText(tr("%1.%2(%3)-%4.%5(%6)")
		      .arg(d1.month())
		      .arg(d1.day())
		      .arg(QObject::tr(d1.dayName(d1.dayOfWeek())))
		      .arg(d2.month())
		      .arg(d2.day())
		      .arg(QObject::tr(d2.dayName(d2.dayOfWeek()))));
}

void QualendarWeekHeader::slotYearChange(int yr) {
  if (ignoreSignals)
    return;
  if (!date_.isValid())
    return;
  QDate d0 = QualendarCalendar::firstDayOfWeek(QDate(yr, 1, 1), 
				      QualendarGlobal::onMonday);
  int wk = sbWeek->value();
  QDate d = d0.addDays((wk-1)*7 + 
		       QualendarCalendar::firstDayOfWeek(date_, 
						QualendarGlobal::onMonday)
		       .daysTo(date_));
  while (d.year() < yr)
    d = d.addDays(7);
  while (d.year() > yr) 
    d = d.addDays(-7);
  setDate(d);
  emitSignal();
}

void QualendarWeekHeader::slotWeekChange(int wk) {
  if (ignoreSignals)
    return;
  if (!date_.isValid())
    return;

  int yr = sbYear->value();
  QDate d0 = QualendarCalendar::firstDayOfWeek(QDate(yr, 1, 1), 
				      QualendarGlobal::onMonday);
  QDate d = d0.addDays((wk-1)*7 + 
		       QualendarCalendar::firstDayOfWeek(date_, 
						QualendarGlobal::onMonday)
		       .daysTo(date_));
  setDate(d);
  emitSignal();
}

void QualendarWeekHeader::emitSignal() {
  qDebug("QualendarWeekHeader::emitSignal");

  sbYear->setEnabled(false);
  sbWeek->setEnabled(false);

  emit dateChanged(date_);

  sbYear->setEnabled(true);
  sbWeek->setEnabled(true);
}

