/**********************************************************************
** Copyright (C) 2000-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qtopia Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include "qualendarsettings.h"

#include <qpe/qpeapplication.h>
#include <qpe/config.h>
#include <qpe/qpemessagebox.h>

#include <qspinbox.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qfontinfo.h>
#include <qfontdatabase.h>
#include <qlayout.h>
#include <qpixmap.h>
#include <qlineedit.h>
#include <qlistview.h>
#include <qfile.h>
#include <stdlib.h>
#include <qtextstream.h>
#include <qpushbutton.h>

//  from http://www.pointer.biz/app/sl/
#include <qtextcodec.h>
#define _T(str) QTextCodec::codecForLocale()->toUnicode(str)

struct {
  char* name;
  char* title;
  int defaultRGB;
} colorData[N_COLORS] = {
  { "Todo", "Todo", 0xe0e0e0 }, 
  { "OtherMonth", "¾η", 0xc0c0c0 }, 
  { "AllDay", "ͽ", 0x00ff00 },
  { "Event", "̾ͽ", 0x00ffff },
  { "Alarm", "顼դͽ", 0xff00ff },
  { "Today", "", 0xffffc0 },
  { "Holiday", "", 0xffd0d0 },
  { "Saturday", "", 0xd0d0ff }, 
  { "TodayHoliday", " & ", 0xffd090 },
  { "TodaySaturday", " & ", 0xd0d0c0 }
};

QualendarSettings::QualendarSettings( QWidget *parent,
                                    const char *name, bool modal, WFlags fl )
    : QualendarSettingsBase( parent, name, modal, fl )
{
  init();

  QObject::connect( qApp, SIGNAL( clockChanged( bool ) ),
		    this, SLOT( slotChangeClock( bool ) ) );
  connect(ListBoxViewer, SIGNAL(highlighted(int)), 
	  this, SLOT(slotViewerSelected(int)));
  connect(LineEditViewer, SIGNAL(textChanged(const QString&)),
	  this, SLOT(slotViewerEntered(const QString&)));

  connect(ButtonEditColor, SIGNAL(clicked()), 
		  this, SLOT(slotEditColor()));

  connect(SpinBoxFilterNumber, SIGNAL(valueChanged(int)), 
	  this, SLOT(slotFilterNumberChange(int)));
  connect(ButtonNewCategory, SIGNAL(clicked()), 
		  this, SLOT(slotNewCategory()));
  connect(ButtonDeleteCategory, SIGNAL(clicked()), 
		  this, SLOT(slotDeleteCategory()));
  connect(ButtonEditCategory, SIGNAL(clicked()), 
		  this, SLOT(slotEditCategory()));

  connect(ButtonNewIcon, SIGNAL(clicked()), 
		  this, SLOT(slotNewIcon()));
  connect(ButtonDeleteIcon, SIGNAL(clicked()), 
		  this, SLOT(slotDeleteIcon()));
  connect(ButtonEditIcon, SIGNAL(clicked()), 
		  this, SLOT(slotEditIcon()));

  connect(ButtonNewBackground, SIGNAL(clicked()), 
		  this, SLOT(slotNewBackground()));
  connect(ButtonDeleteBackground, SIGNAL(clicked()), 
		  this, SLOT(slotDeleteBackground()));
  connect(ButtonEditBackground, SIGNAL(clicked()), 
		  this, SLOT(slotEditBackground()));

  connect(ButtonViewsUp, SIGNAL(clicked()), 
	  this, SLOT(slotViewsUp()));
  connect(ButtonViewsDown, SIGNAL(clicked()), 
	  this, SLOT(slotViewsDown()));
  connect(ButtonTodoUp, SIGNAL(clicked()), 
	  this, SLOT(slotTodoUp()));
  connect(ButtonTodoDown, SIGNAL(clicked()), 
	  this, SLOT(slotTodoDown()));
}

void QualendarSettings::save()
{
  Config config( "qpe" );
  Config configDB( "DateBook" );
  configDB.setGroup( "Main" );
  configDB.writeEntry("startviewtime", startTime());
  configDB.writeEntry("alarmpreset", alarmPreset());
  configDB.writeEntry("presettime",presetTime());
  {
    Config config("Qualendar");
    config.setGroup("Main");
    config.writeEntry("SpaceReturn", spaceReturn());
    config.writeEntry("CancelToClose", cancelToClose());
    config.writeEntry("DayViewToday", dayViewToday());
    config.writeEntry("FontSize", fontSize(ComboBoxFont));

    config.setGroup("Month");
    config.writeEntry("FontSize", fontSize(ComboBoxMonthFont));
    config.writeEntry("DateFontSize", fontSize(ComboBoxMonthDateFont));
    config.writeEntry("ShowOtherMonths", showOtherMonths());
    config.writeEntry("JumpToItem", jumpToItem());
    config.writeEntry("ReturnDayBox", returnDayBox());
    config.writeEntry("TodoStart", todoMonthStart());
    config.writeEntry("TodoDue", todoMonthDue());
    config.writeEntry("EventsFirst", 
		      CheckBoxMonthEventsFirst->isChecked());
    config.writeEntry("DayBoxEventsFirst", 
		      CheckBoxMonthDayBoxEventsFirst->isChecked());

    config.setGroup("Week");
    config.writeEntry("FontSize", fontSize(ComboBoxWeekFont));
    config.writeEntry("DateFontSize", fontSize(ComboBoxWeekDateFont));
    config.writeEntry("TodoStart", todoWeekStart());
    config.writeEntry("TodoDue", todoWeekDue());
    config.writeEntry("EventsFirst", 
		      CheckBoxWeekEventsFirst->isChecked());

    config.setGroup("Day");
    config.writeEntry("FontSize", fontSize(ComboBoxDayFont));
    config.writeEntry("RowHeight", dayRowHeight());
    config.writeEntry("MinimalEventHeight", minimalEventHeight());

    config.setGroup("Event");
    config.writeEntry("FontSize", fontSize(ComboBoxEventFont));
    config.setGroup("Find");
    config.writeEntry("FontSize", fontSize(ComboBoxFindFont));
    config.setGroup("TodoView");
    config.writeEntry("FontSize", fontSize(ComboBoxTodoViewFont));
  }
  {
    Config config("Qualendar");
    config.setGroup("Todo");
    config.writeEntry("Show", CheckBoxShowTodo->isChecked());
    config.writeEntry("Start", todoStart());
    config.writeEntry("Due", todoDue());
    config.writeEntry("Between", todoBetween());
    config.writeEntry("BetweenStart", todoBetweenStart());
    config.writeEntry("BetweenDue", todoBetweenDue());
    config.writeEntry("Today", todoToday());
    config.writeEntry("TodayAfterStart", todoTodayAfterStart());
    config.writeEntry("WeekShowStart", todoWeekShowStart());
    config.writeEntry("WeekShowDue", todoWeekShowDue());
    config.writeEntry("DayShowStart", todoDayShowStart());
    config.writeEntry("DayShowDue", todoDayShowDue());
  }
  {
    Config config("Qualendar");
    config.setGroup("Color");
    QColor c;
	for(int i=0; i<N_COLORS; i++) {
	  c = barColors[i];
	  config.writeEntry(colorData[i].name, 
						(c.red() << 16) + (c.green() << 8) + c.blue());
	}
    config.writeEntry("ColorOnBG", cbColorOnBG->isChecked());
  }
  {
    Config config("Qualendar");
    config.setGroup("Viewers");
    config.writeEntry(ListBoxViewer->text(0), viewers[0]);
    config.writeEntry(ListBoxViewer->text(1), viewers[1]);
    config.writeEntry(ListBoxViewer->text(2), viewers[2]);
  }

  {
    Config config("Qualendar");
    config.setGroup("Filter");
    filterName[SpinBoxFilterNumber->value()] = LineEditFilter->text();
    for (uint i=0; i<10; i++) 
      config.writeEntry(QString("FilterName%1").arg(i), 
			filterName[i]);
  }

  {
	QFile colorFile((QString(getenv("HOME")) + 
					 "/Settings/QualendarColor.txt"));
	colorFile.open(IO_WriteOnly);
	if (colorFile.isOpen()) {
	  QTextStream s(&colorFile);
	  s.setCodec(QTextCodec::codecForName("utf8"));
	  QListViewItem *item = ListViewCategory->firstChild();
	  while(item) {
		QString catName = item->text(0);
		s << catName << "=";
		QColor c(0, 0, 0);
		if (colors.contains(catName))
		  c = colors[catName];
		s << ((c.red() << 16) + (c.green() << 8) + c.blue()) << endl; 
		item = item->nextSibling();
	  }
	}
	colorFile.close();
  }

  {
	QFile iconFile((QString(getenv("HOME")) + "/Settings/QualendarIcon.txt"));
	iconFile.open(IO_WriteOnly);
	if (iconFile.isOpen()) {
	  QTextStream s(&iconFile);
	  s.setCodec(QTextCodec::codecForName("utf8"));

	  IconFileNames::Iterator it;
	  for(it = icons.begin(); it != icons.end(); ++it) {
		s << it.key() << "=" << it.data() << endl;
	  }
	}
	iconFile.close();
  }

  {
	QFile backgroundFile((QString(getenv("HOME")) + "/Settings/QualendarBackground.txt"));
	backgroundFile.open(IO_WriteOnly);
	if (backgroundFile.isOpen()) {
	  QTextStream s(&backgroundFile);
	  s.setCodec(QTextCodec::codecForName("utf8"));

	  BackgroundFileNames::Iterator it;
	  for(it = backgrounds.begin(); it != backgrounds.end(); ++it) {
		s << it.key() << "=" << it.data() << endl;
	  }
	}
	backgroundFile.close();
  }

  {
	QFile filterFile((QString(getenv("HOME")) 
					  + "/Settings/QualendarFilter.txt"));
	filterFile.open(IO_WriteOnly);
	if (filterFile.isOpen()) {
	  QTextStream s(&filterFile);
	  s.setCodec(QTextCodec::codecForName("utf8"));
	  QListViewItem *item = ListViewCategory->firstChild();
	  while(item) {
		QString catName = item->text(0);
		if (filter[Day].contains(catName))
		  s << catName << "[Day]=" << filter[Day][catName] << endl;
		if (filter[Week].contains(catName))
		  s << catName << "[Week]=" << filter[Week][catName] << endl;
		if (filter[Month].contains(catName))
		  s << catName << "[Month]=" << filter[Month][catName] << endl;
		if (filter[Find].contains(catName))
		  s << catName << "[Find]=" << filter[Find][catName] << endl;
		if (filter[Todo].contains(catName))
		  s << catName << "[Todo]=" << filter[Todo][catName] << endl;
		item = item->nextSibling();
	  }
	  filterFile.close();
	}
  }

  {
    Config config("Qualendar");
    config.setGroup("ViewStack");
    bool used = true;
    uint j = 0;
    for(uint i=0; i < ListBoxViews->count(); i++) {
      QString s = "Empty";
      if (used) {
	if ((s = ListBoxViews->text(i)) == "----") {
	  used = false;
	  continue;
	}
      }
      config.writeEntry("View" + QString::number(j), s);
      j++;
    }
  }
  {
    Config config("Qualendar");
    config.setGroup("TodoView");
    bool used = true;
    uint j = 0;
    for(uint i=0; i < ListBoxTodoColumns->count(); i++) {
      QString s = "Empty";
      if (used) {
	if ((s = ListBoxTodoColumns->text(i)) == "----") {
	  used = false;
	  continue;
	}
      }
      config.writeEntry("Column" + QString::number(j), s);
      j++;
    }
  }
}

QualendarSettings::~QualendarSettings()
{
}

void QualendarSettings::setStartTime( int newStartViewTime )
{
    if ( ampm ) {
	if ( newStartViewTime >= 12 ) {
	    newStartViewTime %= 12;
	    if ( newStartViewTime == 0 )
		newStartViewTime = 12;
	    spinStart->setSuffix( tr(":00 PM") );
	}
	else if ( newStartViewTime == 0 ) {
	    newStartViewTime = 12;
	    spinStart->setSuffix( tr(":00 AM") );
	}
	oldtime = newStartViewTime;
    }
    spinStart->setValue( newStartViewTime );
}

int QualendarSettings::startTime() const
{
    int returnMe = spinStart->value();
    if ( ampm ) {
	if ( returnMe != 12 && spinStart->suffix().contains(tr("PM"), FALSE) )
	    returnMe += 12;
	else if (returnMe == 12 && spinStart->suffix().contains(tr("AM"), TRUE))
	    returnMe = 0;
    }
    return returnMe;
}


void QualendarSettings::setAlarmPreset( bool bAlarm, int presetTime )
{
    chkAlarmPreset->setChecked( bAlarm );
    if ( presetTime >=5 )
	spinPreset->setValue( presetTime );
}

bool QualendarSettings::alarmPreset() const
{
    return chkAlarmPreset->isChecked();
}

int QualendarSettings::presetTime() const
{
    return spinPreset->value();
}

int QualendarSettings::fontSize(QComboBox* cb) const
{
  int fs = cb->currentText().toInt();
  return (fs > 0) ? fs : -1;
}

int QualendarSettings::dayRowHeight() const
{
  return SpinBoxDayRowHeight->value();
}

int QualendarSettings::minimalEventHeight() const
{
  return SpinBoxMinimalEventHeight->value();
}

void QualendarSettings::slot12Hour( int i )
{
    if ( ampm ) {
	if ( spinStart->suffix().contains( tr("AM"), FALSE ) ) {
	    if ( oldtime == 12 && i == 11 || oldtime == 11 && i == 12 )
		spinStart->setSuffix( tr(":00 PM") );
	} else {
	    if ( oldtime == 12 && i == 11 || oldtime == 11 && i == 12 )
		spinStart->setSuffix( tr(":00 AM") );
	}
	oldtime = i;
    }
}

void QualendarSettings::init()
{
  { 
    Config config( "qpe" );
    config.setGroup("Time");
    ampm = config.readBoolEntry( "AMPM", TRUE );

    if ( ampm ) {
	spinStart->setMinValue( 1 );
	spinStart->setMaxValue( 12 );
	spinStart->setValue( 12 );
	spinStart->setSuffix( tr(":00 AM") );
	oldtime = 12;
    } else {
	spinStart->setMinValue( 0 );
	spinStart->setMaxValue( 23 );
	spinStart->setSuffix( tr(":00") );
    }
  }

  {
    Config config("DateBook");
    config.setGroup("Main");
    setStartTime(config.readNumEntry("startviewtime", 8));
    setAlarmPreset(config.readBoolEntry("alarmpreset"), 
		   config.readNumEntry("presettime"));
    
  }

  {
    Config config("Qualendar");
    config.setGroup("Main");

    setSpaceReturn(config.readBoolEntry("SpaceReturn"));
    setCancelToClose(config.readBoolEntry("CancelToClose"));
    setDayViewToday(config.readBoolEntry("DayViewToday"));

    int fs = config.readNumEntry("FontSize", -1);
    setFontSize(ComboBoxFont, fs); 
    config.setGroup("Month");
    setShowOtherMonths(config.readBoolEntry("ShowOtherMonths", false));
    setJumpToItem(config.readBoolEntry("JumpToItem", true));
    setReturnDayBox(config.readBoolEntry("ReturnDayBox", false));
    fs = config.readNumEntry("FontSize", -1);
    setFontSize(ComboBoxMonthFont, fs); 
    fs = config.readNumEntry("DateFontSize", -1);
    setFontSize(ComboBoxMonthDateFont, fs); 
    setTodoMonthStart(config.readBoolEntry("TodoStart", true));
    setTodoMonthDue(config.readBoolEntry("TodoDue", true));
    CheckBoxMonthEventsFirst
      ->setChecked(config.readBoolEntry("EventsFirst", false));
    CheckBoxMonthDayBoxEventsFirst
      ->setChecked(config.readBoolEntry("DayBoxEventsFirst", false));

    config.setGroup("Week");
    setTodoWeekStart(config.readBoolEntry("TodoStart", true));
    setTodoWeekDue(config.readBoolEntry("TodoDue", true));
    fs = config.readNumEntry("FontSize", -1);
    setFontSize(ComboBoxWeekFont, fs); 
    fs = config.readNumEntry("DateFontSize", -1);
    setFontSize(ComboBoxWeekDateFont, fs); 
    CheckBoxWeekEventsFirst
      ->setChecked(config.readBoolEntry("EventsFirst", false));

    config.setGroup("Day");
    fs = config.readNumEntry("FontSize", -1);
    setFontSize(ComboBoxDayFont, fs); 
    fs = config.readNumEntry("RowHeight", SpinBoxDayRowHeight->minValue());
    SpinBoxDayRowHeight->setValue(fs); 
    fs = config.readNumEntry("MinimalEventHeight", 60);
    SpinBoxMinimalEventHeight->setValue(fs); 

    config.setGroup("Event");
    fs = config.readNumEntry("FontSize", -1);
    setFontSize(ComboBoxEventFont, fs); 

    config.setGroup("Find");
    fs = config.readNumEntry("FontSize", -1);
    setFontSize(ComboBoxFindFont, fs); 

    config.setGroup("TodoView");
    fs = config.readNumEntry("FontSize", -1);
    setFontSize(ComboBoxTodoViewFont, fs); 
  }
  {
    Config config("Qualendar");
    config.setGroup("Todo");
    CheckBoxShowTodo->setChecked(config.readBoolEntry("Show", true));
    setTodoStart(config.readBoolEntry("Start", true));
    setTodoDue(config.readBoolEntry("Due", true));
    setTodoBetween(config.readBoolEntry("Between", true));
    setTodoBetweenStart(config.readBoolEntry("BetweenStart", true));
    setTodoBetweenDue(config.readBoolEntry("BetweenDue", true));
    setTodoToday(config.readBoolEntry("Today", true));
    setTodoTodayAfterStart(config.readBoolEntry("TodayAfterStart", true));
    setTodoWeekShowStart(config.readBoolEntry("WeekShowStart", true));
    setTodoWeekShowDue(config.readBoolEntry("WeekShowDue", true));
    setTodoDayShowStart(config.readBoolEntry("DayShowStart", true));
    setTodoDayShowDue(config.readBoolEntry("DayShowDue", true));
  }

  {
    Config config("Qualendar");
    config.setGroup("Color");
    bool b = config.readBoolEntry("ColorOnBG", true);
    cbColorOnBG->setChecked(b);
    int col;
	QListViewItem *lvi;
	QPixmap pm(16, 16);

	for(int i=0; i<N_COLORS; i++) {
	  lvi = new QListViewItem(ListViewColor);
	  lvi->setText(0, _T(colorData[i].title));
	  col = config.readNumEntry(colorData[i].name, 
								colorData[i].defaultRGB);
	  barColors[i] = QColor(col >> 16, (col >> 8) & 0xff, col & 0xff);
	  pm.fill(barColors[i]);
	  lvi->setPixmap(1, pm);
	  ListViewColor->insertItem(lvi);
	}
  }
  {
    Config config("Qualendar");
    config.setGroup("Viewers");
    viewers.clear();
    viewers.append(config.readEntry(ListBoxViewer->text(0), "textedit"));
    viewers.append(config.readEntry(ListBoxViewer->text(1), "zimager"));
    viewers.append(config.readEntry(ListBoxViewer->text(2), "netfront3"));
    ListBoxViewer->setSelected(0, true);
    LineEditViewer->setText(viewers[0]);
    viewerLast = 0;
  }

  {
    Config config("Qualendar");
    config.setGroup("Filter");
    for (uint i=0; i<10; i++) 
      filterName[i] = config.readEntry(QString("FilterName%1").arg(i), 
				       QString::number(i));
    LineEditFilter->setText(filterName[0]);
    currentFilter = 0;
  }

  QRegExp re("=[^=]*$");

  colors.clear();
  QFile colorFile((QString(getenv("HOME")) + "/Settings/QualendarColor.txt"));
  colorFile.open(IO_ReadOnly);
  if (colorFile.isOpen()) {
	QTextStream s(&colorFile);
	s.setCodec(QTextCodec::codecForName("utf8"));
	while(!s.atEnd()) {
	  QString str = s.readLine();
	  int i;
	  if ((i = re.find(str, 0)) > 0) {
		int col = str.right(str.length() - (i+1)).toInt();
		colors.insert(str.left(i), 
					  QColor(col >> 16, (col >> 8) & 0xff, col & 0xff));
	  }
	}
	colorFile.close();
  }

  icons.clear();
  QFile iconFile((QString(getenv("HOME")) + "/Settings/QualendarIcon.txt"));
  iconFile.open(IO_ReadOnly);
  if (iconFile.isOpen()) {
	QTextStream s(&iconFile);
	s.setCodec(QTextCodec::codecForName("utf8"));
	while(!s.atEnd()) {
	  QString str = s.readLine();
	  int i;
	  if ((i = re.find(str, 0)) > 0) {
		icons.insert(str.left(i), 
					 str.right(str.length() - (i+1)));
	  }
	}
	iconFile.close();
  }

  backgrounds.clear();
  QFile backgroundFile((QString(getenv("HOME")) + "/Settings/QualendarBackground.txt"));
  backgroundFile.open(IO_ReadOnly);
  if (backgroundFile.isOpen()) {
	QTextStream s(&backgroundFile);
	s.setCodec(QTextCodec::codecForName("utf8"));
	while(!s.atEnd()) {
	  QString str = s.readLine();
	  int i;
	  if ((i = re.find(str, 0)) > 0) {
		backgrounds.insert(str.left(i), 
					 str.right(str.length() - (i+1)));
	  }
	}
	backgroundFile.close();
  }

  for(int i=0; i<N_VIEWS; i++)
    filter[i].clear();

  QFile filterFile((QString(getenv("HOME")) 
					+ "/Settings/QualendarFilter.txt"));
  filterFile.open(IO_ReadOnly);
  QRegExp re1("\\[[a-zA-Z]*\\]=[^=]*$");
  if (filterFile.isOpen()) {
	QTextStream s(&filterFile);
	s.setCodec(QTextCodec::codecForName("utf8"));
	while(!s.atEnd()) {
	  QString str = s.readLine();
	  int i;
	  if ((i = re1.find(str, 0)) > 0) {
		int j = re.find(str, 0);
		QString viewName = str.mid(i+1, (j-1)-(i+1));
		QString filterString = str.right(str.length() - (j+1)); 
		int filterNumber = -1;
		if (viewName == "Day") 
		  filterNumber = Day;
		else if (viewName == "Week") 
		  filterNumber = Week;
		else if (viewName == "Month") 
		  filterNumber = Month;
		else if (viewName == "Find") 
		  filterNumber = Find;
		else if (viewName == "Todo") 
		  filterNumber = Todo;
		if (filterNumber >= 0)
		  filter[filterNumber].insert(str.left(i), filterString);
	  }
	}
	filterFile.close();
  }

  {
    QualendarCategories cat;
    cat.load();
    categories = cat.labels("Calendar");
	int i;
	for(i=0; i<categories.count(); i++) {
	  if (categories[i] == QObject::tr("Unfiled"))
		break;
	}
    if (i == categories.count())
	  categories.prepend(QObject::tr("Unfiled"));

    for(int i=0; i<categories.count(); i++) {
	  QListViewItem *lvi = new QListViewItem(ListViewCategory);
	  QString s = categories[i];
	  lvi->setText(0, s);
	  if (colors.contains(s)) {
		QPixmap pm(16, 16);
		pm.fill(colors[s]);
		lvi->setPixmap(1, pm);
	  }
	  for(int j=0; j<N_VIEWS; j++) 
		if (filter[j].contains(s))
		  lvi->setText(2+j, filter[j][s]);

      ListViewCategory->insertItem(lvi);
	}

    for(IconFileNames::Iterator it = icons.begin(); it != icons.end(); ++it) {
	  QListViewItem *lvi = new QListViewItem(ListViewIcon);
	  lvi->setText(0, it.key());
	  lvi->setText(1, it.data());
      ListViewIcon->insertItem(lvi);
	}
    for(BackgroundFileNames::Iterator it = backgrounds.begin(); it != backgrounds.end(); ++it) {
	  QListViewItem *lvi = new QListViewItem(ListViewBackground);
	  lvi->setText(0, it.key());
	  lvi->setText(1, it.data());
      ListViewBackground->insertItem(lvi);
	}
  }

  {
    Config config("Qualendar");
    config.setGroup("ViewStack");

    QStringList sl, sl0;
    sl.append("Month");
    sl.append("Week");
    sl.append("Day");
    sl.append("Year");
    sl.append("Find");
    sl.append("Todo");

    sl0 = sl;

    QStringList sl1;
    for(uint i = 0; i < 6; i++) {
      QString s = config.readEntry("View" + QString::number(i), sl0[i]);
      int j;
      if ((j = sl.findIndex(s)) >= 0) {
	sl1.append(s);
	sl.remove(s);
      }
    }
    sl1.append("----");
    for(uint i = 0; i < sl.count(); i++)
      sl1.append(sl[i]);

    ListBoxViews->insertStringList(sl1);
  }

  {
    Config config("Qualendar");
    config.setGroup("TodoView");

    QStringList sl, sl0;
    sl.append("Done");
    sl.append("Priority");
    sl.append("Description");
    sl.append("Start");
    sl.append("Due");
    sl.append("Finish");

    sl0 = sl;

    QStringList sl1;
    for(uint i = 0; i < 6; i++) {
      QString s = config.readEntry("Column" + QString::number(i), sl0[i]);
      int j;
      if ((j = sl.findIndex(s)) >= 0) {
	sl1.append(s);
	sl.remove(s);
      }
    }
    sl1.append("----");
    for(uint i = 0; i < sl.count(); i++)
      sl1.append(sl[i]);

    ListBoxTodoColumns->insertStringList(sl1);
  }
}

void QualendarSettings::setFontSize( QComboBox *cb, int fs) {
    QFontDatabase fdb;
    QValueList<int> sz = fdb.pointSizes(font().family());
    for(int i=0; i<sz.count(); i++) {
	int s = sz[i]/10; // FIXME
	cb->insertItem(QString::number(s));
	if (fs == s) 
	    cb->setCurrentItem(i);
    }
}

void QualendarSettings::slotChangeClock( bool whichClock )
{
    int saveMe;
    saveMe = spinStart->value();
    if ( ampm && spinStart->suffix().contains( tr("AM"), FALSE ) ) {
	if ( saveMe == 12 )
	    saveMe = 0;
    } else if ( ampm && spinStart->suffix().contains( tr("PM"), FALSE )  ) {
	if ( saveMe != 12 )
	    saveMe += 12;
    }
    ampm = whichClock;
    init();
    setStartTime( saveMe );
}

void QualendarSettings::slotViewerSelected(int index) {
  viewers[viewerLast] = LineEditViewer->text();
  viewerLast = index;
  LineEditViewer->setText(viewers[index]);
}
void QualendarSettings::slotViewerEntered(const QString &s) {
  viewers[viewerLast] = s;
}


void QualendarSettings::slotEditColor() {
  QListViewItem* i = ListViewColor->selectedItem();
  if (!i)
	return;

  int n = 0;
  for(n=0; n<N_COLORS; n++) {
	if (_T(colorData[n].title) == i->text(0))
	  break;
  }
  EditColorDialog dlg(_T(colorData[n].title), barColors[n]);
  dlg.showMaximized();
  if (dlg.exec() == QDialog::Accepted) {
	barColors[n] = dlg.color();
	QPixmap pm(16, 16);
	pm.fill(barColors[n]);
	i->setPixmap(1, pm);
  }
}

EditCategoryDialog::EditCategoryDialog(QString _name, QColor _color, 
				       QString _day, QString _week, QString _month, 
				       QString _find, QString _todo) 
  : QDialog(0, 0, true), name_(_name), color_(_color) {
  QVBoxLayout *l0 = new QVBoxLayout(this);
  QHBoxLayout *hl = new QHBoxLayout(l0);
  QLabel *label;

  label = new QLabel("Category", this);
  hl->addWidget(label, 0, 0);
  QLineEdit *le = new QLineEdit(_name, this);
  le->setReadOnly(true);
  hl->addWidget(le, 0, 1);

  btnColor = new QPushButton("Edit color", this);
  l0->addWidget(btnColor);
  btnColor->setFixedSize(64, 64);
  QPixmap pm(64, 64);
  pm.fill(color_);
  btnColor->setPixmap(pm);
  connect(btnColor, SIGNAL(clicked()), 
	  this, SLOT(slotColor()));

  label = new QLabel(_T("ե륿(Ȥ˥å)"), this);
  l0->addWidget(label);

  QGridLayout *l = new QGridLayout(l0, 10 + 1, 5);
  label = new QLabel("Day", this);
  l->addWidget(label, 0, 0);
  for(int i=0; i<10; i++) {
    cb[0][i] = new QCheckBox(this);
    l->addWidget(cb[0][i], 0, i+1);
    if (_day.contains(QString::number(i)))
      cb[0][i]->setChecked(true);
  }

  label = new QLabel("Week", this);
  l->addWidget(label, 1, 0);
  for(int i=0; i<10; i++) {
    cb[1][i] = new QCheckBox(this);
    l->addWidget(cb[1][i], 1, i+1);
    if (_week.contains(QString::number(i)))
      cb[1][i]->setChecked(true);
  }

  label = new QLabel("Month", this);
  l->addWidget(label, 2, 0);
  for(int i=0; i<10; i++) {
    cb[2][i] = new QCheckBox(this);
    l->addWidget(cb[2][i], 2, i+1);
    if (_month.contains(QString::number(i)))
      cb[2][i]->setChecked(true);
  }

  label = new QLabel("Find", this);
  l->addWidget(label, 3, 0);
  for(int i=0; i<10; i++) {
    cb[3][i] = new QCheckBox(this);
    l->addWidget(cb[3][i], 3, i+1);
    if (_find.contains(QString::number(i)))
      cb[3][i]->setChecked(true);
  }

  label = new QLabel("Todo", this);
  l->addWidget(label, 4, 0);
  for(int i=0; i<10; i++) {
    cb[4][i] = new QCheckBox(this);
    l->addWidget(cb[4][i], 4, i+1);
    if (_todo.contains(QString::number(i)))
      cb[4][i]->setChecked(true);
  }
}


QColor EditCategoryDialog::color() {
  return color_;
}
QString EditCategoryDialog::day() { return filter(Day);}
QString EditCategoryDialog::week() { return filter(Week);}
QString EditCategoryDialog::month() { return filter(Month);}
QString EditCategoryDialog::find() { return filter(Find);}
QString EditCategoryDialog::todo() { return filter(Todo);}

QString EditCategoryDialog::filter(int n) {
  QString tmp;
  for(int i=0; i<10; i++)
    if (cb[n][i]->isChecked())
      tmp += QString::number(i);
  return tmp;
}

void EditCategoryDialog::slotColor() {
  EditColorDialog dlg(name_, color_);
  dlg.showMaximized();
  if (dlg.exec() == QDialog::Accepted) {
    color_ = dlg.color();
    QPixmap pm(64, 64);
    pm.fill(color_);
    btnColor->setPixmap(pm);
  }
}


class NewCategoryDialog : public QDialog {
public:
  NewCategoryDialog()
	: QDialog(0, 0, true) {
	QHBoxLayout *l = new QHBoxLayout(this);
	QLabel *label;
	label = new QLabel("Category", this);
	l->addWidget(label);
	editCategory = new QLineEdit(this);
	l->addWidget(editCategory);
  }
  QString category() { return editCategory->text();}
private:
  QLineEdit *editCategory;
};

void QualendarSettings::slotFilterNumberChange(int i) {
  filterName[currentFilter] = LineEditFilter->text();
  currentFilter = i;
  LineEditFilter->setText(filterName[i]);
}

void QualendarSettings::slotNewCategory() {
  NewCategoryDialog dlg;
  if (dlg.exec() == QDialog::Accepted) {
	QualendarCategories cat;
    cat.load();
	if (cat.addCategory("Calendar", dlg.category())) {
	  categories.append(dlg.category());
	  QListViewItem *lvi = new QListViewItem(ListViewCategory);
	  lvi->setText(0, dlg.category());

      ListViewCategory->insertItem(lvi);
	  cat.save();
	}
  }
}
void QualendarSettings::slotDeleteCategory() {
  QListViewItem* i = ListViewCategory->selectedItem();
  if (!i)
	return;

  if (!QPEMessageBox::confirmDelete(this, "Dtbk2Util", i->text(0)))
	return;
  categories.remove(i->text(0));
  QualendarCategories cat;
  cat.load();
  cat.removeCategory("Calendar", i->text(0));
  cat.save();
  ListViewCategory->takeItem(i);
}

void QualendarSettings::slotEditCategory() {
  QListViewItem* i = ListViewCategory->selectedItem();
  if (!i)
	return;

  QString name = i->text(0);
  QColor c(0, 0, 0);
  if (colors.contains(name))
	c = colors[name];
  QString d, w, m, f, t;
  if (filter[0].contains(name))
	d = filter[0][name];
  if (filter[1].contains(name))
	w = filter[1][name];
  if (filter[2].contains(name))
	m = filter[2][name];
  if (filter[3].contains(name))
	f = filter[3][name];
  if (filter[4].contains(name))
	t = filter[4][name];

  EditCategoryDialog dlg(name, c, d, w, m, f, t);
  dlg.showMaximized();
  if (dlg.exec() == QDialog::Accepted) {
	colors.remove(name);
	for(uint j = 0; j < N_VIEWS; j++) 
	  filter[j].remove(name);

	colors.insert(name, dlg.color());
	filter[0].insert(name, dlg.day());
	filter[1].insert(name, dlg.week());
	filter[2].insert(name, dlg.month());
	filter[3].insert(name, dlg.find());
	filter[4].insert(name, dlg.todo());

	QPixmap pm(16, 16);
	pm.fill(colors[name]);
	i->setPixmap(1, pm);

	for(uint j = 0; j < 5; j++) 
	  i->setText(j+2, filter[j][name]);
  }
}

void QualendarSettings::slotNewIcon() {
  QListViewItem *lvi = new QListViewItem(ListViewIcon);
  lvi->setText(0, "key");
  lvi->setText(1, "icon");
  ListViewIcon->insertItem(lvi);
}
void QualendarSettings::slotDeleteIcon() {
  QListViewItem* i = ListViewIcon->selectedItem();
  if (!i)
	return;

  if (!QPEMessageBox::confirmDelete(this, "Dtbk2Util", 
				    i->text(0) + "/" + i->text(1)))
    return;
  icons.remove(i->text(0));
  ListViewIcon->takeItem(i);
}

class EditIconDialog : public QDialog {
public:
  EditIconDialog(QString _key, QString _icon) 
	: QDialog(0, 0, true) {
	QGridLayout *l = new QGridLayout(this, 2, 2);
	QLabel *label;
	label = new QLabel("Key", this);
	l->addWidget(label, 0, 0);
	editKey = new QLineEdit(_key, this);
	l->addWidget(editKey, 0, 1);

	label = new QLabel("Image", this);
	l->addWidget(label, 1, 0);
	editIcon = new QLineEdit(_icon, this);
	l->addWidget(editIcon, 1, 1);
  }
  QString key() { return editKey->text();}
  QString icon() { return editIcon->text();}
private:
  QLineEdit *editKey, *editIcon;
};

void QualendarSettings::slotEditIcon() {
  QListViewItem* i = ListViewIcon->selectedItem();
  if (!i)
	return;

  EditIconDialog dlg(i->text(0), i->text(1));
  if (dlg.exec() == QDialog::Accepted) {
	icons.remove(i->text(0));
	icons.insert(dlg.key(), dlg.icon());
	i->setText(0, dlg.key());
	i->setText(1, dlg.icon());
  }
}


void QualendarSettings::slotNewBackground() {
  QListViewItem *lvi = new QListViewItem(ListViewBackground);
  lvi->setText(0, "key");
  lvi->setText(1, "background");
  ListViewBackground->insertItem(lvi);
}
void QualendarSettings::slotDeleteBackground() {
  QListViewItem* i = ListViewBackground->selectedItem();
  if (!i)
	return;

  if (!QPEMessageBox::confirmDelete(this, "Dtbk2Util", 
				    i->text(0) + "/" + i->text(1)))
    return;
  backgrounds.remove(i->text(0));
  ListViewBackground->takeItem(i);
}

void QualendarSettings::slotEditBackground() {
  QListViewItem* i = ListViewBackground->selectedItem();
  if (!i)
	return;

  EditIconDialog dlg(i->text(0), i->text(1));
  if (dlg.exec() == QDialog::Accepted) {
	backgrounds.remove(i->text(0));
	backgrounds.insert(dlg.key(), dlg.icon());
	i->setText(0, dlg.key());
	i->setText(1, dlg.icon());
  }
}

void QualendarSettings::slotViewsUp() {
  int i = ListBoxViews->currentItem();
  if (i < 0 || i == 0)
    return;
  QListBoxItem *item = ListBoxViews->item(i);
  ListBoxViews->takeItem(item);
  ListBoxViews->insertItem(item, i-1);
  ListBoxViews->setCurrentItem(item);
}
void QualendarSettings::slotViewsDown() {
  int i = ListBoxViews->currentItem();
  if (i < 0 || i == ListBoxViews->count() - 1)
    return;
  QListBoxItem *item = ListBoxViews->item(i);
  ListBoxViews->takeItem(item);
  ListBoxViews->insertItem(item, 
			   (i == ListBoxViews->count() - 1) ? - 1 : i + 1);
  ListBoxViews->setCurrentItem(item);
}
void QualendarSettings::slotTodoUp() {
  int i = ListBoxTodoColumns->currentItem();
  if (i < 0 || i == 0)
    return;
  QListBoxItem *item = ListBoxTodoColumns->item(i);
  ListBoxTodoColumns->takeItem(item);
  ListBoxTodoColumns->insertItem(item, i-1);
  ListBoxTodoColumns->setCurrentItem(item);
}
void QualendarSettings::slotTodoDown() {
  int i = ListBoxTodoColumns->currentItem();
  if (i < 0 || i == ListBoxTodoColumns->count() - 1)
    return;
  QListBoxItem *item = ListBoxTodoColumns->item(i);
  ListBoxTodoColumns->takeItem(item);
  ListBoxTodoColumns->insertItem(item, 
				 (i == ListBoxTodoColumns->count() - 1) ? - 1 : i + 1);
  ListBoxTodoColumns->setCurrentItem(item);
}

EditColorDialog::EditColorDialog(QString _name, QColor _color) 
  : QDialog(0, 0, true) {
  QVBoxLayout *l0 = new QVBoxLayout(this);
  QGridLayout *l = new QGridLayout(l0, 2, 3);
  QLabel *label;

  setCaption(_name);

  label = new QLabel("Red", this);
  l->addWidget(label, 0, 0);
  spinRed = new QSpinBox(0, 255, 1, this);
  l->addWidget(spinRed, 0, 1);
  label = new QLabel("Green", this);
  l->addWidget(label, 1, 0);
  spinGreen = new QSpinBox(0, 255, 1, this);
  l->addWidget(spinGreen, 1, 1);
  label = new QLabel("Blue", this);
  l->addWidget(label, 2, 0);
  spinBlue = new QSpinBox(0, 255, 1, this);
  l->addWidget(spinBlue, 2, 1);

  labelColor = new QLabel(this); 
  labelColor->setFixedSize(64, 64);
  l0->addWidget(labelColor);

  connect(spinRed, SIGNAL(valueChanged(int)), 
	  this, SLOT(slotValueChange()));
  connect(spinGreen, SIGNAL(valueChanged(int)), 
	  this, SLOT(slotValueChange()));
  connect(spinBlue, SIGNAL(valueChanged(int)), 
	  this, SLOT(slotValueChange()));

  spinRed->setValue(_color.red());
  spinGreen->setValue(_color.green());
  spinBlue->setValue(_color.blue());
}

QColor EditColorDialog::color() {
  return QColor(spinRed->value(), spinGreen->value(), spinBlue->value()); 
}
void EditColorDialog::slotValueChange() {
  QPixmap pm(64, 64);
  pm.fill(color());
  labelColor->setPixmap(pm);
}

