/**********************************************************************
** Copyright (C) 2000-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qtopia Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#ifndef QUALENDARSETTINGS_H
#define QUALENDARSETTINGS_H
#include "qualendarsettingsbase.h"
#include <qcheckbox.h>
#include <qlabel.h>
#include <qdict.h>
#include <qcolor.h>
#include "sysdep/qualendarcategory.h"
#include <qlistbox.h>

class QComboBox;
class QLabel;
class QSpinBox;

#define N_COLORS 10

#define N_VIEWS 5
enum {Day, Week, Month, Find, Todo};

class QualendarSettings : public QualendarSettingsBase
{
  Q_OBJECT
public:
    QualendarSettings( QWidget *parent = 0,
                      const char *name = 0, bool modal = TRUE, WFlags = 0 );
    ~QualendarSettings();
    void save();
    void setStartTime( int newStartViewTime );
    int startTime() const;
    void setAlarmPreset( bool bAlarm, int presetTime );
    bool alarmPreset() const;
    int presetTime() const;
    void setAlarmType( int alarmType );
    int alarmType() const;
    void setFontSize(QComboBox*, int);
    int fontSize(QComboBox*) const;
    int dayRowHeight() const;
    int minimalEventHeight() const;

    void setShowOtherMonths(bool b) {CheckBoxShowOtherMonths->setChecked(b);};
    bool showOtherMonths() {return CheckBoxShowOtherMonths->isChecked();};

    void setSpaceReturn(bool b) {CheckBoxSpaceReturn->setChecked(b);};
    bool spaceReturn() {return CheckBoxSpaceReturn->isChecked();};
    void setCancelToClose(bool b) {CheckBoxCancelToClose->setChecked(b);};
    bool cancelToClose() {return CheckBoxCancelToClose->isChecked();};
    void setJumpToItem(bool b) {CheckBoxJumpToItem->setChecked(b);};
    bool jumpToItem() {return CheckBoxJumpToItem->isChecked();};
    void setReturnDayBox(bool b) {CheckBoxReturnDayBox->setChecked(b);};
    bool returnDayBox() {return CheckBoxReturnDayBox->isChecked();};
    void setDayViewToday(bool b) {CheckBoxDayViewToday->setChecked(b);};
    bool dayViewToday() {return CheckBoxDayViewToday->isChecked();};

    void setTodoStart(bool b) {CheckBoxStart->setChecked(b);};
    bool todoStart() {return CheckBoxStart->isChecked();};
    void setTodoDue(bool b) {CheckBoxDue->setChecked(b);};
    bool todoDue() {return CheckBoxDue->isChecked();};

    void setTodoMonthStart(bool b) {CheckBoxMonthStart->setChecked(b);};
    bool todoMonthStart() {return CheckBoxMonthStart->isChecked();};
    void setTodoMonthDue(bool b) {CheckBoxMonthDue->setChecked(b);};
    bool todoMonthDue() {return CheckBoxMonthDue->isChecked();};

    void setTodoWeekStart(bool b) {CheckBoxWeekStart->setChecked(b);};
    bool todoWeekStart() {return CheckBoxWeekStart->isChecked();};
    void setTodoWeekDue(bool b) {CheckBoxWeekDue->setChecked(b);};
    bool todoWeekDue() {return CheckBoxWeekDue->isChecked();};

    void setTodoBetween(bool b) {CheckBoxBetween->setChecked(b);};
    bool todoBetween() {return CheckBoxBetween->isChecked();};
    void setTodoBetweenStart(bool b) {CheckBoxBetweenStart->setChecked(b);};
    bool todoBetweenStart() {return CheckBoxBetweenStart->isChecked();};
    void setTodoBetweenDue(bool b) {CheckBoxBetweenDue->setChecked(b);};
    bool todoBetweenDue() {return CheckBoxBetweenDue->isChecked();};
    void setTodoToday(bool b) {CheckBoxToday->setChecked(b);};
    bool todoToday() {return CheckBoxToday->isChecked();};
    void setTodoTodayAfterStart(bool b) {CheckBoxTodayAfterStart->setChecked(b);};
    bool todoTodayAfterStart() {return CheckBoxTodayAfterStart->isChecked();};
    void setTodoWeekShowStart(bool b) {CheckBoxWeekShowStart->setChecked(b);};
    bool todoWeekShowStart() {return CheckBoxWeekShowStart->isChecked();};
    void setTodoWeekShowDue(bool b) {CheckBoxWeekShowDue->setChecked(b);};
    bool todoWeekShowDue() {return CheckBoxWeekShowDue->isChecked();};
    void setTodoDayShowStart(bool b) {CheckBoxDayShowStart->setChecked(b);};
    bool todoDayShowStart() {return CheckBoxDayShowStart->isChecked();};
    void setTodoDayShowDue(bool b) {CheckBoxDayShowDue->setChecked(b);};
    bool todoDayShowDue() {return CheckBoxDayShowDue->isChecked();};

private slots:
    void slot12Hour( int );
    void slotChangeClock( bool );
    void slotViewerSelected(int);
    void slotViewerEntered(const QString &);

	void slotEditColor();

	void slotFilterNumberChange(int);
	void slotNewCategory();
	void slotDeleteCategory();
	void slotEditCategory();

	void slotNewIcon();
	void slotDeleteIcon();
	void slotEditIcon();
	void slotNewBackground();
	void slotDeleteBackground();
	void slotEditBackground();

	void slotViewsUp();
	void slotViewsDown();
	void slotTodoUp();
	void slotTodoDown();

private:
    void init();
    void setColor(QColor);
//    SlColorSelector *colorSelector;
    QLabel *colorLabel;
    bool ampm;
    int oldtime;
    QColor barColors[N_COLORS];
    QStringList categories;
    QStringList viewers;
    int viewerLast;

	QString filterName[10];
	uint currentFilter;

	typedef QMap<QString, QString> IconFileNames;
	typedef QMap<QString, QString> BackgroundFileNames;
	typedef QMap<QString,QColor> Colors;

	Colors colors;
	QMap<QString,QString> filter[N_VIEWS]; 
	IconFileNames icons;
	BackgroundFileNames backgrounds;
};
class EditColorDialog : public QDialog {
  Q_OBJECT
public:
  EditColorDialog(QString _name, QColor _color) ;
  QColor color();
 private slots:
   void slotValueChange();

private:
  QSpinBox *spinRed, *spinGreen, *spinBlue;
  QLabel *labelColor;
};

class EditCategoryDialog : public QDialog {
  Q_OBJECT
 public:
  EditCategoryDialog(QString _name, QColor _color, 
		     QString _day, QString _week, QString _month, 
		     QString _find, QString _todo); 
  QColor color();
  QString day();
  QString week();
  QString month();
  QString find();
  QString todo();
 private slots:
  void slotColor();

 private:
  QString filter(int n);
  QCheckBox *cb[N_VIEWS][10];
  QPushButton *btnColor;
  QString name_;
  QColor color_;
};

#endif
