#include "sysdep/config.h"
#include "qualendarutilbase.h"
#include "qualendarutil.h"
#include "qualendarsettings.h"
#include <qpushbutton.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qlayout.h>
#include <qregexp.h>
#include <qpe/qcopenvelope_qws.h>
#include "version.h"
#include <stdlib.h>
#include <qdatetime.h>
#include <qlistbox.h>
#include <qmessagebox.h>

QualendarUtil::QualendarUtil() : QualendarUtilBase() {
  setCaption(QString("qualendar util: ") + VERSION);

  connect(PushButtonSettings, SIGNAL(clicked()), this, SLOT(slotSettings()));
  connect(PushButtonBackup, SIGNAL(clicked()), this, SLOT(slotBackup()));
  connect(PushButtonRecover, SIGNAL(clicked()), this, SLOT(slotRecover()));
}


void QualendarUtil::slotSettings()
{
    QualendarSettings frmSettings( this );

    frmSettings.showMaximized();

    if ( frmSettings.exec() ) {
	frmSettings.save();
	if (!app.isEmpty()) 
	  QCopEnvelope e(("QPE/Application/" + app).ascii(), 
			 "reloadSettings()");
    }
}

void QualendarUtil::setDocument(const QString &str) {
  app = str;
}

void QualendarUtil::slotBackup() {
  BackupDialog dlg;

  dlg.showMaximized();
  if ( dlg.exec() == QDialog::Accepted) {
    QString srcFile, dstDir;

    if (dlg.box()) 
      srcFile = QString(getenv("HOME")) + "/Applications/dtm/SLDATE* " + QString(getenv("HOME")) + "/Applications/dtm/SLTODO* ";
    else
      srcFile = QString(getenv("HOME")) + "/Applications/datebook/*";

    if (dlg.sd())
      dstDir = "/mnt/card/qualendar_";
    else
      dstDir = QString(getenv("HOME")) + "/Documents/qualendar_";
    QDateTime dt = QDateTime::currentDateTime();
    QDate d = dt.date();
    QTime t = dt.time();
    dstDir += QString("%1%2%3").arg(d.year(), 4).arg(d.month(), 2).arg(d.day(), 2).replace(QRegExp(" "), "0");
    dstDir += QString("%1%2%3").arg(t.hour(), 2).arg(t.minute(), 2).arg(t.second(), 2).replace(QRegExp(" "), "0");
    system(QString("/bin/mkdir %1").arg(dstDir).latin1());
    system(QString("/bin/cp %1 %2").arg(srcFile).arg(dstDir).latin1());
  }
}

void QualendarUtil::slotRecover() {
  RecoverDialog dlg;

  dlg.showMaximized();

  if (dlg.exec() == QDialog::Accepted) {
    QString srcDir = dlg.srcDir(), dstDir;

    if (!srcDir.isEmpty()) {
      if (QMessageBox::warning(this, "Recover", 
			       tr("Are you sure you want to\n"
				  "recover from\n"
				  "%1?\n\n"
				  "(If so, stop all PIM applications.)"
				  )
			       .arg(srcDir),
			       QMessageBox::Cancel, QMessageBox::Ok)
	  == QMessageBox::Cancel)
	return;

      if (dlg.box()) {
	dstDir = QString(getenv("HOME")) + "/Applications/dtm/";
	QDir d; 
	d.cd(srcDir);
	QStringList strList = d.entryList(QDir::Files);
	for(int i=0; i<strList.count(); i++) 
	  system(QString("/bin/cp %1/%2 %3/%4")
		 .arg(srcDir)
		 .arg(strList[i])
		 .arg(dstDir)
		 .arg(strList[i].upper())
		 .latin1());
      } else {
	dstDir = QString(getenv("HOME")) + "/Applications/datebook/";
	system(QString("/bin/cp %1/%2 %3")
	       .arg(srcDir)
	       .arg("datebook.xml")
	       .arg(dstDir)
	       .latin1());
      }
    }
  }
}


BackupDialog::BackupDialog() : QDialog(0, 0, true) {
  QVBoxLayout *l = new QVBoxLayout(this);
  setCaption(tr("Backup data"));
  label = new QLabel(this);
  label->setAlignment(Qt::AlignCenter);
  l->addWidget(label);
  cbBOX = new QCheckBox(tr("BOX version"), this);
  cbBOX->setChecked(true);
  l->addWidget(cbBOX);
  cbSD = new QCheckBox(tr("To SD"), this);
  l->addWidget(cbSD);

  slotSD();

  connect(cbSD, SIGNAL(toggled(bool)), this, SLOT(slotSD()));
}

bool BackupDialog::box() {
  return cbBOX->isChecked();
} 
int BackupDialog::sd() {
  return cbSD->isChecked();
}
void BackupDialog::slotSD() {
  QString s;
  if (sd()) 
    s = tr("qualendar_?????????????? in SD card(/mnt/card/)");
  else
    s = tr("qualendar_?????????????? in Main memory(%1/Documents/)")
      .arg(QString(getenv("HOME")));
  label->setText(tr("Copy data for datebook and todolist to %1\n")
		 .arg(s));
}

RecoverDialog::RecoverDialog() : QDialog(0, 0, true) {
  setCaption(tr("Recover data"));
  QVBoxLayout *l = new QVBoxLayout(this);
  cbBOX = new QCheckBox(tr("BOX version"), this);
  l->addWidget(cbBOX);
  cbSD = new QCheckBox(tr("From SD"), this);
  l->addWidget(cbSD);
  connect(cbBOX, SIGNAL(toggled(bool)), this, SLOT(slotRefresh()));
  connect(cbSD, SIGNAL(toggled(bool)), this, SLOT(slotRefresh()));

  lb = new QListBox(this);
  l->addWidget(lb);

  slotRefresh();
}

QString RecoverDialog::srcDir() {
  int i = lb->currentItem();
  if (i >= 0) {
    return dir.absPath() + "/" + lb->currentText();
  } else {
    return QString();
  }
}

bool RecoverDialog::box() {
  return cbBOX->isChecked();
}

void RecoverDialog::slotRefresh() {
  lb->clear();
  if (cbSD->isChecked()) {
    if (!dir.cd("/mnt/card/"))
      return;
  } else {
    if (!dir.cd(QString(getenv("HOME")) + "/Documents/"))
      return;
  }

  dir.setNameFilter("qualendar_??????????????");
  QStringList strList = dir.entryList(QDir::Dirs);
  for(int i=0; i<strList.count(); i++) {
    QDir d;
    if (!d.cd(dir.absPath() + "/" + strList[i]))
      continue;
    bool b = false;
    if (cbBOX->isChecked()) {
      QStringList strList = d.entryList(QDir::Files);
      for(int i=0; i<strList.count(); i++) 
	if (strList[i].upper() == "SLDATE.BOX")
	  b = true;
    } else {
      b = d.exists("datebook.xml");
    }
    if (b)
      lb->insertItem(strList[i]);
  }
}

